@@ -17,7 +17,7 @@
 */
 
 #ifndef DT_UNIT_TEST
-  #include "common/darktable.h"
+#include "common/darktable.h"
 #endif
 #include "common/cache.h"
 
@@ -124,16 +124,34 @@ get_start_cacheline_bucket(const dt_cache_t *const cache, dt_cache_bucket_t *con
 }
 
 static void
-remove_key(dt_cache_segment_t *segment,
+add_cost(dt_cache_t    *cache,
+         const int32_t  cost)
+{
+  __sync_fetch_and_add(&cache->cost, cost);
+}
+
+static void
+remove_key(dt_cache_t *cache,
+           dt_cache_segment_t *segment,
            dt_cache_bucket_t *const from_bucket,
            dt_cache_bucket_t *const key_bucket,
            dt_cache_bucket_t *const prev_key_bucket,
            const uint32_t hash)
 {
+  // clean up the user data
+  if(cache->cleanup)
+  {
+    // in case cleanup is requested, assume we should set to NULL again.
+    cache->cleanup(cache->cleanup_data, key_bucket->key, key_bucket->data);
+    key_bucket->data = DT_CACHE_EMPTY_DATA;
+  }
+  // else: crucially don't release the data pointer (not for dynamic nor static allocation a good idea)
+  // key_bucket->data = DT_CACHE_EMPTY_DATA;
   key_bucket->hash = DT_CACHE_EMPTY_HASH;
   key_bucket->key  = DT_CACHE_EMPTY_KEY;
-  // crucially don't release the data pointer (not for dynamic nor static allocation a good idea)
-  // key_bucket->data = DT_CACHE_EMPTY_DATA;
+
+  // keep track of cost
+  add_cost(cache, -key_bucket->cost);
 
   if(prev_key_bucket == NULL)
   {
@@ -153,13 +171,6 @@ remove_key(dt_cache_segment_t *segment,
   key_bucket->next_delta = DT_CACHE_NULL_DELTA;
 }
 
-static void 
-add_cost(dt_cache_t    *cache,
-         const int32_t  cost)
-{
-  __sync_fetch_and_add(&cache->cost, cost);
-}
-
 // unexposed helpers to increase the read lock count.
 // the segment needs to be locked by the caller.
 static int
@@ -211,11 +222,11 @@ dt_cache_bucket_write_release(dt_cache_bucket_t *bucket)
 
 static void
 add_key_to_beginning_of_list(
-    dt_cache_t        *cache,
-    dt_cache_bucket_t *const keys_bucket,
-    dt_cache_bucket_t *const free_bucket,
-    const uint32_t     hash,
-    const uint32_t     key)
+  dt_cache_t        *cache,
+  dt_cache_bucket_t *const keys_bucket,
+  dt_cache_bucket_t *const free_bucket,
+  const uint32_t     hash,
+  const uint32_t     key)
 {
   int32_t cost = 1;
   if(cache->allocate)
@@ -250,12 +261,12 @@ add_key_to_beginning_of_list(
 
 static void
 add_key_to_end_of_list(
-    dt_cache_t        *cache,
-    dt_cache_bucket_t *const keys_bucket,
-    dt_cache_bucket_t *const free_bucket,
-    const uint32_t     hash,
-    const uint32_t     key,
-    dt_cache_bucket_t *const last_bucket)
+  dt_cache_t        *cache,
+  dt_cache_bucket_t *const keys_bucket,
+  dt_cache_bucket_t *const free_bucket,
+  const uint32_t     hash,
+  const uint32_t     key,
+  dt_cache_bucket_t *const last_bucket)
 {
   int32_t cost = 1;
   if(cache->allocate)
@@ -341,7 +352,7 @@ dt_cache_init(dt_cache_t *cache, const int32_t capacity, const int32_t num_threa
   cache->optimize_cacheline = 0;//1;
   // No cache_mask offsetting required when not optimizing for cachelines --RAM
   cache->cache_mask = cache->optimize_cacheline ?
-    cache_line_size / sizeof(dt_cache_bucket_t) - 1 : 0;
+                      cache_line_size / sizeof(dt_cache_bucket_t) - 1 : 0;
   cache->segment_mask = adj_num_threads - 1;
   // cache->segment_shift = calc_div_shift(nearest_power_of_two(num_threads/(float)adj_num_threads)-1);
   // we want a minimum of four entries, as the hopscotch code below proceeds by disregarding the first bucket in the list,
@@ -368,12 +379,12 @@ dt_cache_init(dt_cache_t *cache, const int32_t capacity, const int32_t num_threa
   cache->cleanup = NULL;
   cache->cleanup_data = NULL;
 
-  for(int k=0;k<=cache->segment_mask;k++)
+  for(int k=0; k<=cache->segment_mask; k++)
   {
     cache->segments[k].timestamp = 0;
     cache->segments[k].lock = 0;
   }
-  for(int k=0;k<num_buckets;k++)
+  for(int k=0; k<num_buckets; k++)
   {
     cache->table[k].first_delta = DT_CACHE_NULL_DELTA;
     cache->table[k].next_delta  = DT_CACHE_NULL_DELTA;
@@ -405,12 +416,12 @@ dt_cache_cleanup(dt_cache_t *cache)
 
 void
 dt_cache_static_allocation(
-    dt_cache_t *cache,
-    uint8_t *buf,
-    const uint32_t stride)
+  dt_cache_t *cache,
+  uint8_t *buf,
+  const uint32_t stride)
 {
   const int num_buckets = cache->bucket_mask + 1;
-  for(int k=0;k<num_buckets;k++)
+  for(int k=0; k<num_buckets; k++)
   {
     cache->table[k].data = (void *)(buf + k*stride);
   }
@@ -448,7 +459,7 @@ dt_cache_size(const dt_cache_t *const cache)
 {
   uint32_t cnt = 0;
   const uint32_t num = cache->bucket_mask + 1;
-  for(int k=0;k<num;k++)
+  for(int k=0; k<num; k++)
   {
     if(cache->table[k].hash != DT_CACHE_EMPTY_HASH) cnt++;
   }
@@ -456,12 +467,12 @@ dt_cache_size(const dt_cache_t *const cache)
 }
 
 #if 0 // not sure we need this diagnostic tool:
-  float
+float
 dt_cache_percent_keys_in_cache_line(const dt_cache_t *const cache)
 {
   uint32_t total_in_cache = 0;
   uint32_t total = 0;
-  for(int k=0;k<=cache->bucket_mask;k++)
+  for(int k=0; k<=cache->bucket_mask; k++)
   {
     const dt_cache_bucket_t *curr_bucket = cache->table + k;
     if(curr_bucket->first_delta != DT_CACHE_NULL_DELTA)
@@ -547,9 +558,9 @@ lru_insert_locked(dt_cache_t        *cache,
 
 int
 dt_cache_for_all(
-    dt_cache_t *cache,
-    int (*process)(const uint32_t key, const void *data, void *user_data),
-    void *user_data)
+  dt_cache_t *cache,
+  int (*process)(const uint32_t key, const void *data, void *user_data),
+  void *user_data)
 {
   // this is not thread safe.
   //dt_cache_lock(&cache->lru_lock);
@@ -611,8 +622,8 @@ lru_check_consistency_reverse(dt_cache_t *cache)
 // never attempt to allocate a new slot.
 void*
 dt_cache_read_testget(
-    dt_cache_t     *cache,
-    const uint32_t  key)
+  dt_cache_t     *cache,
+  const uint32_t  key)
 {
   // just to support different keys:
   const uint32_t hash = key;
@@ -649,8 +660,8 @@ dt_cache_read_testget(
 //
 void*
 dt_cache_read_get(
-    dt_cache_t     *cache,
-    const uint32_t  key)
+  dt_cache_t     *cache,
+  const uint32_t  key)
 {
   assert(key != DT_CACHE_EMPTY_KEY);
 
@@ -664,12 +675,9 @@ dt_cache_read_get(
   dt_cache_bucket_t *last_bucket = NULL;
   dt_cache_bucket_t *compare_bucket = start_bucket;
 
+retry_cache_full:
   while(1)
   {
-    // we might be allocing, so first try to clean up.
-    // also wait if we can't free more than the requested fill ratio.
-    if(dt_cache_gc(cache, 0.8f)) goto wait;
-
     // block and try our luck
     dt_cache_lock(&segment->lock);
 
@@ -696,11 +704,22 @@ dt_cache_read_get(
     }
     // end of the loop, didn't find it. need to alloc (and keep segment locked)
     break;
-wait:;
+wait:
+    ;
     // try again in 5 milliseconds
     dt_cache_sleep_ms(5);
   }
 
+  // we will be allocing, so first try to clean up.
+  // also wait if we can't free more than the requested fill ratio.
+  if(cache->cost > 0.8f * cache->cost_quota)
+  {
+    dt_cache_unlock(&segment->lock);
+    // need to roll back all the way to get a consistent lock state:
+    dt_cache_gc(cache, 0.8f);
+    goto retry_cache_full;
+  }
+
   if(cache->optimize_cacheline)
   {
     dt_cache_bucket_t *free_bucket = start_bucket;
@@ -736,13 +755,23 @@ wait:;
   {
     if(free_max_bucket->hash == DT_CACHE_EMPTY_HASH)
     {
-      dt_cache_bucket_read_lock(free_max_bucket);
-      add_key_to_end_of_list(cache, start_bucket, free_max_bucket, hash, key, last_bucket);
-      void *data = free_max_bucket->data;
-      dt_cache_unlock(&segment->lock);
-      lru_insert_locked(cache, free_max_bucket);
-      return data;
+      dt_cache_lock(&cache->lru_lock);
+      if(free_max_bucket->hash == DT_CACHE_EMPTY_HASH)
+      {
+        // try that again if it's still empty
+        dt_cache_bucket_read_lock(free_max_bucket);
+        add_key_to_end_of_list(cache, start_bucket, free_max_bucket, hash, key, last_bucket);
+        void *data = free_max_bucket->data;
+        dt_cache_unlock(&segment->lock);
+        lru_insert(cache, free_max_bucket);
+        dt_cache_unlock(&cache->lru_lock);
+        return data;
+      }
+      dt_cache_unlock(&cache->lru_lock);
     }
+    // this could walk outside the range where the segment lock is valid.
+    // that's why we abuse the lru lock above to shield grabbing a new bucket
+    // at this stage.
     ++free_max_bucket;
   }
 
@@ -755,12 +784,18 @@ wait:;
   {
     if(free_min_bucket->hash == DT_CACHE_EMPTY_HASH)
     {
-      dt_cache_bucket_read_lock(free_min_bucket);
-      add_key_to_end_of_list(cache, start_bucket, free_min_bucket, hash, key, last_bucket);
-      void *data = free_min_bucket->data;
-      dt_cache_unlock(&segment->lock);
-      lru_insert_locked(cache, free_min_bucket);
-      return data;
+      dt_cache_lock(&cache->lru_lock);
+      if(free_min_bucket->hash == DT_CACHE_EMPTY_HASH)
+      {
+        dt_cache_bucket_read_lock(free_min_bucket);
+        add_key_to_end_of_list(cache, start_bucket, free_min_bucket, hash, key, last_bucket);
+        void *data = free_min_bucket->data;
+        dt_cache_unlock(&segment->lock);
+        lru_insert(cache, free_min_bucket);
+        dt_cache_unlock(&cache->lru_lock);
+        return data;
+      }
+      dt_cache_unlock(&cache->lru_lock);
     }
     --free_min_bucket;
   }
@@ -817,19 +852,12 @@ dt_cache_remove(dt_cache_t *cache, const uint32_t key)
         dt_cache_unlock(&segment->lock);
         return 1;
       }
-      void *rc = curr_bucket->data;
-      const int32_t cost = curr_bucket->cost;
-      remove_key(segment, start_bucket, curr_bucket, last_bucket, hash);
+      remove_key(cache, segment, start_bucket, curr_bucket, last_bucket, hash);
       if(cache->optimize_cacheline)
         optimize_cacheline_use(cache, segment, curr_bucket);
       // put back into unused part of the cache: remove from lru list.
       dt_cache_unlock(&segment->lock);
       lru_remove_locked(cache, curr_bucket);
-      // clean up the user data
-      if(cache->cleanup)
-        cache->cleanup(cache->cleanup_data, key, rc);
-      // keep track of cost
-      add_cost(cache, -cost);
       // fprintf(stderr, "[cache remove] freeing %d for %u\n", cost, key);
       return 0;
     }
@@ -873,7 +901,7 @@ dt_cache_remove_no_lru_lock(dt_cache_t *cache, const uint32_t key)
       }
       void *rc = curr_bucket->data;
       const int32_t cost = curr_bucket->cost;
-      remove_key(segment, start_bucket, curr_bucket, last_bucket, hash);
+      remove_key(cache, segment, start_bucket, curr_bucket, last_bucket, hash);
       if(cache->optimize_cacheline)
         optimize_cacheline_use(cache, segment, curr_bucket);
       // put back into unused part of the cache: remove from lru list.
@@ -1049,7 +1077,8 @@ dt_cache_write_get(dt_cache_t *cache, const uint32_t key)
     }
     // didn't find any entry :(
     break;
-wait:;
+wait:
+    ;
     // try again in 5 milliseconds
     dt_cache_sleep_ms(5);
   }
@@ -1093,6 +1122,7 @@ dt_cache_realloc(dt_cache_t *cache, const uint32_t key, const int32_t cost, void
   dt_cache_unlock(&segment->lock);
   // clear user error, he should hold a write lock already, so this has to be there.
   fprintf(stderr, "[cache] realloc: bucket for key %u not found!\n", key);
+  assert(0);
   return;
 }
 
@@ -1126,14 +1156,14 @@ dt_cache_write_release(dt_cache_t *cache, const uint32_t key)
 void dt_cache_print(dt_cache_t *cache)
 {
   fprintf(stderr, "[cache] full entries:\n");
-  for(int k=0;k<=cache->bucket_mask;k++)
+  for(int k=0; k<=cache->bucket_mask; k++)
   {
     if(cache->table[k].key != DT_CACHE_EMPTY_KEY)
       fprintf(stderr, "[cache] bucket %d holds key %u with locks r %d w %d\n",
-          k, (cache->table[k].key & 0x1fffffff)+1, cache->table[k].read, cache->table[k].write);
+              k, (cache->table[k].key & 0x1fffffff)+1, cache->table[k].read, cache->table[k].write);
     else
       fprintf(stderr, "[cache] bucket %d is empty with locks r %d w %d\n",
-          k, cache->table[k].read, cache->table[k].write);
+              k, cache->table[k].read, cache->table[k].write);
   }
   fprintf(stderr, "[cache] lru entries:\n");
   dt_cache_lock(&cache->lru_lock);
@@ -1142,11 +1172,11 @@ void dt_cache_print(dt_cache_t *cache)
   {
     if(cache->table[curr].key != DT_CACHE_EMPTY_KEY)
       fprintf(stderr, "[cache] bucket %d holds key %u with locks r %d w %d\n",
-          curr, (cache->table[curr].key & 0x1fffffff)+1, cache->table[curr].read, cache->table[curr].write);
+              curr, (cache->table[curr].key & 0x1fffffff)+1, cache->table[curr].read, cache->table[curr].write);
     else
     {
       fprintf(stderr, "[cache] bucket %d is empty with locks r %d w %d\n",
-          curr, cache->table[curr].read, cache->table[curr].write);
+              curr, cache->table[curr].read, cache->table[curr].write);
       // this list should only ever contain valid buffers.
       assert(0);
     }
@@ -1169,7 +1199,7 @@ void dt_cache_print_locked(dt_cache_t *cache)
     if(cache->table[curr].key != DT_CACHE_EMPTY_KEY && (cache->table[curr].read || cache->table[curr].write))
     {
       fprintf(stderr, "[cache] bucket[%d|%d] holds key %u with locks r %d w %d\n",
-          i, curr, (cache->table[curr].key & 0x1fffffff)+1, cache->table[curr].read, cache->table[curr].write);
+              i, curr, (cache->table[curr].key & 0x1fffffff)+1, cache->table[curr].read, cache->table[curr].write);
     }
     if(curr == cache->mru) break;
     int32_t next = cache->table[curr].mru;
@@ -1180,3 +1210,6 @@ void dt_cache_print_locked(dt_cache_t *cache)
   dt_cache_unlock(&cache->lru_lock);
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
