@@ -1,7 +1,7 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2011 johannes hanika.
-    copyright (c) 2010--2011 henrik andersson.
+    copyright (c) 2009--2012 johannes hanika.
+    copyright (c) 2010--2012 henrik andersson.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,10 +40,12 @@
 #include "libs/lib.h"
 #include "views/view.h"
 #include "control/control.h"
+#include "control/jobs/control_jobs.h"
 #include "control/signal.h"
 #include "control/conf.h"
 #include "gui/gtk.h"
 #include "gui/presets.h"
+#include "bauhaus/bauhaus.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -54,7 +56,7 @@
 #include <unistd.h>
 #include <locale.h>
 
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__DragonFly__)
 #include <malloc.h>
 #endif
 #ifdef __APPLE__
@@ -73,20 +75,28 @@ const char dt_supported_extensions[] = "3fr,arw,bay,bmq,cap,cine,cr2,crw,cs1,dc2
 
 static int usage(const char *argv0)
 {
-  printf("usage: %s [-d {all,cache,camctl,control,dev,fswatch,memory,opencl,perf,pwstorage,sql}] [IMG_1234.{RAW,..}|image_folder/]", argv0);
+  printf("usage: %s [-d {all,cache,camctl,control,dev,fswatch,lighttable,memory,opencl,perf,pwstorage,sql}] [IMG_1234.{RAW,..}|image_folder/]", argv0);
 #ifdef HAVE_OPENCL
   printf(" [--disable-opencl]");
 #endif
+  printf(" [--library <library file>]");
+  printf(" [--datadir <data directory>]");
+  printf(" [--moduledir <module directory>]");
+  printf(" [--tmpdir <tmp directory>]");
+  printf(" [--configdir <user config directory>]");
+  printf(" [--cachedir <user config directory>]");
+  printf(" [--localedir <locale directory>]");
   printf("\n");
   return 1;
 }
 
+#ifndef __APPLE__
 typedef void (dt_signal_handler_t)(int) ;
 // static dt_signal_handler_t *_dt_sigill_old_handler = NULL;
 static dt_signal_handler_t *_dt_sigsegv_old_handler = NULL;
+#endif
 
-#if (defined(__APPLE__) && defined(APPLE_NEED_DPRINTF)) ||        \
-  (defined(__FreeBSD_version) && (__FreeBSD_version < 800071)) || \
+#if (defined(__FreeBSD_version) && (__FreeBSD_version < 800071)) || \
   defined(__SUNOS__)
 static int dprintf(int fd,const char *fmt, ...)
 {
@@ -100,6 +110,7 @@ static int dprintf(int fd,const char *fmt, ...)
 }
 #endif
 
+#ifndef __APPLE__
 static
 void _dt_sigsegv_handler(int param)
 {
@@ -108,12 +119,14 @@ void _dt_sigsegv_handler(int param)
   gchar *name_used;
   int fout;
   gboolean delete_file = FALSE;
+  char datadir[DT_MAX_PATH_LEN];
 
   if((fout = g_file_open_tmp("darktable_bt_XXXXXX.txt", &name_used, NULL)) == -1)
     fout = STDOUT_FILENO; // just print everything to stdout
 
   dprintf(fout, "this is %s reporting a segfault:\n\n", PACKAGE_STRING);
-  gchar *command = g_strdup_printf("gdb %s %d -batch -x %s/gdb_commands", darktable.progname, (int)getpid(), DARKTABLE_DATADIR);
+  dt_loc_get_datadir(datadir, DT_MAX_PATH_LEN);
+  gchar *command = g_strdup_printf("gdb %s %d -batch -x %s/gdb_commands", darktable.progname, (int)getpid(), datadir);
 
   if((fd = popen(command, "r")) != NULL)
   {
@@ -151,6 +164,7 @@ void _dt_sigsegv_handler(int param)
   /* pass it further to the old handler*/
   _dt_sigsegv_old_handler(param);
 }
+#endif
 
 #if 0
 static
@@ -158,7 +172,7 @@ void _dt_sigill_handler(int param)
 {
   fprintf(stderr, "[this doesn't seem to work]\n");
   GtkWidget *dlg = gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_(
-"darktable has trapped an illegal instruction which probably means that \
+      "darktable has trapped an illegal instruction which probably means that \
 an invalid processor optimized codepath is used for your cpu, please try reproduce the crash running 'gdb darktable' from \
 the console and post the backtrace log to mailing list with information about your CPU and where you got the package from."));
   gtk_dialog_run(GTK_DIALOG(dlg));
@@ -184,8 +198,8 @@ the console and post the backtrace log to mailing list with information about yo
 #endif
 
 
-static 
-void dt_check_cpu(int argc,char **argv) 
+static
+void dt_check_cpu(int argc,char **argv)
 {
   /* hook up SIGILL handler */
   _dt_sigill_old_handler = signal(SIGILL,&_dt_sigill_handler);
@@ -193,46 +207,46 @@ void dt_check_cpu(int argc,char **argv)
   /* call cpuid for  SSE level */
   int ax,bx,cx,dx;
   cpuid(0x1,ax,bx,cx,dx);
-  
+
   ax = bx = 0;
-  char message[512]={0};
+  char message[512]= {0};
   strcat(message,_("SIMD extensions found: "));
   if((cx & 1) && (darktable.cpu_flags |= DT_CPU_FLAG_SSE3))
     strcat(message,"SSE3 ");
   if( ((dx >> 26) & 1) && (darktable.cpu_flags |= DT_CPU_FLAG_SSE2))
     strcat(message,"SSE2 ");
   if (((dx >> 25) & 1) && (darktable.cpu_flags |= DT_CPU_FLAG_SSE))
-   strcat(message,"SSE ");
+    strcat(message,"SSE ");
   if (!darktable.cpu_flags)
     strcat(message,"none");
- 
+
   /* for now, bail out if SSE2 is not availble */
   if(!(darktable.cpu_flags & DT_CPU_FLAG_SSE2))
   {
     gtk_init (&argc, &argv);
-    
+
     GtkWidget *dlg = gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_(
-"darktable is very cpu intensive and uses SSE2 SIMD instructions \
+        "darktable is very cpu intensive and uses SSE2 SIMD instructions \
 for heavy calculations. This gives a better user experience but also defines a minimum \
 processor requirement.\n\nThe processor in YOUR system does NOT support SSE2. \
 darktable will now close down.\n\n%s"),message);
-    
+
     gtk_dialog_run(GTK_DIALOG(dlg));
-    
+
     exit(11);
   }
 }
 #endif
 
-/*  TODO: make this case insensitive */
 gboolean dt_supported_image(const gchar *filename)
 {
   gboolean supported = FALSE;
   char **extensions = g_strsplit(dt_supported_extensions, ",", 100);
   char *ext = g_strrstr(filename,".");
   if(!ext) return FALSE;
+  ext++;
   for(char **i=extensions; *i!=NULL; i++)
-    if(!g_ascii_strncasecmp(ext+1, *i,strlen(*i)))
+    if(!g_ascii_strncasecmp(ext, *i,strlen(*i)))
     {
       supported = TRUE;
       break;
@@ -243,13 +257,13 @@ gboolean dt_supported_image(const gchar *filename)
 
 static void strip_semicolons_from_keymap(const char* path)
 {
-  char pathtmp[1024];
+  char pathtmp[DT_MAX_PATH_LEN];
   FILE *fin = fopen(path, "r");
   FILE *fout;
   int i;
   int c = '\0';
 
-  snprintf(pathtmp, 1024, "%s_tmp", path);
+  snprintf(pathtmp, DT_MAX_PATH_LEN, "%s_tmp", path);
   fout = fopen(pathtmp, "w");
 
   // First ignoring the first three lines
@@ -269,7 +283,8 @@ static void strip_semicolons_from_keymap(const char* path)
       c = fgetc(fin);
       if(c != EOF)
         fputc(c, fout);
-    }while(c != '\n' && c != EOF);
+    }
+    while(c != '\n' && c != EOF);
   }
 
   fclose(fin);
@@ -279,6 +294,94 @@ static void strip_semicolons_from_keymap(const char* path)
               NULL, NULL, NULL, NULL);
 }
 
+int dt_load_from_string(const gchar* input, gboolean open_image_in_dr)
+{
+  int id = 0;
+  if(input == NULL || input[0] == '\0')
+    return 0;
+
+  char* filename;
+  if(g_str_has_prefix(input, "file://")) // in this case we should take care of %XX encodings in the string (for example %20 = ' ')
+  {
+    input += strlen("file://");
+    filename = g_uri_unescape_string(input, NULL);
+  }
+  else
+    filename = g_strdup(input);
+
+  if(g_path_is_absolute(filename) == FALSE)
+  {
+    char* current_dir = g_get_current_dir();
+    char* tmp_filename = g_build_filename(current_dir, filename, NULL);
+    g_free(filename);
+    filename = (char*)g_malloc(sizeof(char)*MAXPATHLEN);
+    if(realpath(tmp_filename, filename) == NULL)
+    {
+      dt_control_log(_("found strange path `%s'"), tmp_filename);
+      g_free(current_dir);
+      g_free(tmp_filename);
+      g_free(filename);
+      return 0;
+    }
+    g_free(current_dir);
+    g_free(tmp_filename);
+  }
+
+  if(g_file_test(filename, G_FILE_TEST_IS_DIR))
+  {
+    // import a directory into a film roll
+    unsigned int last_char = strlen(filename)-1;
+    if(filename[last_char] == '/')
+      filename[last_char] = '\0';
+    id = dt_film_import(filename);
+    if(id)
+    {
+      dt_film_open(id);
+      dt_ctl_switch_mode_to(DT_LIBRARY);
+    }
+    else
+    {
+      dt_control_log(_("error loading directory `%s'"), filename);
+    }
+  }
+  else
+  {
+    // import a single image
+    gchar *directory = g_path_get_dirname((const gchar *)filename);
+    dt_film_t film;
+    const int filmid = dt_film_new(&film, directory);
+    id = dt_image_import(filmid, filename, TRUE);
+    g_free (directory);
+    if(id)
+    {
+      dt_film_open(filmid);
+      // make sure buffers are loaded (load full for testing)
+      dt_mipmap_buffer_t buf;
+      dt_mipmap_cache_read_get(darktable.mipmap_cache, &buf, id, DT_MIPMAP_FULL, DT_MIPMAP_BLOCKING);
+      if(!buf.buf)
+      {
+        id = 0;
+        dt_control_log(_("file `%s' has unknown format!"), filename);
+      }
+      else
+      {
+        dt_mipmap_cache_read_release(darktable.mipmap_cache, &buf);
+        if(open_image_in_dr)
+        {
+          DT_CTL_SET_GLOBAL(lib_image_mouse_over_id, id);
+          dt_ctl_switch_mode_to(DT_DEVELOP);
+        }
+      }
+    }
+    else
+    {
+      dt_control_log(_("error loading file `%s'"), filename);
+    }
+  }
+  g_free(filename);
+  return id;
+}
+
 int dt_init(int argc, char *argv[], const int init_gui)
 {
 #ifndef __APPLE__
@@ -292,28 +395,33 @@ int dt_init(int argc, char *argv[], const int init_gui)
 #endif
 
 #ifdef M_MMAP_THRESHOLD
-  mallopt(M_MMAP_THRESHOLD,128*1024) ; /* use mmap() for large allocations */   
+  mallopt(M_MMAP_THRESHOLD,128*1024) ; /* use mmap() for large allocations */
 #endif
 
   bindtextdomain (GETTEXT_PACKAGE, DARKTABLE_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  
+
   // init all pointers to 0:
   memset(&darktable, 0, sizeof(darktable_t));
 
   darktable.progname = argv[0];
-   
+
   // database
-  gchar *dbfilenameFromCommand = NULL;
+  gchar *dbfilename_from_command = NULL;
+  char *datadirFromCommand = NULL;
+  char *moduledirFromCommand = NULL;
+  char *tmpdirFromCommand = NULL;
+  char *configdirFromCommand = NULL;
+  char *cachedirFromCommand = NULL;
 
   darktable.num_openmp_threads = 1;
 #ifdef _OPENMP
   darktable.num_openmp_threads = omp_get_num_procs();
 #endif
   darktable.unmuted = 0;
-  char *image_to_load = NULL;
+  GSList *images_to_load = NULL;
   for(int k=1; k<argc; k++)
   {
     if(argv[k][0] == '-')
@@ -322,6 +430,10 @@ int dt_init(int argc, char *argv[], const int init_gui)
       {
         return usage(argv[0]);
       }
+      if(!strcmp(argv[k], "-h"))
+      {
+        return usage(argv[0]);
+      }
       else if(!strcmp(argv[k], "--version"))
       {
         printf("this is "PACKAGE_STRING"\ncopyright (c) 2009-2012 johannes hanika\n"PACKAGE_BUGREPORT"\n");
@@ -329,21 +441,46 @@ int dt_init(int argc, char *argv[], const int init_gui)
       }
       else if(!strcmp(argv[k], "--library"))
       {
-        dbfilenameFromCommand = argv[++k];
+        dbfilename_from_command = argv[++k];
+      }
+      else if(!strcmp(argv[k], "--datadir"))
+      {
+        datadirFromCommand = argv[++k];
+      }
+      else if(!strcmp(argv[k], "--moduledir"))
+      {
+        moduledirFromCommand = argv[++k];
+      }
+      else if(!strcmp(argv[k], "--tmpdir"))
+      {
+        tmpdirFromCommand = argv[++k];
+      }
+      else if(!strcmp(argv[k], "--configdir"))
+      {
+        configdirFromCommand = argv[++k];
+      }
+      else if(!strcmp(argv[k], "--cachedir"))
+      {
+        cachedirFromCommand = argv[++k];
+      }
+      else if(!strcmp(argv[k], "--localedir"))
+      {
+        bindtextdomain (GETTEXT_PACKAGE, argv[++k]);
       }
       else if(argv[k][1] == 'd' && argc > k+1)
       {
-        if(!strcmp(argv[k+1], "all"))            darktable.unmuted = 0xffffffff;   // enable all debug information
-        else if(!strcmp(argv[k+1], "cache"))     darktable.unmuted |= DT_DEBUG_CACHE;   // enable debugging for lib/film/cache module
-        else if(!strcmp(argv[k+1], "control"))   darktable.unmuted |= DT_DEBUG_CONTROL; // enable debugging for scheduler module
-        else if(!strcmp(argv[k+1], "dev"))       darktable.unmuted |= DT_DEBUG_DEV; // develop module
-        else if(!strcmp(argv[k+1], "fswatch"))   darktable.unmuted |= DT_DEBUG_FSWATCH; // fswatch module
-        else if(!strcmp(argv[k+1], "camctl"))    darktable.unmuted |= DT_DEBUG_CAMCTL; // camera control module
-        else if(!strcmp(argv[k+1], "perf"))      darktable.unmuted |= DT_DEBUG_PERF; // performance measurements
-        else if(!strcmp(argv[k+1], "pwstorage")) darktable.unmuted |= DT_DEBUG_PWSTORAGE; // pwstorage module
-        else if(!strcmp(argv[k+1], "opencl"))    darktable.unmuted |= DT_DEBUG_OPENCL;    // gpu accel via opencl
-        else if(!strcmp(argv[k+1], "sql"))       darktable.unmuted |= DT_DEBUG_SQL; // SQLite3 queries
-        else if(!strcmp(argv[k+1], "memory"))    darktable.unmuted |= DT_DEBUG_MEMORY; // some stats on mem usage now and then.
+        if(!strcmp(argv[k+1], "all"))             darktable.unmuted = 0xffffffff;   // enable all debug information
+        else if(!strcmp(argv[k+1], "cache"))      darktable.unmuted |= DT_DEBUG_CACHE;   // enable debugging for lib/film/cache module
+        else if(!strcmp(argv[k+1], "control"))    darktable.unmuted |= DT_DEBUG_CONTROL; // enable debugging for scheduler module
+        else if(!strcmp(argv[k+1], "dev"))        darktable.unmuted |= DT_DEBUG_DEV; // develop module
+        else if(!strcmp(argv[k+1], "fswatch"))    darktable.unmuted |= DT_DEBUG_FSWATCH; // fswatch module
+        else if(!strcmp(argv[k+1], "camctl"))     darktable.unmuted |= DT_DEBUG_CAMCTL; // camera control module
+        else if(!strcmp(argv[k+1], "perf"))       darktable.unmuted |= DT_DEBUG_PERF; // performance measurements
+        else if(!strcmp(argv[k+1], "pwstorage"))  darktable.unmuted |= DT_DEBUG_PWSTORAGE; // pwstorage module
+        else if(!strcmp(argv[k+1], "opencl"))     darktable.unmuted |= DT_DEBUG_OPENCL;    // gpu accel via opencl
+        else if(!strcmp(argv[k+1], "sql"))        darktable.unmuted |= DT_DEBUG_SQL; // SQLite3 queries
+        else if(!strcmp(argv[k+1], "memory"))     darktable.unmuted |= DT_DEBUG_MEMORY; // some stats on mem usage now and then.
+        else if(!strcmp(argv[k+1], "lighttable")) darktable.unmuted |= DT_DEBUG_LIGHTTABLE; // lighttable related stuff.
         else return usage(argv[0]);
         k ++;
       }
@@ -354,10 +491,12 @@ int dt_init(int argc, char *argv[], const int init_gui)
         k ++;
       }
     }
+#ifndef MAC_INTEGRATION
     else
     {
-      image_to_load = argv[k];
+      images_to_load = g_slist_append(images_to_load, argv[k]);
     }
+#endif
   }
 
   if(darktable.unmuted & DT_DEBUG_MEMORY)
@@ -369,6 +508,15 @@ int dt_init(int argc, char *argv[], const int init_gui)
 #ifdef _OPENMP
   omp_set_num_threads(darktable.num_openmp_threads);
 #endif
+  dt_loc_init_datadir(datadirFromCommand);
+  dt_loc_init_plugindir(moduledirFromCommand);
+  if(dt_loc_init_tmp_dir(tmpdirFromCommand))
+  {
+    printf(_("ERROR : invalid temporary directory : %s\n"),darktable.tmpdir);
+    return usage(argv[0]);
+  }
+  dt_loc_init_user_config_dir(configdirFromCommand);
+  dt_loc_init_user_cache_dir(cachedirFromCommand);
 
   g_type_init();
 
@@ -379,16 +527,20 @@ int dt_init(int argc, char *argv[], const int init_gui)
   // dt_check_cpu(argc,argv);
 
 #ifdef HAVE_GEGL
-  (void)setenv("GEGL_PATH", DARKTABLE_DATADIR"/gegl:/usr/lib/gegl-0.0", 1);
+  char geglpath[DT_MAX_PATH_LEN];
+  char datadir[DT_MAX_PATH_LEN];
+  dt_loc_get_datadir(datadir, DT_MAX_PATH_LEN);
+  snprintf(geglpath, DT_MAX_PATH_LEN, "%s/gegl:/usr/lib/gegl-0.0", datadir);
+  (void)setenv("GEGL_PATH", geglpath, 1);
   gegl_init(&argc, &argv);
 #endif
 
   // thread-safe init:
   dt_exif_init();
-  char datadir[1024];
-  dt_util_get_user_config_dir (datadir,1024);
-  char filename[1024];
-  snprintf(filename, 1024, "%s/darktablerc", datadir);
+  char datadir[DT_MAX_PATH_LEN];
+  dt_loc_get_user_config_dir (datadir,DT_MAX_PATH_LEN);
+  char filename[DT_MAX_PATH_LEN];
+  snprintf(filename, DT_MAX_PATH_LEN, "%s/darktablerc", datadir);
 
   // intialize the config backend. this needs to be done first...
   darktable.conf = (dt_conf_t *)malloc(sizeof(dt_conf_t));
@@ -404,7 +556,12 @@ int dt_init(int argc, char *argv[], const int init_gui)
   }
 
   // initialize the database
-  darktable.db = dt_database_init(dbfilenameFromCommand);
+  darktable.db = dt_database_init(dbfilename_from_command);
+  if(darktable.db == NULL)
+  {
+    printf("ERROR : cannot open database\n");
+    return 1;
+  }
 
   // Initialize the signal system
   darktable.signals = dt_control_signal_init();
@@ -432,6 +589,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
   // FIXME: move there into dt_database_t
   dt_pthread_mutex_init(&(darktable.db_insert), NULL);
   dt_pthread_mutex_init(&(darktable.plugin_threadsafe), NULL);
+  dt_pthread_mutex_init(&(darktable.capabilities_threadsafe), NULL);
   darktable.control = (dt_control_t *)malloc(sizeof(dt_control_t));
   memset(darktable.control, 0, sizeof(dt_control_t));
   if(init_gui)
@@ -440,14 +598,12 @@ int dt_init(int argc, char *argv[], const int init_gui)
   }
   else
   {
-#if 0 // TODO: move int dt_database_t 
     // this is in memory, so schema can't exist yet.
-    if(!strcmp(dbfilename, ":memory:"))
+    if(dbfilename_from_command && !strcmp(dbfilename_from_command, ":memory:"))
     {
       dt_control_create_database_schema();
       dt_gui_presets_init(); // also init preset db schema.
     }
-#endif
     darktable.control->running = 0;
     darktable.control->accelerators = NULL;
     dt_pthread_mutex_init(&darktable.control->run_mutex, NULL);
@@ -460,6 +616,9 @@ int dt_init(int argc, char *argv[], const int init_gui)
   /* initialize sellection */
   darktable.selection = dt_selection_new();
 
+  /* capabilities set to NULL */
+  darktable.capabilities = NULL;
+
   darktable.opencl = (dt_opencl_t *)malloc(sizeof(dt_opencl_t));
   memset(darktable.opencl, 0, sizeof(dt_opencl_t));
   dt_opencl_init(darktable.opencl, argc, argv);
@@ -491,6 +650,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
     darktable.gui = (dt_gui_gtk_t *)malloc(sizeof(dt_gui_gtk_t));
     memset(darktable.gui,0,sizeof(dt_gui_gtk_t));
     if(dt_gui_gtk_init(darktable.gui, argc, argv)) return 1;
+    dt_bauhaus_init();
   }
   else darktable.gui = NULL;
 
@@ -509,119 +669,61 @@ int dt_init(int argc, char *argv[], const int init_gui)
 
     dt_control_load_config(darktable.control);
     g_strlcpy(darktable.control->global_settings.dbname, filename, 512); // overwrite if relocated.
-
-    darktable.imageio = (dt_imageio_t *)malloc(sizeof(dt_imageio_t));
-    memset(darktable.imageio, 0, sizeof(dt_imageio_t));
-    dt_imageio_init(darktable.imageio);
   }
+  darktable.imageio = (dt_imageio_t *)malloc(sizeof(dt_imageio_t));
+  memset(darktable.imageio, 0, sizeof(dt_imageio_t));
+  dt_imageio_init(darktable.imageio);
 
   if(init_gui)
   {
     // Loading the keybindings
-    char keyfile[1024];
+    char keyfile[DT_MAX_PATH_LEN];
 
     // First dump the default keymapping
-    snprintf(keyfile, 1024, "%s/keyboardrc_default", datadir);
+    snprintf(keyfile, DT_MAX_PATH_LEN, "%s/keyboardrc_default", datadir);
     gtk_accel_map_save(keyfile);
 
     // Removing extraneous semi-colons from the default keymap
     strip_semicolons_from_keymap(keyfile);
 
     // Then load any modified keys if available
-    snprintf(keyfile, 1024, "%s/keyboardrc", datadir);
+    snprintf(keyfile, DT_MAX_PATH_LEN, "%s/keyboardrc", datadir);
     if(g_file_test(keyfile, G_FILE_TEST_EXISTS))
       gtk_accel_map_load(keyfile);
     else
       gtk_accel_map_save(keyfile); // Save the default keymap if none is present
-  }
 
-  int id = 0;
-  if(init_gui && image_to_load)
-  {
-    char* filename;
-    if(g_str_has_prefix(image_to_load, "file://"))
-      image_to_load += strlen("file://");
-    if(g_path_is_absolute(image_to_load) == FALSE)
+    // load image(s) specified on cmdline
+    if (images_to_load)
     {
-      char* current_dir = g_get_current_dir();
-      char* tmp_filename = g_build_filename(current_dir, image_to_load, NULL);
-      filename = (char*)g_malloc(sizeof(char)*MAXPATHLEN);
-      if(realpath(tmp_filename, filename) == NULL)
+      // If only one image is listed, attempt to load it in darkroom
+      gboolean load_in_dr = (g_slist_next(images_to_load) == NULL);
+      GSList *p = images_to_load;
+
+      while (p != NULL)
       {
-        dt_control_log(_("found strange path `%s'"), tmp_filename);
-        g_free(current_dir);
-        g_free(tmp_filename);
-        g_free(filename);
-        return 0;
+	dt_load_from_string((gchar*)p->data, load_in_dr);
+	p = g_slist_next(p);
       }
-      g_free(current_dir);
-      g_free(tmp_filename);
+
+      if (!load_in_dr)
+	dt_ctl_switch_mode_to(DT_LIBRARY);
+
+      g_slist_free(images_to_load);
     }
     else
-    {
-      filename = g_strdup(image_to_load);
-    }
+      dt_ctl_switch_mode_to(DT_LIBRARY);
+  }
 
-    if(g_file_test(filename, G_FILE_TEST_IS_DIR))
-    {
-      // import a directory into a film roll
-      unsigned int last_char = strlen(filename)-1;
-      if(filename[last_char] == '/')
-        filename[last_char] = '\0';
-      id = dt_film_import(filename);
-      if(id)
-      {
-        dt_film_open(id);
-        dt_ctl_switch_mode_to(DT_LIBRARY);
-      }
-      else
-      {
-        dt_control_log(_("error loading directory `%s'"), filename);
-      }
-    }
-    else
-    {
-      // import a single image
-      gchar *directory = g_path_get_dirname((const gchar *)filename);
-      dt_film_t film;
-      const int filmid = dt_film_new(&film, directory);
-      id = dt_image_import(filmid, filename, TRUE);
-      g_free (directory);
-      if(id)
-      {
-        dt_film_open(filmid);
-        // make sure buffers are loaded (load full for testing)
-        dt_mipmap_buffer_t buf;
-        dt_mipmap_cache_read_get(darktable.mipmap_cache, &buf, id, DT_MIPMAP_FULL, DT_MIPMAP_BLOCKING);
-        if(!buf.buf)
-        {
-          id = 0;
-          dt_control_log(_("file `%s' has unknown format!"), filename);
-        }
-        else
-        {
-          dt_mipmap_cache_read_release(darktable.mipmap_cache, &buf);
-          DT_CTL_SET_GLOBAL(lib_image_mouse_over_id, id);
-          dt_ctl_switch_mode_to(DT_DEVELOP);
-        }
-      }
-      else
-      {
-        dt_control_log(_("error loading file `%s'"), filename);
-      }
-    }
-    g_free(filename);
-  }
-  if(init_gui && !id)
-  {
-    dt_ctl_switch_mode_to(DT_LIBRARY);
-  }
+  /* start the indexer background job */
+  dt_control_start_indexer();
 
   if(darktable.unmuted & DT_DEBUG_MEMORY)
   {
     fprintf(stderr, "[memory] after successful startup\n");
     dt_print_mem_usage();
   }
+
   return 0;
 }
 
@@ -670,9 +772,14 @@ void dt_cleanup()
   dt_fswatch_destroy(darktable.fswatch);
 
   dt_database_destroy(darktable.db);
- 
+
+  dt_bauhaus_cleanup();
+
+  dt_capabilities_cleanup();
+
   dt_pthread_mutex_destroy(&(darktable.db_insert));
   dt_pthread_mutex_destroy(&(darktable.plugin_threadsafe));
+  dt_pthread_mutex_destroy(&(darktable.capabilities_threadsafe));
 
   dt_exif_cleanup();
 #ifdef HAVE_GEGL
@@ -740,10 +847,11 @@ void dt_show_times(const dt_times_t *start, const char *prefix, const char *suff
 
 void dt_configure_defaults()
 {
+  const int atom_cores = dt_get_num_atom_cores();
   const int threads = dt_get_num_threads();
   const size_t mem = dt_get_total_memory();
   const int bits = (sizeof(void*) == 4) ? 32 : 64;
-  fprintf(stderr, "[defaults] found a %d-bit system with %zu kb ram and %d cores\n", bits, mem, threads);
+  fprintf(stderr, "[defaults] found a %d-bit system with %zu kb ram and %d cores (%d atom based)\n", bits, mem, threads, atom_cores);
   if(mem > (2u<<20) && threads > 4)
   {
     fprintf(stderr, "[defaults] setting high quality defaults\n");
@@ -752,14 +860,64 @@ void dt_configure_defaults()
     dt_conf_set_int("plugins/lighttable/thumbnail_width", 1300);
     dt_conf_set_int("plugins/lighttable/thumbnail_height", 1000);
   }
-  if(mem < (1u<<20) || threads <= 2 || bits < 64)
+  if(mem < (1u<<20) || threads <= 2 || bits < 64 || atom_cores > 0)
   {
     fprintf(stderr, "[defaults] setting very conservative defaults\n");
     dt_conf_set_int("worker_threads", 1);
-    dt_conf_set_int("cache_memory", 200u<<10);
+    dt_conf_set_int("cache_memory", 200u<<20);
+    dt_conf_set_int("host_memory_limit", 500);
+    dt_conf_set_int("singlebuffer_limit", 8);
     dt_conf_set_int("plugins/lighttable/thumbnail_width", 800);
     dt_conf_set_int("plugins/lighttable/thumbnail_height", 500);
+    dt_conf_set_string("plugins/darkroom/demosaic/quality", "always bilinear (fast)");
   }
 }
 
+
+int dt_capabilities_check(char *capability)
+{
+  GList *capabilities = darktable.capabilities;
+
+  while(capabilities)
+  {
+    if(!strcmp(capabilities->data, capability))
+    {
+      return TRUE;
+    }
+    capabilities = g_list_next(capabilities);
+  }
+  return FALSE;
+}
+
+
+void dt_capabilities_add(char *capability)
+{
+  dt_pthread_mutex_lock(&darktable.capabilities_threadsafe);
+
+  if(!dt_capabilities_check(capability))
+    darktable.capabilities = g_list_append(darktable.capabilities, capability);
+
+  dt_pthread_mutex_unlock(&darktable.capabilities_threadsafe);
+}
+
+
+void dt_capabilities_remove(char *capability)
+{
+  dt_pthread_mutex_lock(&darktable.capabilities_threadsafe);
+
+  darktable.capabilities = g_list_remove(darktable.capabilities, capability);
+
+  dt_pthread_mutex_unlock(&darktable.capabilities_threadsafe);
+}
+
+
+void dt_capabilities_cleanup()
+{
+  while(darktable.capabilities)
+    darktable.capabilities = g_list_delete_link(darktable.capabilities, darktable.capabilities);
+}
+
+
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
