@@ -29,8 +29,10 @@
 
 #include <CL/cl.h>
 
-#ifdef WIN32
+#if defined(WIN32)
 #define DT_OPENCL_LIBRARY "OpenCL"
+#elif defined(__APPLE__)
+#define DT_OPENCL_LIBRARY "/System/Library/Frameworks/OpenCL.framework/Versions/Current/OpenCL"
 #else
 #define DT_OPENCL_LIBRARY "libOpenCL"
 #endif
@@ -41,7 +43,7 @@ typedef cl_int (* dt_clGetDeviceIDs_t)(cl_platform_id, cl_device_type, cl_uint,
 typedef cl_int (* dt_clGetDeviceInfo_t)(cl_device_id, cl_device_info, size_t, void *, size_t *);
 typedef cl_context (* dt_clCreateContext_t)(const cl_context_properties *, cl_uint, const cl_device_id *, void (*)(const char *, const void *, size_t, void *), void *, cl_int * );
 typedef cl_context (* dt_clCreateContextFromType_t)(const cl_context_properties *, cl_device_type, void (*)(const char *, const void *, size_t, void *), void *, cl_int * );
-typedef cl_int (* dt_clRetainContext_t)(cl_context); 
+typedef cl_int (* dt_clRetainContext_t)(cl_context);
 typedef cl_int (* dt_clReleaseContext_t)(cl_context);
 typedef cl_int (* dt_clGetContextInfo_t)(cl_context, cl_context_info, size_t, void *, size_t *);
 typedef cl_command_queue (* dt_clCreateCommandQueue_t)(cl_context, cl_device_id, cl_command_queue_properties, cl_int *);
@@ -58,7 +60,7 @@ typedef cl_int (* dt_clReleaseMemObject_t)(cl_mem);
 typedef cl_int (* dt_clGetSupportedImageFormats_t)(cl_context, cl_mem_flags, cl_mem_object_type, cl_uint, cl_image_format *, cl_uint *);
 typedef cl_int (* dt_clGetMemObjectInfo_t)(cl_mem, cl_mem_info, size_t, void *, size_t *);
 typedef cl_int (* dt_clGetImageInfo_t)(cl_mem, cl_image_info, size_t, void *, size_t *);
-typedef cl_int (* dt_clSetMemObjectDestructorCallback_t)( cl_mem, void (*), void * ); 
+typedef cl_int (* dt_clSetMemObjectDestructorCallback_t)( cl_mem, void (*), void * );
 typedef cl_sampler (* dt_clCreateSampler_t)(cl_context, cl_bool, cl_addressing_mode, cl_filter_mode, cl_int *);
 typedef cl_int (* dt_clRetainSampler_t)(cl_sampler);
 typedef cl_int (* dt_clReleaseSampler_t)(cl_sampler);
@@ -80,7 +82,7 @@ typedef cl_int (* dt_clGetKernelInfo_t)(cl_kernel, cl_kernel_info, size_t, void
 typedef cl_int (* dt_clGetKernelWorkGroupInfo_t)(cl_kernel, cl_device_id, cl_kernel_work_group_info, size_t, void *, size_t *);
 typedef cl_int (* dt_clWaitForEvents_t)(cl_uint, const cl_event *);
 typedef cl_int (* dt_clGetEventInfo_t)(cl_event, cl_event_info, size_t, void *, size_t *);
-typedef cl_event (* dt_clCreateUserEvent_t)(cl_context, cl_int *); 
+typedef cl_event (* dt_clCreateUserEvent_t)(cl_context, cl_int *);
 typedef cl_int (* dt_clRetainEvent_t)(cl_event);
 typedef cl_int (* dt_clReleaseEvent_t)(cl_event);
 typedef cl_int (* dt_clSetUserEventStatus_t)(cl_event, cl_int);
@@ -91,7 +93,7 @@ typedef cl_int (* dt_clFinish_t)(cl_command_queue);
 typedef cl_int (* dt_clEnqueueReadBuffer_t)(cl_command_queue, cl_mem, cl_bool, size_t, size_t, void *, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueReadBufferRect_t)(cl_command_queue, cl_mem, cl_bool, const size_t *, const size_t *, const size_t *, size_t, size_t, size_t, size_t, void *, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueWriteBuffer_t)(cl_command_queue, cl_mem, cl_bool, size_t, size_t, const void *, cl_uint, const cl_event *, cl_event *);
-typedef cl_int (* dt_clEnqueueWriteBufferRect_t)(cl_command_queue, cl_mem, cl_bool, const size_t *, const size_t *, const size_t *, size_t, size_t, size_t, size_t, const void *, cl_uint, const cl_event *, cl_event *); 
+typedef cl_int (* dt_clEnqueueWriteBufferRect_t)(cl_command_queue, cl_mem, cl_bool, const size_t *, const size_t *, const size_t *, size_t, size_t, size_t, size_t, const void *, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueCopyBuffer_t)(cl_command_queue, cl_mem, cl_mem, size_t, size_t, size_t, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueCopyBufferRect_t)(cl_command_queue, cl_mem, cl_mem, const size_t *, const size_t *, const size_t *, size_t, size_t, size_t, size_t, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueReadImage_t)(cl_command_queue, cl_mem, cl_bool, const size_t *, const size_t *, size_t, size_t, void *, cl_uint, const cl_event *, cl_event *);
@@ -103,7 +105,7 @@ typedef void * (* dt_clEnqueueMapBuffer_t)(cl_command_queue, cl_mem, cl_bool, cl
 typedef void * (* dt_clEnqueueMapImage_t)(cl_command_queue, cl_mem, cl_bool, cl_map_flags, const size_t *, const size_t *, size_t *, size_t *, cl_uint, const cl_event *, cl_event *, cl_int *);
 typedef cl_int (* dt_clEnqueueUnmapMemObject_t)(cl_command_queue, cl_mem, void *, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueNDRangeKernel_t)(cl_command_queue, cl_kernel, cl_uint, const size_t *, const size_t *, const size_t *, cl_uint, const cl_event *, cl_event *);
-typedef cl_int (* dt_clEnqueueTask_t)(cl_command_queue, cl_kernel, cl_uint, const cl_event *, cl_event *); 
+typedef cl_int (* dt_clEnqueueTask_t)(cl_command_queue, cl_kernel, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueNativeKernel_t)(cl_command_queue, void (*user_func)(void *), void *, size_t, cl_uint, const cl_mem *, const void **, cl_uint, const cl_event *, cl_event *);
 typedef cl_int (* dt_clEnqueueMarker_t)(cl_command_queue, cl_event *);
 typedef cl_int (* dt_clEnqueueWaitForEvents_t)(cl_command_queue, cl_uint, const cl_event *);
@@ -206,3 +208,6 @@ int dt_dlopencl_init(const char *, dt_dlopencl_t **);
 #endif
 #endif
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
