@@ -69,11 +69,13 @@ void dt_film_set_query(const int32_t id)
   dt_conf_set_int("plugins/lighttable/collect/num_rules", 1);
   dt_conf_set_int("plugins/lighttable/collect/item0", 0);
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id, folder from film_rolls where id = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id, folder from film_rolls where id = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   if(sqlite3_step(stmt) == SQLITE_ROW)
   {
-    dt_conf_set_string("plugins/lighttable/collect/string0", (gchar *)sqlite3_column_text (stmt, 1));
+    dt_conf_set_string("plugins/lighttable/collect/string0",
+                       (gchar *)sqlite3_column_text (stmt, 1));
   }
   sqlite3_finalize (stmt);
   dt_collection_update_query(darktable.collection);
@@ -88,7 +90,8 @@ dt_film_open2 (dt_film_t *film)
 
   /* query database for id and folder */
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id, folder from film_rolls where id = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id, folder from film_rolls where id = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, film->id);
   if(sqlite3_step(stmt) == SQLITE_ROW)
   {
@@ -98,8 +101,11 @@ dt_film_open2 (dt_film_t *film)
     char datetime[20];
     dt_gettime (datetime);
 
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "update film_rolls set datetime_accessed = ?1 where id = ?2", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime), SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "update film_rolls set datetime_accessed = ?1 where id = ?2",
+                                -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime),
+                               SQLITE_STATIC);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, film->id);
     sqlite3_step (stmt);
 
@@ -118,7 +124,8 @@ dt_film_open2 (dt_film_t *film)
 int dt_film_open(const int32_t id)
 {
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id, folder from film_rolls where id = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id, folder from film_rolls where id = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   if(sqlite3_step(stmt) == SQLITE_ROW)
   {
@@ -126,8 +133,11 @@ int dt_film_open(const int32_t id)
     char datetime[20];
     dt_gettime(datetime);
 
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "update film_rolls set datetime_accessed = ?1 where id = ?2", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime), SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "update film_rolls set datetime_accessed = ?1 where id = ?2",
+                                -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime),
+                               SQLITE_STATIC);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, id);
     sqlite3_step(stmt);
   }
@@ -143,7 +153,9 @@ int dt_film_open(const int32_t id)
 int dt_film_open_recent(const int num)
 {
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from film_rolls order by datetime_accessed desc limit ?1,1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id from film_rolls order by datetime_accessed desc limit ?1,1",
+                              -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, num);
   if(sqlite3_step(stmt) == SQLITE_ROW)
   {
@@ -152,8 +164,11 @@ int dt_film_open_recent(const int num)
     if(dt_film_open(id)) return 1;
     char datetime[20];
     dt_gettime(datetime);
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "update film_rolls set datetime_accessed = ?1 where id = ?2", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime), SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "update film_rolls set datetime_accessed = ?1 where id = ?2",
+                                -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime),
+                               SQLITE_STATIC);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, id);
     sqlite3_step(stmt);
   }
@@ -168,9 +183,12 @@ int dt_film_new(dt_film_t *film, const char *directory)
   film->id = -1;
   int rc;
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from film_rolls where folder = ?1", -1, &stmt, NULL);
-  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, directory, strlen(directory), SQLITE_STATIC);
-  if(sqlite3_step(stmt) == SQLITE_ROW) film->id = sqlite3_column_int(stmt, 0);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id from film_rolls where folder = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, directory, strlen(directory),
+                             SQLITE_STATIC);
+  if(sqlite3_step(stmt) == SQLITE_ROW)
+    film->id = sqlite3_column_int(stmt, 0);
   sqlite3_finalize(stmt);
 
   if(film->id <= 0)
@@ -179,16 +197,25 @@ int dt_film_new(dt_film_t *film, const char *directory)
     sqlite3_stmt *stmt;
     char datetime[20];
     dt_gettime(datetime);
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into film_rolls (id, datetime_accessed, folder) values (null, ?1, ?2)", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime), SQLITE_STATIC);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 2, directory, strlen(directory), SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "insert into film_rolls (id, datetime_accessed, folder) "
+                                "values (null, ?1, ?2)", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime),
+                               SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 2, directory, strlen(directory),
+                               SQLITE_STATIC);
     dt_pthread_mutex_lock(&darktable.db_insert);
     rc = sqlite3_step(stmt);
-    if(rc != SQLITE_DONE) fprintf(stderr, "[film_new] failed to insert film roll! %s\n", sqlite3_errmsg(dt_database_get(darktable.db)));
+    if(rc != SQLITE_DONE)
+      fprintf(stderr, "[film_new] failed to insert film roll! %s\n",
+              sqlite3_errmsg(dt_database_get(darktable.db)));
     sqlite3_finalize(stmt);
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from film_rolls where folder=?1", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, directory, strlen(directory), SQLITE_STATIC);
-    if(sqlite3_step(stmt) == SQLITE_ROW) film->id = sqlite3_column_int(stmt, 0);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "select id from film_rolls where folder=?1", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, directory, strlen(directory),
+                               SQLITE_STATIC);
+    if(sqlite3_step(stmt) == SQLITE_ROW)
+      film->id = sqlite3_column_int(stmt, 0);
     sqlite3_finalize(stmt);
     dt_pthread_mutex_unlock(&darktable.db_insert);
   }
@@ -217,11 +244,13 @@ dt_film_import_blocking(const char *dirname, const int blocking)
   dt_film_t *film = (dt_film_t *)malloc(sizeof(dt_film_t));
   dt_film_init(film);
   film->id = -1;
- 
+
   /* lookup if film exists and reuse id */
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from film_rolls where folder = ?1", -1, &stmt, NULL);
-  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, dirname, strlen(dirname), SQLITE_STATIC);
-  if(sqlite3_step(stmt) == SQLITE_ROW) 
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id from film_rolls where folder = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, dirname, strlen(dirname),
+                             SQLITE_STATIC);
+  if(sqlite3_step(stmt) == SQLITE_ROW)
     film->id = sqlite3_column_int(stmt, 0);
   sqlite3_finalize(stmt);
 
@@ -230,20 +259,85 @@ dt_film_import_blocking(const char *dirname, const int blocking)
   {
     char datetime[20];
     dt_gettime(datetime);
+#if 0
+    /* Should we use one for the whole app? */
+    GVolumeMonitor *gv_monitor;
+    gv_monitor = g_volume_monitor_get ();
+
+    GList *mounts;
+    mounts = g_volume_monitor_get_mounts(gv_monitor);
+
+    gchar *mount_name = NULL;
+    if (mounts != NULL)
+    {
+      GFile *mount_gfile;
+      GMount *filmroll_mount;
+      GError *error = NULL;
+      gchar *filmroll_path;
+
+      filmroll_mount = g_file_find_enclosing_mount(g_file_new_for_path(dirname), NULL, &error);
+      if (!error)
+      /* We are considering that the only error is that there is no mount
+       * because the filmroll added is in a local drive */
+      {
+        filmroll_path = g_file_get_path((g_mount_get_default_location(filmroll_mount)));
+
+        for (int i=0; i < g_list_length (mounts); i++)
+        {
+          gchar *p;
+
+          mount_gfile = g_mount_get_default_location((GMount *)g_list_nth_data(mounts, i));
+          p = g_file_get_path(mount_gfile);
+
+          if (g_strcmp0(p, filmroll_path))
+          {
+            mount_name = g_mount_get_name(g_list_nth_data(mounts, i));
+            break;
+          }
+
+          g_free(p);
+          g_object_unref (mount_gfile);
+        }
+
+        g_free (filmroll_path);
+      }
+
+      if (filmroll_mount != NULL) g_object_unref (filmroll_mount);
+
+      /* We haven't found the device in the list of connected devices. Let's suppose it is local */
+      if (mount_name == NULL)
+        mount_name = g_strdup("Local");
+    }
+    else
+      mount_name = g_strdup("Local");
+#endif
     /* insert a new film roll into database */
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into film_rolls (id, datetime_accessed, folder) values (null, ?1, ?2)", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime), SQLITE_STATIC);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 2, dirname, strlen(dirname), SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+//                                "insert into film_rolls (id, datetime_accessed, folder, external_drive) values "
+//                                "(null, ?1, ?2, ?3)", -1, &stmt, NULL);
+                                "insert into film_rolls (id, datetime_accessed, folder) values "
+                                "(null, ?1, ?2)", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, datetime, strlen(datetime),
+                               SQLITE_STATIC);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 2, dirname, strlen(dirname),
+                               SQLITE_STATIC);
+//    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 3, mount_name, strlen(mount_name),
+//                               SQLITE_STATIC);
+
     rc = sqlite3_step(stmt);
-    if(rc != SQLITE_DONE) 
-      fprintf(stderr, "[film_import] failed to insert film roll! %s\n", 
-	      sqlite3_errmsg(dt_database_get(darktable.db)));
+    if(rc != SQLITE_DONE)
+      fprintf(stderr, "[film_import] failed to insert film roll! %s\n",
+              sqlite3_errmsg(dt_database_get(darktable.db)));
     sqlite3_finalize(stmt);
-    
+
+//    if (mount_name != NULL) g_free (mount_name);
+
     /* requery for filmroll and fetch new id */
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from film_rolls where folder=?1", -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, dirname, strlen(dirname), SQLITE_STATIC);
-    if(sqlite3_step(stmt) == SQLITE_ROW) 
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "select id from film_rolls where folder=?1", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, dirname, strlen(dirname),
+                               SQLITE_STATIC);
+    if(sqlite3_step(stmt) == SQLITE_ROW)
       film->id = sqlite3_column_int(stmt, 0);
     sqlite3_finalize(stmt);
   }
@@ -283,10 +377,10 @@ static GList *_film_recursive_get_files(const gchar *path, gboolean recursive,GL
   {
     /* get the current filename */
     const gchar *filename = g_dir_read_name(cdir);
-        
+
     /* return if no more files are in current dir */
     if (!filename) break;
-  
+
     /* build full path for filename */
     fullname = g_build_filename(G_DIR_SEPARATOR_S, path, filename, NULL);
 
@@ -297,12 +391,14 @@ static GList *_film_recursive_get_files(const gchar *path, gboolean recursive,GL
       g_free(fullname);
     }
     /* or test if we found a support image format to import */
-    else if (!g_file_test(fullname, G_FILE_TEST_IS_DIR) && dt_supported_image(filename))
+    else if (!g_file_test(fullname, G_FILE_TEST_IS_DIR) &&
+             dt_supported_image(filename))
       *result = g_list_append(*result, fullname);
     else
       g_free(fullname);
-    
-  } while (TRUE);
+
+  }
+  while (TRUE);
 
   /* cleanup and return results */
   g_dir_close(cdir);
@@ -325,9 +421,9 @@ void dt_film_import1(dt_film_t *film)
   /* first of all gather all images to import */
   GList *images = NULL;
   images = _film_recursive_get_files(film->dirname, recursive, &images);
-  if(g_list_length(images) == 0) 
+  if(g_list_length(images) == 0)
   {
-    dt_control_log(_("no supported images was found to be imported"));
+    dt_control_log(_("no supported images were found to be imported"));
     return;
   }
 
@@ -337,25 +433,49 @@ void dt_film_import1(dt_film_t *film)
   /* let's start import of images */
   gchar message[512] = {0};
   double fraction = 0;
-  uint32_t total = g_list_length(images); 
-  g_snprintf(message, 512, ngettext("importing %d image","importing %d images", total), total);
+  uint32_t total = g_list_length(images);
+  g_snprintf(message, sizeof(message) - 1,
+             ngettext("importing %d image","importing %d images", total), total);
   const guint *jid = dt_control_backgroundjobs_create(darktable.control, 0, message);
 
   /* loop thru the images and import to current film roll */
   dt_film_t *cfr = film;
   GList *image = g_list_first(images);
-  do {
+  do
+  {
     gchar *cdn = g_path_get_dirname((const gchar *)image->data);
-   
+
     /* check if we need to initialize a new filmroll */
     if(!cfr || g_strcmp0(cfr->dirname, cdn) != 0)
     {
-      /* cleanup previously imported filmroll*/
-      if(cfr && cfr!=film) 
+
+#if GLIB_CHECK_VERSION (2, 26, 0)
+      if(cfr && cfr->dir)
       {
-	dt_film_cleanup(cfr);
-	g_free(cfr);
-	cfr = NULL;
+        /* check if we can find a gpx data file to be auto applied
+           to images in the jsut imported filmroll */
+        g_dir_rewind(cfr->dir);
+        const gchar *dfn = NULL;
+        while ((dfn = g_dir_read_name(cfr->dir)) != NULL)
+        {
+          /* check if we have a gpx to be auto applied to filmroll */
+          if(strcmp(dfn+strlen(dfn)-4,".gpx") == 0 ||
+              strcmp(dfn+strlen(dfn)-4,".GPX") == 0)
+          {
+            gchar *gpx_file = g_build_path (G_DIR_SEPARATOR_S, cfr->dirname, dfn, NULL);
+            dt_control_gpx_apply(gpx_file, cfr->id, dt_conf_get_string("plugins/lighttable/geotagging/tz"));
+            g_free(gpx_file);
+          }
+        }
+      }
+#endif
+
+      /* cleanup previously imported filmroll*/
+      if(cfr && cfr!=film)
+      {
+        dt_film_cleanup(cfr);
+        g_free(cfr);
+        cfr = NULL;
       }
 
       /* initialize and create a new film to import to */
@@ -371,15 +491,40 @@ void dt_film_import1(dt_film_t *film)
     fraction+=1.0/total;
     dt_control_backgroundjobs_progress(darktable.control, jid, fraction);
 
-  } while( (image = g_list_next(image)) != NULL);
-  
-  dt_control_backgroundjobs_destroy(darktable.control, jid);
+  }
+  while( (image = g_list_next(image)) != NULL);
 
+  dt_control_backgroundjobs_destroy(darktable.control, jid);
+  dt_control_signal_raise(darktable.signals , DT_SIGNAL_FILMROLLS_CHANGED);
+
+#if GLIB_CHECK_VERSION (2, 26, 0)
+  if(cfr && cfr->dir)
+  {
+    /* check if we can find a gpx data file to be auto applied
+       to images in the just imported filmroll */
+    g_dir_rewind(cfr->dir);
+    const gchar *dfn = NULL;
+    while ((dfn = g_dir_read_name(cfr->dir)) != NULL)
+    {
+      /* check if we have a gpx to be auto applied to filmroll */
+      if(strcmp(dfn+strlen(dfn)-4,".gpx") == 0 ||
+          strcmp(dfn+strlen(dfn)-4,".GPX") == 0)
+      {
+        gchar *gpx_file = g_build_path (G_DIR_SEPARATOR_S, cfr->dirname, dfn, NULL);
+        dt_control_gpx_apply(gpx_file, cfr->id, dt_conf_get_string("plugins/lighttable/geotagging/tz"));
+        g_free(gpx_file);
+      }
+    }
+  }
+#endif
 }
 
+
 int dt_film_import(const char *dirname)
 {
-  return dt_film_import_blocking(dirname,0);
+  int v = dt_film_import_blocking(dirname,0);
+  dt_control_signal_raise(darktable.signals , DT_SIGNAL_FILMROLLS_CHANGED);
+  return v;
 }
 
 void dt_film_remove_empty()
@@ -389,51 +534,69 @@ void dt_film_remove_empty()
                         "delete from film_rolls where id in (select id from film_rolls as B where "
                         "(select count(A.id) from images as A where A.film_id=B.id)=0)",
                         NULL, NULL, NULL);
+
+  dt_control_signal_raise(darktable.signals , DT_SIGNAL_FILMROLLS_CHANGED);
 }
 
 int dt_film_is_empty(const int id)
 {
   int empty=0;
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from images where film_id = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id from images where film_id = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   if( sqlite3_step(stmt) != SQLITE_ROW) empty=1;
   sqlite3_finalize(stmt);
   return empty;
 }
 
-// This is basically the same as dt_image_remove() from common/image.c. It just does the iteration over all images in the SQL statement
+// This is basically the same as dt_image_remove() from common/image.c.
+// It just does the iteration over all images in the SQL statement
 void dt_film_remove(const int id)
 {
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "update tagxtag set count = count - 1 where "
-                              "(id2 in (select tagid from tagged_images where imgid in (select id from images where film_id = ?1))) or "
-                              "(id1 in (select tagid from tagged_images where imgid in (select id from images where film_id = ?1)))", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "update tagxtag set count = count - 1 where "
+                              "(id2 in (select tagid from tagged_images where imgid in "
+                              "(select id from images where film_id = ?1))) or (id1 in "
+                              "(select tagid from tagged_images where imgid in "
+                              "(select id from images where film_id = ?1)))", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from tagged_images where imgid in (select id from images where film_id = ?1)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from tagged_images where imgid in "
+                              "(select id from images where film_id = ?1)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from history where imgid in (select id from images where film_id = ?1)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from history where imgid in "
+                              "(select id from images where film_id = ?1)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from color_labels where imgid in (select id from images where film_id = ?1)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from color_labels where imgid in "
+                              "(select id from images where film_id = ?1)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from meta_data where id in (select id from images where film_id = ?1)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from meta_data where id in "
+                              "(select id from images where film_id = ?1)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from selected_images where imgid in (select id from images where film_id = ?1)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from selected_images where imgid in "
+                              "(select id from images where film_id = ?1)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
 
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from images where film_id = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "select id from images where film_id = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   while(sqlite3_step(stmt) == SQLITE_ROW)
   {
@@ -443,16 +606,22 @@ void dt_film_remove(const int id)
   }
   sqlite3_finalize(stmt);
 
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from images where id in (select id from images where film_id = ?1)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from images where id in "
+                              "(select id from images where film_id = ?1)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
 
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from film_rolls where id = ?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "delete from film_rolls where id = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
   // dt_control_update_recent_films();
+  dt_control_signal_raise(darktable.signals , DT_SIGNAL_FILMROLLS_CHANGED);
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
