@@ -1,7 +1,7 @@
 /*
     This file is part of darktable,
     copyright (c) 2010 henrik andersson,
-    copyright (c) 2011 johannes hanika
+    copyright (c) 2011-2012 johannes hanika
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,19 @@
 #include "common/tags.h"
 #include "common/utility.h"
 
+static void
+remove_preset_flag(const int imgid)
+{
+  const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, imgid);
+  dt_image_t *image = dt_image_cache_write_get(darktable.image_cache, cimg);
+
+  // clear flag
+  image->flags &= ~DT_IMAGE_AUTO_PRESETS_APPLIED;
+
+  // write through to sql+xmp
+  dt_image_cache_write_release(darktable.image_cache, image, DT_IMAGE_CACHE_SAFE);
+  dt_image_cache_read_release(darktable.image_cache, cimg);
+}
 
 void dt_history_delete_on_image(int32_t imgid)
 {
@@ -37,9 +50,7 @@ void dt_history_delete_on_image(int32_t imgid)
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
   sqlite3_step (stmt);
   sqlite3_finalize (stmt);
-
-  dt_image_t tmp;
-  dt_image_init (&tmp);
+  remove_preset_flag(imgid);
 
   /* if current image in develop reload history */
   if (dt_dev_is_current_image (darktable.develop, imgid))
@@ -49,8 +60,7 @@ void dt_history_delete_on_image(int32_t imgid)
   dt_mipmap_cache_remove(darktable.mipmap_cache, imgid);
 
   /* remove darktable|style|* tags */
-  dt_tag_detach_by_string("darktable|style%",imgid);
-
+  dt_tag_detach_by_string("darktable|style%", imgid);
 }
 
 void
@@ -109,7 +119,7 @@ dt_history_copy_and_paste_on_image (int32_t imgid, int32_t dest_imgid, gboolean
     dt_control_log(_("you need to copy history from an image before you paste it onto another"));
     return 1;
   }
-    
+
   /* if merge onto history stack, lets find history offest in destination image */
   int32_t offs = 0;
   if (merge)
@@ -129,7 +139,7 @@ dt_history_copy_and_paste_on_image (int32_t imgid, int32_t dest_imgid, gboolean
   sqlite3_finalize (stmt);
 
   /* add the history items to stack offest */
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into history (imgid, num, module, operation, op_params, enabled, blendop_params) select ?1, num+?2, module, operation, op_params, enabled, blendop_params from history where imgid = ?3", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into history (imgid, num, module, operation, op_params, enabled, blendop_params, blendop_version) select ?1, num+?2, module, operation, op_params, enabled, blendop_params, blendop_version from history where imgid = ?3", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, dest_imgid);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, offs);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 3, imgid);
@@ -138,7 +148,10 @@ dt_history_copy_and_paste_on_image (int32_t imgid, int32_t dest_imgid, gboolean
 
   /* if current image in develop reload history */
   if (dt_dev_is_current_image(darktable.develop, dest_imgid))
+  {
     dt_dev_reload_history_items (darktable.develop);
+    dt_dev_modulegroups_set(darktable.develop, dt_dev_modulegroups_get(darktable.develop));
+  }
 
   /* update xmp file */
   dt_image_synch_xmp(dest_imgid);
@@ -160,8 +173,9 @@ dt_history_get_items(int32_t imgid)
     char name[512]= {0};
     dt_history_item_t *item=g_malloc (sizeof (dt_history_item_t));
     item->num = sqlite3_column_int (stmt, 0);
-    g_snprintf(name,512,"%s (%s)",sqlite3_column_text (stmt, 1),(sqlite3_column_int (stmt, 2)!=0)?_("on"):_("off"));
+    g_snprintf(name,512,"%s (%s)",dt_iop_get_localized_name((char*)sqlite3_column_text(stmt, 1)), (sqlite3_column_int(stmt, 2)!=0)?_("on"):_("off"));
     item->name = g_strdup (name);
+    item->op = g_strdup((gchar *)sqlite3_column_text(stmt, 1));
     result = g_list_append (result,item);
   }
   return result;
@@ -216,4 +230,6 @@ dt_history_copy_and_paste_on_selection (int32_t imgid, gboolean merge)
   return res;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
