@@ -25,6 +25,7 @@
 #include "control/conf.h"
 #include "control/jobs.h"
 #include "libraw/libraw.h"
+#include "squish/csquish.h"
 
 #include <assert.h>
 #include <string.h>
@@ -39,7 +40,7 @@
 #include <xmmintrin.h>
 
 #define DT_MIPMAP_CACHE_FILE_MAGIC 0xD71337
-#define DT_MIPMAP_CACHE_FILE_VERSION 21
+#define DT_MIPMAP_CACHE_FILE_VERSION 22
 #define DT_MIPMAP_CACHE_DEFAULT_FILE_NAME "mipmaps"
 
 #define DT_MIPMAP_BUFFER_DSC_FLAG_GENERATE (1<<0)
@@ -67,14 +68,16 @@ dead_image_8(dt_mipmap_buffer_t *buf)
   const uint32_t X = 0xffffffffu;
   const uint32_t o = 0u;
   const uint32_t image[] =
-  { o, o, o, o, o, o, o, o,
+  {
+    o, o, o, o, o, o, o, o,
     o, o, X, X, X, X, o, o,
     o, X, o, X, X, o, X, o,
     o, X, X, X, X, X, X, o,
     o, o, X, o, o, X, o, o,
     o, o, o, o, o, o, o, o,
     o, o, X, X, X, X, o, o,
-    o, o, o, o, o, o, o, o };
+    o, o, o, o, o, o, o, o
+  };
   memcpy(buf->buf, image, sizeof(uint32_t)*64);
 }
 
@@ -88,17 +91,33 @@ dead_image_f(dt_mipmap_buffer_t *buf)
   const __m128 X = _mm_set1_ps(1.0f);
   const __m128 o = _mm_set1_ps(0.0f);
   const __m128 image[] =
-  { o, o, o, o, o, o, o, o,
+  {
+    o, o, o, o, o, o, o, o,
     o, o, X, X, X, X, o, o,
     o, X, o, X, X, o, X, o,
     o, X, X, X, X, X, X, o,
     o, o, X, o, o, X, o, o,
     o, o, o, o, o, o, o, o,
     o, o, X, X, X, X, o, o,
-    o, o, o, o, o, o, o, o };
+    o, o, o, o, o, o, o, o
+  };
   memcpy(buf->buf, image, sizeof(__m128)*64);
 }
 
+static inline int32_t
+compressed_buffer_size(const int32_t compression_type, const int width, const int height)
+{
+  if(width <= 8 && height <= 8)
+    // skulls are uncompressed
+    return 8*8*sizeof(uint32_t);
+  else if(compression_type)
+    // need 8 byte for each 4x4 block of pixels.
+    // round correctly, so a 3x3 image will still consume one block:
+    return ((width-1)/4 + 1) * ((height-1)/4 + 1) * 8;
+  else // uncompressed:
+    return width*height*sizeof(uint32_t);
+}
+
 static inline int32_t
 buffer_is_broken(dt_mipmap_buffer_t *buf)
 {
@@ -134,6 +153,7 @@ typedef struct _iterate_data_t
 {
   FILE *f;
   uint8_t *blob;
+  int compression_type;
   dt_mipmap_size_t mip;
 }
 _iterate_data_t;
@@ -152,22 +172,35 @@ _write_buffer(const uint32_t key, const void *data, void *user_data)
   written = fwrite(&key, sizeof(uint32_t), 1, d->f);
   if(written != 1) return 1;
 
-  dt_mipmap_buffer_t buf;
-  buf.width  = dsc->width;
-  buf.height = dsc->height;
-  buf.imgid  = get_imgid(key);
-  buf.size   = get_size(key);
-  // skip to next 8-byte alignment, for sse buffers.
-  buf.buf    = (uint8_t *)(dsc+1);
+  if(d->compression_type)
+  {
+    // write buffer size, wd, ht and the full blob, as it is in memory.
+    const int32_t length = compressed_buffer_size(d->compression_type, dsc->width, dsc->height);
+    written = fwrite(&length, sizeof(int32_t), 1, d->f);
+    if(written != 1) return 1;
+    written = fwrite(&dsc->width, sizeof(int32_t), 1, d->f);
+    if(written != 1) return 1;
+    written = fwrite(&dsc->height, sizeof(int32_t), 1, d->f);
+    if(written != 1) return 1;
+    written = fwrite(dsc+1, sizeof(uint8_t), length, d->f);
+    if(written != length) return 1;
+  }
+  else
+  {
+    dt_mipmap_buffer_t buf;
+    buf.width  = dsc->width;
+    buf.height = dsc->height;
+    buf.imgid  = get_imgid(key);
+    buf.size   = get_size(key);
+    // skip to next 8-byte alignment, for sse buffers.
+    buf.buf    = (uint8_t *)(dsc+1);
 
-  const int32_t length = dt_imageio_jpeg_compress(buf.buf, d->blob, buf.width, buf.height, MIN(100, MAX(10, dt_conf_get_int("database_cache_quality"))));
-  written = fwrite(&length, sizeof(int32_t), 1, d->f);
-  if(written != 1) return 1;
-  written = fwrite(d->blob, sizeof(uint8_t), length, d->f);
-  if(written != length) return 1;
-
-
-//  fprintf(stderr, "[mipmap_cache] serializing image %u (%d x %d) with %d bytes in level %d\n", get_imgid(key), buf.width, buf.height, length, d->mip);
+    const int32_t length = dt_imageio_jpeg_compress(buf.buf, d->blob, buf.width, buf.height, MIN(100, MAX(10, dt_conf_get_int("database_cache_quality"))));
+    written = fwrite(&length, sizeof(int32_t), 1, d->f);
+    if(written != 1) return 1;
+    written = fwrite(d->blob, sizeof(uint8_t), length, d->f);
+    if(written != length) return 1;
+  }
 
   return 0;
 }
@@ -180,8 +213,8 @@ dt_mipmap_cache_get_filename(
   char* abspath = NULL;
 
   // Directory
-  char cachedir[1024];
-  dt_util_get_user_cache_dir(cachedir, sizeof(cachedir));
+  char cachedir[DT_MAX_PATH_LEN];
+  dt_loc_get_user_cache_dir(cachedir, sizeof(cachedir));
 
   // Build the mipmap filename
   const gchar *dbfilename = dt_database_get_path(darktable.db);
@@ -212,15 +245,15 @@ dt_mipmap_cache_get_filename(
   r = 0;
 
 exit:
-    free(abspath);
+  free(abspath);
 
-    return r;
+  return r;
 }
 
 static int
 dt_mipmap_cache_serialize(dt_mipmap_cache_t *cache)
 {
-  gchar dbfilename[1024];
+  gchar dbfilename[DT_MAX_PATH_LEN];
   if (dt_mipmap_cache_get_filename(dbfilename, sizeof(dbfilename)))
   {
     fprintf(stderr, "[mipmap_cache] could not retrieve cache filename; not serializing\n");
@@ -249,7 +282,11 @@ dt_mipmap_cache_serialize(dt_mipmap_cache_t *cache)
   written = fwrite(&magic, sizeof(int32_t), 1, f);
   if(written != 1) goto write_error;
 
-  for(int i=0;i<=mip;i++)
+  // store compression type
+  written = fwrite(&cache->compression_type, sizeof(int32_t), 1, f);
+  if(written != 1) goto write_error;
+
+  for(int i=0; i<=mip; i++)
   {
     // print max sizes for this cache
     written = fwrite(&cache->mip[i].max_width, sizeof(int32_t), 1, f);
@@ -258,9 +295,10 @@ dt_mipmap_cache_serialize(dt_mipmap_cache_t *cache)
     if(written != 1) goto write_error;
   }
 
-  for(int i=0;i<=mip;i++)
+  for(int i=0; i<=mip; i++)
   {
     d.mip = (dt_mipmap_size_t)i;
+    d.compression_type = cache->compression_type;
     if(dt_cache_for_all(&cache->mip[i].cache, _write_buffer, &d)) goto write_error;
   }
 
@@ -283,7 +321,7 @@ dt_mipmap_cache_deserialize(dt_mipmap_cache_t *cache)
   uint8_t *blob = NULL;
   int file_width[mip+1], file_height[mip+1];
 
-  gchar dbfilename[1024];
+  gchar dbfilename[DT_MAX_PATH_LEN];
   if (dt_mipmap_cache_get_filename(dbfilename, sizeof(dbfilename)))
   {
     fprintf(stderr, "[mipmap_cache] could not retrieve cache filename; not deserializing\n");
@@ -296,7 +334,7 @@ dt_mipmap_cache_deserialize(dt_mipmap_cache_t *cache)
   }
 
   FILE *f = fopen(dbfilename, "rb");
-  if(!f) 
+  if(!f)
   {
     if (errno == ENOENT)
     {
@@ -317,9 +355,22 @@ dt_mipmap_cache_deserialize(dt_mipmap_cache_t *cache)
   if(magic_file != magic)
   {
     if(magic_file > DT_MIPMAP_CACHE_FILE_MAGIC && magic_file < magic)
-        fprintf(stderr, "[mipmap_cache] cache version too old, dropping `%s' cache\n", dbfilename);
+      fprintf(stderr, "[mipmap_cache] cache version too old, dropping `%s' cache\n", dbfilename);
     else
-        fprintf(stderr, "[mipmap_cache] invalid cache file, dropping `%s' cache\n", dbfilename);
+      fprintf(stderr, "[mipmap_cache] invalid cache file, dropping `%s' cache\n", dbfilename);
+    goto read_finalize;
+  }
+
+  // also read compression type and yell out on missmatch.
+  int32_t compression = -1;
+  rd = fread(&compression, sizeof(int32_t), 1, f);
+  if(rd != 1) goto read_error;
+  if(compression != cache->compression_type)
+  {
+    fprintf(stderr, "[mipmap_cache] cache is %s, but settings say we should use %s, dropping `%s' cache\n",
+            compression == 0 ? "uncompressed" : (compression == 1 ? "low quality compressed" : "high quality compressed"),
+            cache->compression_type == 0 ? "no compression" : (cache->compression_type == 1 ? "low quality compression" : "high quality compression"),
+            dbfilename);
     goto read_finalize;
   }
 
@@ -330,45 +381,64 @@ dt_mipmap_cache_deserialize(dt_mipmap_cache_t *cache)
     rd = fread(&file_height[i], sizeof(int32_t), 1, f);
     if(rd != 1) goto read_error;
     if(file_width[i]  != cache->mip[i].max_width ||
-       file_height[i] != cache->mip[i].max_height)
+        file_height[i] != cache->mip[i].max_height)
     {
       fprintf(stderr, "[mipmap_cache] cache settings changed, dropping `%s' cache\n", dbfilename);
       goto read_finalize;
     }
   }
-  blob = (uint8_t *)malloc(4*sizeof(uint8_t)*file_width[mip]*file_height[mip]);
+
+  if(cache->compression_type) blob = NULL;
+  else blob = malloc(sizeof(uint32_t)*file_width[mip]*file_height[mip]);
 
   while(!feof(f))
   {
     int level = 0;
     rd = fread(&level, sizeof(int), 1, f);
     if (level > mip) break;
-    
+
     int32_t key = 0;
     rd = fread(&key, sizeof(int32_t), 1, f);
     if(rd != 1) break; // first value is break only, goes to eof.
     int32_t length = 0;
     rd = fread(&length, sizeof(int32_t), 1, f);
-//    fprintf(stderr, "[mipmap_cache] thumbnail for image %d length %d bytes (%d x %d) in level %d\n", get_imgid(key), length, file_width[level], file_height[level], level);
-    if(rd != 1 || length > 4*sizeof(uint8_t)*file_width[level]*file_height[level])
-      goto read_error;
-    rd = fread(blob, sizeof(uint8_t), length, f);
-    if(rd != length) goto read_error;
+    if(rd != 1) goto read_error;
 
-    dt_imageio_jpeg_t jpg;
     uint8_t *data = (uint8_t *)dt_cache_read_get(&cache->mip[level].cache, key);
-
     struct dt_mipmap_buffer_dsc* dsc = (struct dt_mipmap_buffer_dsc*)data;
     if(dsc->flags & DT_MIPMAP_BUFFER_DSC_FLAG_GENERATE)
     {
-      if(dt_imageio_jpeg_decompress_header(blob, length, &jpg) ||
-          (jpg.width > file_width[level] || jpg.height > file_height[level]) ||
-          dt_imageio_jpeg_decompress(&jpg, data+sizeof(*dsc)))
+      if(cache->compression_type)
       {
-        fprintf(stderr, "[mipmap_cache] failed to decompress thumbnail for image %d!\n", get_imgid(key));
+        int32_t wd, ht;
+        rd = fread(&wd, sizeof(int32_t), 1, f);
+        if(rd != 1) goto read_error;
+        rd = fread(&ht, sizeof(int32_t), 1, f);
+        if(rd != 1) goto read_error;
+        dsc->width = wd;
+        dsc->height = ht;
+        if(length != compressed_buffer_size(cache->compression_type, wd, ht)) goto read_error;
+        // directly read from disk into cache:
+        rd = fread(data + sizeof(*dsc), 1, length, f);
+        if(rd != length) goto read_error;
+      }
+      else
+      {
+        // jpg too large?
+        if(length > sizeof(uint32_t)*file_width[mip]*file_height[mip]) goto read_error;
+        rd = fread(blob, sizeof(uint8_t), length, f);
+        if(rd != length) goto read_error;
+        // no compression, the image is still compressed on disk, as jpg
+        dt_imageio_jpeg_t jpg;
+        if(dt_imageio_jpeg_decompress_header(blob, length, &jpg) ||
+            (jpg.width > file_width[level] || jpg.height > file_height[level]) ||
+            dt_imageio_jpeg_decompress(&jpg, data+sizeof(*dsc)))
+        {
+          fprintf(stderr, "[mipmap_cache] failed to decompress thumbnail for image %d!\n", get_imgid(key));
+        }
+        dsc->width = jpg.width;
+        dsc->height = jpg.height;
       }
-      dsc->width = jpg.width;
-      dsc->height = jpg.height;
       dsc->flags &= ~DT_MIPMAP_BUFFER_DSC_FLAG_GENERATE;
       // these come write locked in case idata[3] == 1, so release that!
       dt_cache_write_release(&cache->mip[level].cache, key);
@@ -392,6 +462,15 @@ read_finalize:
 static void _init_f(float   *buf, uint32_t *width, uint32_t *height, const uint32_t imgid);
 static void _init_8(uint8_t *buf, uint32_t *width, uint32_t *height, const uint32_t imgid, const dt_mipmap_size_t size);
 
+static int32_t
+scratchmem_allocate(void *data, const uint32_t key, int32_t *cost, void **buf)
+{
+  dt_mipmap_cache_one_t *c = (dt_mipmap_cache_one_t *)data;
+  // slot is exactly aligned with encapsulated cache's position and already allocated
+  *cost = c->buffer_size;
+  return 0;
+}
+
 int32_t
 dt_mipmap_cache_allocate(void *data, const uint32_t key, int32_t *cost, void **buf)
 {
@@ -429,16 +508,18 @@ dt_mipmap_cache_alloc(dt_image_t *img, dt_mipmap_size_t size, dt_mipmap_cache_al
 
   struct dt_mipmap_buffer_dsc** dsc = (struct dt_mipmap_buffer_dsc**)a;
 
-  const uint32_t buffer_size = 
-      (img->width * img->height * img->bpp) +
-      sizeof(**dsc);
-
+  int32_t wd = img->width;
+  int32_t ht = img->height;
+  int32_t bpp = img->bpp;
+  const uint32_t buffer_size =
+    ((wd*ht*bpp) + sizeof(**dsc));
 
   // buf might have been alloc'ed before,
   // so only check size and re-alloc if necessary:
   if(!(*dsc) || ((*dsc)->size < buffer_size) || ((void *)*dsc == (void *)dt_mipmap_cache_static_dead_image))
   {
-    free(*dsc);
+    if((void *)*dsc != (void *)dt_mipmap_cache_static_dead_image)
+      free(*dsc);
     *dsc = dt_alloc_align(64, buffer_size);
     // fprintf(stderr, "[mipmap cache] alloc for key %u %lX\n", get_key(img->id, size), (uint64_t)*buf);
     if(!(*dsc))
@@ -451,8 +532,8 @@ dt_mipmap_cache_alloc(dt_image_t *img, dt_mipmap_size_t size, dt_mipmap_cache_al
     // set buffer size only if we're making it larger.
     (*dsc)->size = buffer_size;
   }
-  (*dsc)->width = img->width;
-  (*dsc)->height = img->height;
+  (*dsc)->width = wd;
+  (*dsc)->height = ht;
   (*dsc)->flags = DT_MIPMAP_BUFFER_DSC_FLAG_GENERATE;
 
   // fprintf(stderr, "full buffer allocating img %u %d x %d = %u bytes (%lX)\n", img->id, img->width, img->height, buffer_size, (uint64_t)*buf);
@@ -465,12 +546,21 @@ dt_mipmap_cache_alloc(dt_image_t *img, dt_mipmap_size_t size, dt_mipmap_cache_al
 int32_t
 dt_mipmap_cache_allocate_dynamic(void *data, const uint32_t key, int32_t *cost, void **buf)
 {
+  dt_mipmap_cache_one_t *cache = (dt_mipmap_cache_one_t *)data;
   // for full image buffers
   struct dt_mipmap_buffer_dsc* dsc = *buf;
   // alloc mere minimum for the header + broken image buffer:
   if(!dsc)
   {
-    *buf = dt_alloc_align(16, sizeof(*dsc)+sizeof(float)*4*64);
+    if(cache->size == DT_MIPMAP_F)
+    {
+      // these are fixed-size:
+      *buf = dt_alloc_align(16, cache->buffer_size);
+    }
+    else
+    {
+      *buf = dt_alloc_align(16, sizeof(*dsc)+sizeof(float)*4*64);
+    }
     // fprintf(stderr, "[mipmap cache] alloc dynamic for key %u %lX\n", key, (uint64_t)*buf);
     if(!(*buf))
     {
@@ -478,9 +568,18 @@ dt_mipmap_cache_allocate_dynamic(void *data, const uint32_t key, int32_t *cost,
       exit(1);
     }
     dsc = *buf;
-    dsc->width = 0;
-    dsc->height = 0;
-    dsc->size = sizeof(*dsc)+sizeof(float)*4*64;
+    if(cache->size == DT_MIPMAP_F)
+    {
+      dsc->width = cache->max_width;
+      dsc->height = cache->max_height;
+      dsc->size = cache->buffer_size;
+    }
+    else
+    {
+      dsc->width = 0;
+      dsc->height = 0;
+      dsc->size = sizeof(*dsc)+sizeof(float)*4*64;
+    }
   }
   assert(dsc->size >= sizeof(*dsc));
   dsc->flags = DT_MIPMAP_BUFFER_DSC_FLAG_GENERATE;
@@ -492,13 +591,17 @@ dt_mipmap_cache_allocate_dynamic(void *data, const uint32_t key, int32_t *cost,
   return 1; // request write lock
 }
 
-#if 0
 void
 dt_mipmap_cache_deallocate_dynamic(void *data, const uint32_t key, void *payload)
 {
-  // don't clean up anything, as we are re-allocating. 
+  dt_mipmap_cache_one_t *cache = (dt_mipmap_cache_one_t *)data;
+  if(cache->size == DT_MIPMAP_F)
+  {
+    free(payload);
+  }
+  // else:
+  // don't clean up anything, as we are re-allocating.
 }
-#endif
 
 static uint32_t
 nearest_power_of_two(const uint32_t value)
@@ -514,9 +617,23 @@ void dt_mipmap_cache_init(dt_mipmap_cache_t *cache)
   struct dt_mipmap_buffer_dsc *dsc = (struct dt_mipmap_buffer_dsc *)dt_mipmap_cache_static_dead_image;
   dead_image_f((dt_mipmap_buffer_t *)(dsc+1));
 
+  cache->compression_type = 0;
+  gchar *compression = dt_conf_get_string("cache_compression");
+  if(compression)
+  {
+    if(!strcmp(compression, "low quality (fast)"))
+      cache->compression_type = 1;
+    else if(!strcmp(compression, "high quality (slow)"))
+      cache->compression_type = 2;
+    g_free(compression);
+  }
+
+  dt_print(DT_DEBUG_CACHE, "[mipmap_cache_init] using %s\n", cache->compression_type == 0 ? "no compression" :
+           (cache->compression_type == 1 ? "low quality compression" : "slow high quality compression"));
+
   // adjust numbers to be large enough to hold what mem limit suggests.
   // we want at least 100MB, and consider 2G just still reasonable.
-  const uint32_t max_mem = CLAMPS(dt_conf_get_int("cache_memory"), 100u<<20, 2u<<30)/5;
+  uint32_t max_mem = CLAMPS(dt_conf_get_int("cache_memory"), 100u<<20, 2u<<30);
   const uint32_t parallel = CLAMP(dt_conf_get_int ("worker_threads"), 1, 8);
   const int32_t max_size = 2048, min_size = 32;
   int32_t wd = darktable.thumbnail_width;
@@ -531,92 +648,144 @@ void dt_mipmap_cache_init(dt_mipmap_cache_t *cache)
   cache->mip[DT_MIPMAP_F].max_height = ht;
   cache->mip[DT_MIPMAP_F-1].max_width  = wd;
   cache->mip[DT_MIPMAP_F-1].max_height = ht;
-  for(int k=DT_MIPMAP_F-2;k>=DT_MIPMAP_0;k--)
+  for(int k=DT_MIPMAP_F-2; k>=DT_MIPMAP_0; k--)
   {
     cache->mip[k].max_width  = cache->mip[k+1].max_width  / 2;
     cache->mip[k].max_height = cache->mip[k+1].max_height / 2;
   }
 
-  for(int k=0;k<=DT_MIPMAP_F;k++)
+  // initialize some per-thread cached scratchmem for uncompressed buffers during thumb creation:
+  if(cache->compression_type)
+  {
+    cache->scratchmem.max_width = wd;
+    cache->scratchmem.max_height = ht;
+    cache->scratchmem.buffer_size = wd*ht*sizeof(uint32_t);
+    cache->scratchmem.size = DT_MIPMAP_3; // at max.
+    // TODO: use thread local storage instead (zero performance penalty on linux)
+    dt_cache_init(&cache->scratchmem.cache, parallel, parallel, 64, 0.9f*parallel*wd*ht*sizeof(uint32_t));
+    // might have been rounded to power of two:
+    const int cnt = dt_cache_capacity(&cache->scratchmem.cache);
+    cache->scratchmem.buf = dt_alloc_align(64, cnt * wd*ht*sizeof(uint32_t));
+    dt_cache_static_allocation(&cache->scratchmem.cache, (uint8_t *)cache->scratchmem.buf, wd*ht*sizeof(uint32_t));
+    dt_cache_set_allocate_callback(&cache->scratchmem.cache,
+                                   scratchmem_allocate, &cache->scratchmem);
+    dt_print(DT_DEBUG_CACHE,
+             "[mipmap_cache_init] cache has % 5d entries for temporary compression buffers (% 4.02f MB).\n",
+             cnt, cnt* wd*ht*sizeof(uint32_t)/(1024.0*1024.0));
+  }
+
+  for(int k=DT_MIPMAP_3; k>=0; k--)
   {
     // buffer stores width and height + actual data
     const int width  = cache->mip[k].max_width;
     const int height = cache->mip[k].max_height;
-    if(k == DT_MIPMAP_F)
-      cache->mip[k].buffer_size = (4 + 4 * width * height)*sizeof(float);
-    else
-      cache->mip[k].buffer_size = (4 + width * height)*sizeof(uint32_t);
+    // header + adjusted for dxt compression:
+    cache->mip[k].buffer_size = 4*sizeof(uint32_t) + compressed_buffer_size(cache->compression_type, width, height);
     cache->mip[k].size = k;
     // level of parallelism also gives minimum size (which is twice that)
     // is rounded to a power of two by the cache anyways, we might as well.
-    uint32_t thumbnails = MAX(2*parallel, nearest_power_of_two((uint32_t)((float)max_mem/cache->mip[k].buffer_size)));
-    while(thumbnails > 2*parallel && thumbnails * cache->mip[k].buffer_size > max_mem) thumbnails /= 2;
+    const uint32_t max_mem2 = MAX(0, (k == 0) ? (max_mem) : (max_mem/(k+4)));
+    uint32_t thumbnails = MAX(2, nearest_power_of_two((uint32_t)((float)max_mem2/cache->mip[k].buffer_size)));
+    while(thumbnails > parallel && thumbnails * cache->mip[k].buffer_size > max_mem2) thumbnails /= 2;
 
     // try to utilize that memory well (use 90% quota), the hopscotch paper claims good scalability up to
     // even more than that.
-    dt_cache_init(&cache->mip[k].cache, thumbnails, parallel, 64, 0.9f*thumbnails*cache->mip[k].buffer_size);
+    dt_cache_init(&cache->mip[k].cache, thumbnails,
+                  parallel,
+                  64, 0.9f*thumbnails*cache->mip[k].buffer_size);
 
     // might have been rounded to power of two:
     thumbnails = dt_cache_capacity(&cache->mip[k].cache);
+    max_mem -= thumbnails * cache->mip[k].buffer_size;
+    // dt_print(DT_DEBUG_CACHE, "[mipmap mem] %4.02f left\n", max_mem/(1024.0*1024.0));
     cache->mip[k].buf = dt_alloc_align(64, thumbnails * cache->mip[k].buffer_size);
     dt_cache_static_allocation(&cache->mip[k].cache, (uint8_t *)cache->mip[k].buf, cache->mip[k].buffer_size);
     dt_cache_set_allocate_callback(&cache->mip[k].cache,
-        dt_mipmap_cache_allocate, &cache->mip[k]);
+                                   dt_mipmap_cache_allocate, &cache->mip[k]);
     // dt_cache_set_cleanup_callback(&cache->mip[k].cache,
-        // &dt_mipmap_cache_deallocate, &cache->mip[k]);
-
+    // &dt_mipmap_cache_deallocate, &cache->mip[k]);
 
     dt_print(DT_DEBUG_CACHE,
-        "[mipmap_cache_init] cache has % 5d entries for mip %d (% 4.02f MB).\n",
-        thumbnails, k, thumbnails * cache->mip[k].buffer_size/(1024.0*1024.0));
+             "[mipmap_cache_init] cache has % 5d entries for mip %d (% 4.02f MB).\n",
+             thumbnails, k, thumbnails * cache->mip[k].buffer_size/(1024.0*1024.0));
   }
+
   // full buffer needs dynamic alloc:
-  const int full_entries = 2*parallel;
+  const int full_entries = MAX(2, parallel); // even with one thread you want two buffers. one for dr one for thumbs.
   int32_t max_mem_bufs = nearest_power_of_two(full_entries);
 
   // for this buffer, because it can be very busy during import, we want the minimum
   // number of entries in the hashtable to be 16, but leave the quota as is. the dynamic
   // alloc/free properties of this cache take care that no more memory is required.
-  dt_cache_init(&cache->mip[DT_MIPMAP_FULL].cache, MAX(16, max_mem_bufs), parallel, 64, 0.9f*max_mem_bufs);
+  dt_cache_init(&cache->mip[DT_MIPMAP_FULL].cache, max_mem_bufs, parallel, 64, max_mem_bufs);
   dt_cache_set_allocate_callback(&cache->mip[DT_MIPMAP_FULL].cache,
-      dt_mipmap_cache_allocate_dynamic, &cache->mip[DT_MIPMAP_FULL]);
+                                 dt_mipmap_cache_allocate_dynamic, &cache->mip[DT_MIPMAP_FULL]);
   // dt_cache_set_cleanup_callback(&cache->mip[DT_MIPMAP_FULL].cache,
-      // &dt_mipmap_cache_deallocate_dynamic, &cache->mip[DT_MIPMAP_FULL]);
+  // &dt_mipmap_cache_deallocate_dynamic, &cache->mip[DT_MIPMAP_FULL]);
   cache->mip[DT_MIPMAP_FULL].buffer_size = 0;
   cache->mip[DT_MIPMAP_FULL].size = DT_MIPMAP_FULL;
   cache->mip[DT_MIPMAP_FULL].buf = NULL;
 
+  // same for mipf:
+  dt_cache_init(&cache->mip[DT_MIPMAP_F].cache, max_mem_bufs, parallel, 64, max_mem_bufs);
+  dt_cache_set_allocate_callback(&cache->mip[DT_MIPMAP_F].cache,
+                                 dt_mipmap_cache_allocate_dynamic, &cache->mip[DT_MIPMAP_F]);
+  dt_cache_set_cleanup_callback(&cache->mip[DT_MIPMAP_F].cache,
+                                dt_mipmap_cache_deallocate_dynamic, &cache->mip[DT_MIPMAP_F]);
+  cache->mip[DT_MIPMAP_F].buffer_size = 4*sizeof(uint32_t) +
+                                        4*sizeof(float) * cache->mip[DT_MIPMAP_F].max_width * cache->mip[DT_MIPMAP_F].max_height;
+  cache->mip[DT_MIPMAP_F].size = DT_MIPMAP_F;
+  cache->mip[DT_MIPMAP_F].buf = NULL;
+
   dt_mipmap_cache_deserialize(cache);
 }
 
 void dt_mipmap_cache_cleanup(dt_mipmap_cache_t *cache)
 {
   dt_mipmap_cache_serialize(cache);
-  for(int k=0;k<=DT_MIPMAP_F;k++)
+  for(int k=0; k<DT_MIPMAP_F; k++)
   {
     dt_cache_cleanup(&cache->mip[k].cache);
     // now mem is actually freed, not during cache cleanup
     free(cache->mip[k].buf);
   }
   dt_cache_cleanup(&cache->mip[DT_MIPMAP_FULL].cache);
+  dt_cache_cleanup(&cache->mip[DT_MIPMAP_F].cache);
+
+  // clean up temporary buffers for decompressed images, if any:
+  if(cache->compression_type)
+  {
+    dt_cache_cleanup(&cache->scratchmem.cache);
+    free(cache->scratchmem.buf);
+  }
 }
 
 void dt_mipmap_cache_print(dt_mipmap_cache_t *cache)
 {
-  for(int k=0; k<(int)DT_MIPMAP_FULL; k++)
+  for(int k=0; k<(int)DT_MIPMAP_F; k++)
   {
     printf("[mipmap_cache] level %d fill %.2f/%.2f MB (%.2f%% in %u/%u buffers)\n", k, cache->mip[k].cache.cost/(1024.0*1024.0),
-      cache->mip[k].cache.cost_quota/(1024.0*1024.0),
-      100.0f*(float)cache->mip[k].cache.cost/(float)cache->mip[k].cache.cost_quota,
-      dt_cache_size(&cache->mip[k].cache),
-      dt_cache_capacity(&cache->mip[k].cache));
+           cache->mip[k].cache.cost_quota/(1024.0*1024.0),
+           100.0f*(float)cache->mip[k].cache.cost/(float)cache->mip[k].cache.cost_quota,
+           dt_cache_size(&cache->mip[k].cache),
+           dt_cache_capacity(&cache->mip[k].cache));
+  }
+  for(int k=(int)DT_MIPMAP_F; k<=(int)DT_MIPMAP_FULL; k++)
+  {
+    printf("[mipmap_cache] level [f%d] fill %d/%d slots (%.2f%% in %u/%u buffers)\n", k, cache->mip[k].cache.cost,
+           cache->mip[k].cache.cost_quota,
+           100.0f*(float)cache->mip[k].cache.cost/(float)cache->mip[k].cache.cost_quota,
+           dt_cache_size(&cache->mip[k].cache),
+           dt_cache_capacity(&cache->mip[k].cache));
+  }
+  if(cache->compression_type)
+  {
+    printf("[mipmap_cache] scratch fill %.2f/%.2f MB (%.2f%% in %u/%u buffers)\n", cache->scratchmem.cache.cost/(1024.0*1024.0),
+           cache->scratchmem.cache.cost_quota/(1024.0*1024.0),
+           100.0f*(float)cache->scratchmem.cache.cost/(float)cache->scratchmem.cache.cost_quota,
+           dt_cache_size(&cache->scratchmem.cache),
+           dt_cache_capacity(&cache->scratchmem.cache));
   }
-  const int k = DT_MIPMAP_FULL;
-  printf("[mipmap_cache] level [full] fill %d/%d slots (%.2f%% in %u/%u buffers)\n", cache->mip[k].cache.cost,
-    cache->mip[k].cache.cost_quota,
-    100.0f*(float)cache->mip[k].cache.cost/(float)cache->mip[k].cache.cost_quota,
-    dt_cache_size(&cache->mip[k].cache),
-    dt_cache_capacity(&cache->mip[k].cache));
   printf("\n\n");
   // very verbose stats about locks/users
   //dt_cache_print(&cache->mip[DT_MIPMAP_3].cache);
@@ -624,11 +793,11 @@ void dt_mipmap_cache_print(dt_mipmap_cache_t *cache)
 
 void
 dt_mipmap_cache_read_get(
-    dt_mipmap_cache_t *cache,
-    dt_mipmap_buffer_t *buf,
-    const uint32_t imgid,
-    const dt_mipmap_size_t mip,
-    const dt_mipmap_get_flags_t flags)
+  dt_mipmap_cache_t *cache,
+  dt_mipmap_buffer_t *buf,
+  const uint32_t imgid,
+  const dt_mipmap_size_t mip,
+  const dt_mipmap_get_flags_t flags)
 {
   const uint32_t key = get_key(imgid, mip);
   if(flags == DT_MIPMAP_TESTLOCK)
@@ -670,7 +839,7 @@ dt_mipmap_cache_read_get(
     if(!dsc)
     {
       // should never happen for anything but full images which have been moved.
-      assert(mip == DT_MIPMAP_FULL);
+      assert(mip == DT_MIPMAP_FULL || mip == DT_MIPMAP_F);
       // fprintf(stderr, "[mipmap cache get] no data in cache for imgid %u size %d!\n", imgid, mip);
       // sorry guys, no image for you :(
       buf->width = buf->height = 0;
@@ -714,7 +883,7 @@ dt_mipmap_cache_read_get(
           if(ret != DT_IMAGEIO_OK)
           {
             // fprintf(stderr, "[mipmap read get] error loading image: %d\n", ret);
-            // 
+            //
             // we can only return a zero dimension buffer if the buffer has been allocated.
             // in case dsc couldn't be allocated and points to the static buffer, it contains
             // a dead image already.
@@ -733,9 +902,34 @@ dt_mipmap_cache_read_get(
           }
         }
         else if(mip == DT_MIPMAP_F)
+        {
           _init_f((float *)(dsc+1), &dsc->width, &dsc->height, imgid);
+        }
         else
-          _init_8((uint8_t *)(dsc+1), &dsc->width, &dsc->height, imgid, mip);
+        {
+          // 8-bit thumbs, possibly need to be compressed:
+          if(cache->compression_type)
+          {
+            // get per-thread temporary storage without malloc from a separate cache:
+            const int key = dt_control_get_threadid();
+            // const void *cbuf =
+            dt_cache_read_get(&cache->scratchmem.cache, key);
+            uint8_t *scratchmem = (uint8_t *)dt_cache_write_get(&cache->scratchmem.cache, key);
+            _init_8(scratchmem, &dsc->width, &dsc->height, imgid, mip);
+            buf->width  = dsc->width;
+            buf->height = dsc->height;
+            buf->imgid  = imgid;
+            buf->size   = mip;
+            buf->buf = (uint8_t *)(dsc+1);
+            dt_mipmap_cache_compress(buf, scratchmem);
+            dt_cache_write_release(&cache->scratchmem.cache, key);
+            dt_cache_read_release(&cache->scratchmem.cache, key);
+          }
+          else
+          {
+            _init_8((uint8_t *)(dsc+1), &dsc->width, &dsc->height, imgid, mip);
+          }
+        }
         dsc->flags &= ~DT_MIPMAP_BUFFER_DSC_FLAG_GENERATE;
         // drop the write lock
         dt_cache_write_release(&cache->mip[mip].cache, key);
@@ -761,7 +955,7 @@ dt_mipmap_cache_read_get(
     // best-effort, might also return NULL.
     // never decrease mip level for float buffer or full image:
     dt_mipmap_size_t min_mip = (mip >= DT_MIPMAP_F) ? mip : DT_MIPMAP_0;
-    for(int k=mip;k>=min_mip && k>=0;k--)
+    for(int k=mip; k>=min_mip && k>=0; k--)
     {
       // already loaded?
       dt_mipmap_cache_read_get(cache, buf, imgid, k, DT_MIPMAP_TESTLOCK);
@@ -781,8 +975,8 @@ dt_mipmap_cache_read_get(
 
 void
 dt_mipmap_cache_write_get(
-    dt_mipmap_cache_t *cache,
-    dt_mipmap_buffer_t *buf)
+  dt_mipmap_cache_t *cache,
+  dt_mipmap_buffer_t *buf)
 {
   assert(buf->imgid > 0);
   assert(buf->size >= DT_MIPMAP_0);
@@ -799,10 +993,10 @@ dt_mipmap_cache_write_get(
 
 void
 dt_mipmap_cache_read_release(
-    dt_mipmap_cache_t *cache,
-    dt_mipmap_buffer_t *buf)
+  dt_mipmap_cache_t *cache,
+  dt_mipmap_buffer_t *buf)
 {
-  if(buf->size == DT_MIPMAP_NONE || buf->buf == NULL) return;
+  if(buf->size == DT_MIPMAP_NONE) return;
   assert(buf->imgid > 0);
   assert(buf->size >= DT_MIPMAP_0);
   assert(buf->size <  DT_MIPMAP_NONE);
@@ -814,8 +1008,8 @@ dt_mipmap_cache_read_release(
 // drop a write lock, read will still remain.
 void
 dt_mipmap_cache_write_release(
-    dt_mipmap_cache_t *cache,
-    dt_mipmap_buffer_t *buf)
+  dt_mipmap_cache_t *cache,
+  dt_mipmap_buffer_t *buf)
 {
   if(buf->size == DT_MIPMAP_NONE || buf->buf == NULL) return;
   assert(buf->imgid > 0);
@@ -831,17 +1025,17 @@ dt_mipmap_cache_write_release(
 // return the closest mipmap size
 dt_mipmap_size_t
 dt_mipmap_cache_get_matching_size(
-    const dt_mipmap_cache_t *cache,
-    const int32_t width,
-    const int32_t height)
+  const dt_mipmap_cache_t *cache,
+  const int32_t width,
+  const int32_t height)
 {
   // find `best' match to width and height.
   uint32_t error = 0xffffffff;
   dt_mipmap_size_t best = DT_MIPMAP_NONE;
-  for(int k=DT_MIPMAP_0;k<DT_MIPMAP_F;k++)
+  for(int k=DT_MIPMAP_0; k<DT_MIPMAP_F; k++)
   {
     uint32_t new_error = abs(cache->mip[k].max_width + cache->mip[k].max_height
-                       - width - height);
+                             - width - height);
     if(new_error < error)
     {
       best = k;
@@ -853,11 +1047,11 @@ dt_mipmap_cache_get_matching_size(
 
 void
 dt_mipmap_cache_remove(
-    dt_mipmap_cache_t *cache,
-    const uint32_t imgid)
+  dt_mipmap_cache_t *cache,
+  const uint32_t imgid)
 {
   // get rid of all ldr thumbnails:
-  for(int k=DT_MIPMAP_0;k<DT_MIPMAP_F;k++)
+  for(int k=DT_MIPMAP_0; k<DT_MIPMAP_F; k++)
   {
     const uint32_t key = get_key(imgid, k);
     dt_cache_remove(&cache->mip[k].cache, key);
@@ -866,10 +1060,10 @@ dt_mipmap_cache_remove(
 
 static void
 _init_f(
-    float          *out,
-    uint32_t       *width,
-    uint32_t       *height,
-    const uint32_t  imgid)
+  float          *out,
+  uint32_t       *width,
+  uint32_t       *height,
+  const uint32_t  imgid)
 {
   const uint32_t wd = *width, ht = *height;
 
@@ -894,7 +1088,7 @@ _init_f(
   roi_in.width = image->width;
   roi_in.height = image->height;
   roi_in.scale = 1.0f;
-  
+
   roi_out.x = roi_out.y = 0;
   roi_out.scale = fminf(wd/(float)image->width, ht/(float)image->height);
   roi_out.width  = roi_out.scale * roi_in.width;
@@ -915,20 +1109,20 @@ _init_f(
     // demosaic during downsample
     if(image->bpp == sizeof(float))
       dt_iop_clip_and_zoom_demosaic_half_size_f(
-          out, (const float *)buf.buf,
-          &roi_out, &roi_in, roi_out.width, roi_in.width,
-          dt_image_flipped_filter(image), 1.0f);
+        out, (const float *)buf.buf,
+        &roi_out, &roi_in, roi_out.width, roi_in.width,
+        dt_image_flipped_filter(image), 1.0f);
     else
       dt_iop_clip_and_zoom_demosaic_half_size(
-          out, (const uint16_t *)buf.buf,
-          &roi_out, &roi_in, roi_out.width, roi_in.width,
-          dt_image_flipped_filter(image));
+        out, (const uint16_t *)buf.buf,
+        &roi_out, &roi_in, roi_out.width, roi_in.width,
+        dt_image_flipped_filter(image));
   }
   else
   {
     // downsample
     dt_iop_clip_and_zoom(out, (const float *)buf.buf,
-          &roi_out, &roi_in, roi_out.width, roi_in.width);
+                         &roi_out, &roi_in, roi_out.width, roi_in.width);
   }
   dt_image_cache_read_release(darktable.image_cache, image);
   dt_mipmap_cache_read_release(darktable.mipmap_cache, &buf);
@@ -954,31 +1148,31 @@ _bpp(dt_imageio_module_data_t *data)
 
 static int
 _write_image(
-    dt_imageio_module_data_t *data,
-    const char               *filename,
-    const void               *in,
-    void                     *exif,
-    int                       exif_len,
-    int                       imgid)
+  dt_imageio_module_data_t *data,
+  const char               *filename,
+  const void               *in,
+  void                     *exif,
+  int                       exif_len,
+  int                       imgid)
 {
   _dummy_data_t *d = (_dummy_data_t *)data;
   memcpy(d->buf, in, data->width*data->height*sizeof(uint32_t));
   return 0;
 }
 
-static void 
+static void
 _init_8(
-    uint8_t                *buf,
-    uint32_t               *width,
-    uint32_t               *height,
-    const uint32_t          imgid,
-    const dt_mipmap_size_t  size)
+  uint8_t                *buf,
+  uint32_t               *width,
+  uint32_t               *height,
+  const uint32_t          imgid,
+  const dt_mipmap_size_t  size)
 {
-  const uint32_t wd = *width, ht = *height;  
+  const uint32_t wd = *width, ht = *height;
+  char filename[DT_MAX_PATH_LEN] = {0};
 
   /* do not even try to process file if it isnt available */
-  char filename[2048] = {0};
-  dt_image_full_path(imgid, filename, 2048);
+  dt_image_full_path(imgid, filename, DT_MAX_PATH_LEN);
   if (strlen(filename) == 0 || !g_file_test(filename, G_FILE_TEST_EXISTS))
   {
     *width = *height = 0;
@@ -999,7 +1193,7 @@ _init_8(
   if(!altered && !dt_conf_get_bool("never_use_embedded_thumb") && !incompatible)
   {
     int ret;
-    char filename[DT_MAX_PATH_LEN];
+    memset(filename, 0, DT_MAX_PATH_LEN);
     dt_image_full_path(imgid, filename, DT_MAX_PATH_LEN);
     const char *c = filename + strlen(filename);
     while(*c != '.' && c > filename) c--;
@@ -1051,12 +1245,6 @@ _init_8(
         free(tmp);
         res = 0;
       }
-      else
-      {
-        // bmp
-        dt_iop_flip_and_zoom_8(image->data, image->width, image->height, buf, wd, ht, orientation, width, height);
-        res = 0;
-      }
 
       // clean up raw stuff.
       libraw_recycle(raw);
@@ -1108,3 +1296,61 @@ libraw_fail:
   // TODO: if output is cropped, don't use mipf!
 }
 
+// compression stuff: alloc a buffer if needed
+uint8_t*
+dt_mipmap_cache_alloc_scratchmem(
+  const dt_mipmap_cache_t *cache)
+{
+  const size_t size = cache->mip[DT_MIPMAP_3].max_width *
+                      cache->mip[DT_MIPMAP_3].max_height;
+
+  if(cache->compression_type)
+  {
+    return dt_alloc_align(64, size * 4 * sizeof(uint8_t));
+  }
+  else // no compression, no buffer:
+    return NULL;
+}
+
+// decompress the raw mipmapm buffer into the scratchmemory.
+// returns a pointer to the decompressed memory block. that's because
+// for uncompressed settings, it will point directly to the mipmap
+// buffer and scratchmem can be NULL.
+uint8_t*
+dt_mipmap_cache_decompress(
+  const dt_mipmap_buffer_t *buf,
+  uint8_t *scratchmem)
+{
+  if(darktable.mipmap_cache->compression_type && buf->width > 8 && buf->height > 8)
+  {
+    squish_decompress_image(scratchmem, buf->width, buf->height, buf->buf, squish_dxt1);
+    return scratchmem;
+  }
+  else
+  {
+    return buf->buf;
+  }
+}
+
+// writes the scratchmem buffer to compressed
+// format into the mipmap cache. does nothing
+// if compression is disabled.
+void
+dt_mipmap_cache_compress(
+  dt_mipmap_buffer_t *buf,
+  uint8_t *const scratchmem)
+{
+  // only do something if compression is on, don't compress skulls:
+  if(darktable.mipmap_cache->compression_type && buf->width > 8 && buf->height > 8)
+  {
+    int flags = squish_dxt1;
+    // low quality:
+    if(darktable.mipmap_cache->compression_type == 1) flags |= squish_colour_range_fit;
+    squish_compress_image(scratchmem, buf->width, buf->height, buf->buf, squish_dxt1);
+  }
+}
+
+
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
