@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010 - 2011 Henrik Andersson.
+    copyright (c) 2010 - 2012 Henrik Andersson.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ int32_t dt_captured_image_import_job_run(dt_job_t *job)
     dt_control_queue_redraw();
     //dt_ctl_switch_mode_to(DT_DEVELOP);
   }
- 
+
   dt_control_backgroundjobs_progress(darktable.control, jid, 1.0);
   dt_control_backgroundjobs_destroy(darktable.control, jid);
   return 0;
@@ -65,12 +65,12 @@ int32_t dt_camera_capture_job_run(dt_job_t *job)
   char message[512]= {0};
   double fraction=0;
   snprintf(message, 512, ngettext ("capturing %d image", "capturing %d images", total), total );
-  
+
   /* try to get exp program mode for nikon */
   char *expprogram = (char *)dt_camctl_camera_get_property(darktable.camctl, NULL, "expprogram");
-  
+
   /* if fail, lets try fetching mode for cannon */
-  if(!expprogram) 
+  if(!expprogram)
     expprogram = (char *)dt_camctl_camera_get_property(darktable.camctl, NULL, "autoexposuremode");
 
   /* Fetch all values for shutterspeed and initialize current value */
@@ -78,7 +78,7 @@ int32_t dt_camera_capture_job_run(dt_job_t *job)
   gconstpointer orginal_value=NULL;
   const char *cvalue = dt_camctl_camera_get_property(darktable.camctl, NULL, "shutterspeed");
   const char *value = dt_camctl_camera_property_get_first_choice(darktable.camctl, NULL, "shutterspeed");
-  
+
   /* get values for bracketing */
   if (t->brackets && expprogram && expprogram[0]=='M' && value && cvalue)
   {
@@ -291,7 +291,7 @@ void _camera_image_downloaded(const dt_camera_t *camera,const char *filename,voi
   dt_control_log(_("%d/%d imported to %s"), t->import_count+1,g_list_length(t->images), g_path_get_basename(filename));
 
   t->fraction+=1.0/g_list_length(t->images);
-  
+
   dt_control_backgroundjobs_progress(darktable.control, t->bgj, t->fraction );
 
   if( dt_conf_get_bool("plugins/capture/camera/import/backup/enable") == TRUE )
@@ -322,25 +322,53 @@ const char *_camera_import_request_image_filename(const dt_camera_t *camera,cons
   g_free(t->path);
   t->path = fixed_path;
   dt_variables_expand( t->vp, t->path, FALSE );
-  const gchar *storage=dt_variables_get_result(t->vp);
+  gchar *storage = dt_variables_get_result(t->vp);
 
   dt_variables_expand( t->vp, t->filename, TRUE );
-  const gchar *file = dt_variables_get_result(t->vp);
+  gchar *file = dt_variables_get_result(t->vp);
 
   // Start check if file exist if it does, increase sequence and check again til we know that file doesnt exists..
-  gchar *fullfile=g_build_path(G_DIR_SEPARATOR_S,storage,file,(char *)NULL);
+  gchar *prev_filename;
+  gchar *fullfile;
+  prev_filename = fullfile = g_build_path(G_DIR_SEPARATOR_S,
+					  storage, file,
+					  (char *)NULL);
+
   if( g_file_test(fullfile, G_FILE_TEST_EXISTS) == TRUE )
   {
     do
     {
-      g_free(fullfile);
       dt_variables_expand( t->vp, t->filename, TRUE );
+      g_free(file);
       file = dt_variables_get_result(t->vp);
-      fullfile=g_build_path(G_DIR_SEPARATOR_S,storage,file,(char *)NULL);
+      fullfile = g_build_path(G_DIR_SEPARATOR_S, storage, file, (char *)NULL);
+
+      // if we expanded to same filename the variables are wrong and ${SEQUENCE}
+      // is probably missing...
+      if (strcmp(prev_filename, fullfile) == 0)
+      {
+	if (prev_filename != fullfile)
+	  g_free(prev_filename);
+
+	g_free(fullfile);
+	g_free(storage);
+
+	dt_control_log(_("Couldn't expand to a uniq filename for import, please check your import settings."));
+
+	return NULL;
+      }
+
+      g_free(prev_filename);
+      prev_filename = fullfile;
     }
     while( g_file_test(fullfile, G_FILE_TEST_EXISTS) == TRUE);
   }
 
+  if (prev_filename != fullfile)
+    g_free(prev_filename);
+
+  g_free(fullfile);
+  g_free(storage);
   return file;
 }
 
@@ -413,4 +441,6 @@ int32_t dt_camera_import_job_run(dt_job_t *job)
   return 0;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
