@@ -1,6 +1,7 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2011 johannes hanika.
+    copyright (c) 2009--2010 johannes hanika.
+    copyright (c) 2011 henrik andersson.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -56,7 +57,7 @@ static char *_pipe_type_to_str(int pipe_type)
     case DT_DEV_PIXELPIPE_EXPORT:
     default:
       r = "export";
-    break;
+      break;
   }
   return r;
 }
@@ -75,6 +76,13 @@ int dt_dev_pixelpipe_init_thumbnail(dt_dev_pixelpipe_t *pipe, int32_t width, int
   return res;
 }
 
+int dt_dev_pixelpipe_init_preview(dt_dev_pixelpipe_t *pipe)
+{
+  int res = dt_dev_pixelpipe_init_cached(pipe, 4*sizeof(float)*darktable.thumbnail_width*darktable.thumbnail_height, 5);
+  pipe->type = DT_DEV_PIXELPIPE_PREVIEW;
+  return res;
+}
+
 int dt_dev_pixelpipe_init(dt_dev_pixelpipe_t *pipe)
 {
   int res = dt_dev_pixelpipe_init_cached(pipe, 4*sizeof(float)*darktable.thumbnail_width*darktable.thumbnail_height, 5);
@@ -92,10 +100,13 @@ int dt_dev_pixelpipe_init_cached(dt_dev_pixelpipe_t *pipe, int32_t size, int32_t
   pipe->backbuf_size = size;
   if(!dt_dev_pixelpipe_cache_init(&(pipe->cache), entries, pipe->backbuf_size))
     return 0;
+  pipe->cache_obsolete = 0;
   pipe->backbuf = NULL;
   pipe->processing = 0;
   pipe->shutdown = 0;
   pipe->opencl_error = 0;
+  pipe->tiling = 0;
+  pipe->mask_display = 0;
   pipe->input_timestamp = 0;
   dt_pthread_mutex_init(&(pipe->backbuf_mutex), NULL);
   dt_pthread_mutex_init(&(pipe->busy_mutex), NULL);
@@ -106,10 +117,10 @@ void dt_dev_pixelpipe_set_input(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, flo
 {
   pipe->iwidth  = width;
   pipe->iheight = height;
+  pipe->iflipped = 0;
   pipe->iscale = iscale;
   pipe->input = input;
   pipe->image = dev->image_storage;
-  if(width < pipe->image.width && height < pipe->image.height) pipe->type = DT_DEV_PIXELPIPE_PREVIEW;
 }
 
 void dt_dev_pixelpipe_cleanup(dt_dev_pixelpipe_t *pipe)
@@ -211,7 +222,7 @@ void dt_dev_pixelpipe_synch_all(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev)
   }
   // go through all history items and adjust params
   GList *history = dev->history;
-  for(int k=0; k<dev->history_end; k++)
+  for(int k=0; k<dev->history_end && history; k++)
   {
     dt_dev_pixelpipe_synch(pipe, dev, history);
     history = g_list_next(history);
@@ -275,6 +286,59 @@ get_output_bpp(dt_iop_module_t *module, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpi
 }
 
 
+// helper for color picking
+static void
+pixelpipe_picker(dt_iop_module_t *module, const float *img, const dt_iop_roi_t *roi,
+                 float *picked_color, float *picked_color_min, float *picked_color_max)
+{
+  int box[4];
+  int point[2];
+  float Lab[3];
+
+  for(int k=0; k<3; k++) picked_color_min[k] =  666.0f;
+  for(int k=0; k<3; k++) picked_color_max[k] = -666.0f;
+
+  for(int k=0; k<3; k++) Lab[k] = 0.0f;
+
+  // Initializing bounds of colorpicker box
+  for(int k=0; k<4; k+=2) box[k] = MIN(roi->width -1, MAX(0, module->color_picker_box[k]*roi->width));
+  for(int k=1; k<4; k+=2) box[k] = MIN(roi->height-1, MAX(0, module->color_picker_box[k]*roi->height));
+
+  // Initializing bounds of colorpicker point
+  point[0] = MIN(roi->width - 1, MAX(0, module->color_picker_point[0] * roi->width));
+  point[1] = MIN(roi->height - 1, MAX(0, module->color_picker_point[1] * roi->height));
+
+  if(darktable.lib->proxy.colorpicker.size)
+  {
+    const float w = 1.0/((box[3]-box[1]+1)*(box[2]-box[0]+1));
+    for(int j=box[1]; j<=box[3]; j++) for(int i=box[0]; i<=box[2]; i++)
+      {
+        const float L = img[4*(roi->width*j + i) + 0];
+        const float a = img[4*(roi->width*j + i) + 1];
+        const float b = img[4*(roi->width*j + i) + 2];
+        Lab[0] += w*L;
+        Lab[1] += w*a;
+        Lab[2] += w*b;
+        picked_color_min[0] = fminf(picked_color_min[0], L);
+        picked_color_min[1] = fminf(picked_color_min[1], a);
+        picked_color_min[2] = fminf(picked_color_min[2], b);
+        picked_color_max[0] = fmaxf(picked_color_max[0], L);
+        picked_color_max[1] = fmaxf(picked_color_max[1], a);
+        picked_color_max[2] = fmaxf(picked_color_max[2], b);
+        for(int k=0; k<3; k++) picked_color[k] = Lab[k];
+      }
+  }
+  else
+  {
+    for(int i = 0; i < 3; i++)
+      picked_color[i]
+      = picked_color_min[i]
+        = picked_color_max[i]
+          = img[4*(roi->width*point[1] + point[0]) + i];
+  }
+}
+
+
 // recursive helper for process:
 static int
 dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void **output, void **cl_mem_output, int *out_bpp,
@@ -466,7 +530,6 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
           histogram_pre[4*L+3] ++;
         }
       // don't count <= 0 pixels
-      for(int k=3; k<4*64; k+=4) histogram_pre[k] = logf(1.0 +histogram_pre[k]);
       for(int k=19; k<4*64; k+=4) *histogram_pre_max = *histogram_pre_max > histogram_pre[k] ? *histogram_pre_max : histogram_pre[k];
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
       if(module->widget) dt_control_queue_redraw_widget(module->widget);
@@ -481,79 +544,64 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
       return 1;
     }
 
+#if 0
     // Lab color picking for module
     if(dev->gui_attached && pipe == dev->preview_pipe && // pick from preview pipe to get pixels outside the viewport
         (module == dev->gui_module || !strcmp(module->op, "colorout")) && // only modules with focus or colorout for bottom panel can pick
         module->request_color_pick) // and they need to want to pick ;)
     {
-      for(int k=0; k<3; k++) module->picked_color_min[k] =  666.0f;
-      for(int k=0; k<3; k++) module->picked_color_max[k] = -666.0f;
-      int box[4];
-      int point[2];
-      float Lab[3], *in = (float *)input;
-      for(int k=0; k<3; k++) Lab[k] = 0.0f;
-
-      // Initializing bounds of colorpicker box
-      for(int k=0; k<4; k+=2) box[k] = MIN(roi_in.width -1, MAX(0, module->color_picker_box[k]*roi_in.width));
-      for(int k=1; k<4; k+=2) box[k] = MIN(roi_in.height-1, MAX(0, module->color_picker_box[k]*roi_in.height));
-
-      // Initializing bounds of colorpicker point
-      point[0] = MIN(roi_in.width - 1, MAX(0, module->color_picker_point[0] * roi_in.width));
-      point[1] = MIN(roi_in.height - 1, MAX(0, module->color_picker_point[1] * roi_in.height));
-
-      if(darktable.lib->proxy.colorpicker.size)
-      {
-        const float w = 1.0/((box[3]-box[1]+1)*(box[2]-box[0]+1));
-        for(int j=box[1]; j<=box[3]; j++) for(int i=box[0]; i<=box[2]; i++)
-        {
-          const float L = in[4*(roi_in.width*j + i) + 0];
-          const float a = in[4*(roi_in.width*j + i) + 1];
-          const float b = in[4*(roi_in.width*j + i) + 2];
-          Lab[0] += w*L;
-          Lab[1] += w*a;
-          Lab[2] += w*b;
-          module->picked_color_min[0] = fminf(module->picked_color_min[0], L);
-          module->picked_color_min[1] = fminf(module->picked_color_min[1], a);
-          module->picked_color_min[2] = fminf(module->picked_color_min[2], b);
-          module->picked_color_max[0] = fmaxf(module->picked_color_max[0], L);
-          module->picked_color_max[1] = fmaxf(module->picked_color_max[1], a);
-          module->picked_color_max[2] = fmaxf(module->picked_color_max[2], b);
-          for(int k=0; k<3; k++) module->picked_color[k] = Lab[k];
-        }
-      }
-      else
-      {
-        for(int i = 0; i < 3; i++)
-          module->picked_color[i]
-              = module->picked_color_min[i]
-                = module->picked_color_max[i]
-                  = in[4*(roi_in.width*point[1] + point[0]) + i];
-      }
+      pixelpipe_picker(module, (float *)input, &roi_in, module->picked_color, module->picked_color_min, module->picked_color_max);
 
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
 
-      gboolean i_own_lock = dt_control_gdk_lock();      
+      gboolean i_own_lock = dt_control_gdk_lock();
       if(module->widget) gtk_widget_queue_draw(module->widget);
       if (i_own_lock) dt_control_gdk_unlock();
     }
     else dt_pthread_mutex_unlock(&pipe->busy_mutex);
+#else
+    dt_pthread_mutex_unlock(&pipe->busy_mutex);
+#endif
 
     // actual pixel processing done by module
     dt_pthread_mutex_lock(&pipe->busy_mutex);
+
     if(pipe->shutdown)
     {
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
       return 1;
     }
+
     dt_times_t start;
     dt_get_times(&start);
 
     dt_develop_tiling_t tiling = { 0 };
+    dt_develop_tiling_t tiling_blendop = { 0 };
 
     /* get tiling requirement of module */
     module->tiling_callback(module, piece, &roi_in, roi_out, &tiling);
 
-    assert(tiling.factor > 0.0f && tiling.factor < 100.0f);      
+    /* get specific requirement for blending */
+    tiling_callback_blendop(module, piece, &roi_in, roi_out, &tiling_blendop);
+
+    /* aggregate in structure tiling */
+    tiling.factor = fmax(tiling.factor, tiling_blendop.factor);
+    tiling.maxbuf = fmax(tiling.maxbuf, tiling_blendop.maxbuf);
+    tiling.overhead = fmax(tiling.overhead, tiling_blendop.overhead);
+
+    /* remark: we do not do tiling for blendop step, neither in opencl nor on cpu. if overall tiling
+       requirements (maximum of module and blendop) require tiling for opencl path, then following blend
+       step is anyhow done on cpu. we assume that blending itself will never require tiling in cpu path,
+       because memory requirements will still be low enough. */
+
+    assert(tiling.factor > 0.0f && tiling.factor < 100.0f);
+
+    if(pipe->shutdown)
+    {
+      dt_pthread_mutex_unlock(&pipe->busy_mutex);
+      return 1;
+    }
+
 
 #ifdef HAVE_OPENCL
     /* do we have opencl at all? did user tell us to use it? did we get a resource? */
@@ -563,12 +611,13 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
 
       /* if input is on gpu memory only, remember this fact to later take appropriate action */
       int valid_input_on_gpu_only = (cl_mem_input != NULL);
-      
+
       /* general remark: in case of opencl errors within modules or out-of-memory on GPU, we transparently
-         fall back to the respective cpu module and continue in pixelpipe. If we encounter late errors(*), we set 
-         pipe->opencl_error=1, return this function with value 1, and leave appropriate action to the calling function,
-         which normally would restart pixelpipe without opencl.
-         (*) late errors are sometimes detected when trying to get back data from device into host memory */
+         fall back to the respective cpu module and continue in pixelpipe. If we encounter errors we set
+         pipe->opencl_error=1, return this function with value 1, and leave appropriate action to the calling
+         function, which normally would restart pixelpipe without opencl.
+         Late errors are sometimes detected when trying to get back data from device into host memory and
+         are treated in the same manner. */
 
       /* try to enter opencl path after checking some module specific pre-requisites */
       if(module->process_cl && piece->process_cl_ready)
@@ -596,6 +645,13 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
             }
           }
 
+          if(pipe->shutdown)
+          {
+            if(cl_mem_input != NULL) dt_opencl_release_mem_object(cl_mem_input);
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
           /* try to allocate GPU memory for output */
           if (success_opencl)
           {
@@ -609,14 +665,36 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
 
           // fprintf(stderr, "[opencl_pixelpipe 2] for module `%s', have bufs %lX and %lX \n", module->op, (long int)cl_mem_input, (long int)*cl_mem_output);
 
+          // indirectly give gpu some air to breathe (and to do display related stuff)
+          dt_iop_nap(1000);
+
           /* now call process_cl of module; module should emit meaningful messages in case of error */
           if (success_opencl)
             success_opencl = module->process_cl(module, piece, cl_mem_input, *cl_mem_output, &roi_in, roi_out);
 
+          if(pipe->shutdown)
+          {
+            if(cl_mem_input != NULL) dt_opencl_release_mem_object(cl_mem_input);
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
 
           /* process blending */
           if (success_opencl)
             success_opencl = dt_develop_blend_process_cl(module, piece, cl_mem_input, *cl_mem_output, &roi_in, roi_out);
+
+          /* synchronization point for opencl pipe */
+          if (success_opencl)
+            success_opencl = dt_opencl_finish(pipe->devid);
+
+
+          if(pipe->shutdown)
+          {
+            if(cl_mem_input != NULL) dt_opencl_release_mem_object(cl_mem_input);
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
         }
         else if(module->flags() & IOP_FLAGS_ALLOW_TILING)
         {
@@ -645,13 +723,39 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
             valid_input_on_gpu_only = FALSE;
           }
 
+          if(pipe->shutdown)
+          {
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
+          // indirectly give gpu some air to breathe (and to do display related stuff)
+          dt_iop_nap(1000);
+
           /* now call process_tiling_cl of module; module should emit meaningful messages in case of error */
           if (success_opencl)
             success_opencl = module->process_tiling_cl(module, piece, input, *output, &roi_in, roi_out, in_bpp);
 
+          if(pipe->shutdown)
+          {
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
           /* do process blending on cpu (this is anyhow fast enough) */
           if (success_opencl)
             dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
+
+          /* synchronization point for opencl pipe */
+          if (success_opencl)
+            success_opencl = dt_opencl_finish(pipe->devid);
+
+          if(pipe->shutdown)
+          {
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
         }
         else
         {
@@ -659,16 +763,53 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
           success_opencl = FALSE;
         }
 
-        
+        if(pipe->shutdown)
+        {
+          if(cl_mem_input) dt_opencl_release_mem_object(cl_mem_input);
+          dt_pthread_mutex_unlock(&pipe->busy_mutex);
+          return 1;
+        }
+
         // if (rand() % 20 == 0) success_opencl = FALSE; // Test code: simulate spurious failures
 
         /* finally check, if we were successful */
         if (success_opencl)
         {
           /* Nice, everything went fine */
-        
+
+          /* write back input into cache for faster re-usal (not for export or thumbnails) */
+          if (cl_mem_input != NULL && pipe->type != DT_DEV_PIXELPIPE_EXPORT && pipe->type != DT_DEV_PIXELPIPE_THUMBNAIL)
+          {
+            cl_int err;
+
+            /* copy input to host memory, so we can find it in cache */
+            err = dt_opencl_copy_device_to_host(pipe->devid, input, cl_mem_input, roi_in.width, roi_in.height, in_bpp);
+            if (err != CL_SUCCESS)
+            {
+              /* late opencl error, not likely to happen here */
+              dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (e)] late opencl error detected while copying back to cpu buffer: %d\n", err);
+              /* that's all we do here, we later make sure to invalidate cache line */
+            }
+            else
+            {
+              /* success: cache line is valid now, so we will not need to invalidate it later */
+              valid_input_on_gpu_only = FALSE;
+          
+              // TODO: check if we need to wait for finished opencl pipe before we release cl_mem_input
+              // dt_dev_finish(pipe->devid);
+            }
+          }
+
+          if(pipe->shutdown)
+          {
+            if(cl_mem_input != NULL) dt_opencl_release_mem_object(cl_mem_input);
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+ 
           /* we can now release cl_mem_input */
-          if (cl_mem_input) dt_opencl_release_mem_object(cl_mem_input);
+          if(cl_mem_input != NULL) dt_opencl_release_mem_object(cl_mem_input);
+          cl_mem_input = NULL;
           // we speculate on the next plug-in to possibly copy back cl_mem_output to output,
           // so we're not just yet invalidating the (empty) output cache line.
         }
@@ -685,7 +826,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
             dt_opencl_release_mem_object(*cl_mem_output);
             *cl_mem_output = NULL;
           }
-          
+
           /* check where our input buffer is located */
           if (cl_mem_input != NULL)
           {
@@ -708,22 +849,41 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
             valid_input_on_gpu_only = FALSE;
           }
 
+          if(pipe->shutdown)
+          {
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
           /* process module on cpu. use tiling if needed and possible. */
-          if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
-                !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
-                max(in_bpp, bpp), tiling.factor, tiling.overhead))
+          if((module->flags() & IOP_FLAGS_ALLOW_TILING) &&
+              !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height),
+                                                max(in_bpp, bpp), tiling.factor, tiling.overhead))
             module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
           else
             module->process(module, piece, input, *output, &roi_in, roi_out);
 
+          if(pipe->shutdown)
+          {
+            dt_pthread_mutex_unlock(&pipe->busy_mutex);
+            return 1;
+          }
+
           /* process blending on cpu */
           dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
         }
+
+        if(pipe->shutdown)
+        {
+          dt_pthread_mutex_unlock(&pipe->busy_mutex);
+          return 1;
+        }
+
       }
       else
       {
         /* we are not allowed to use opencl for this module */
-  
+
         // fprintf(stderr, "[opencl_pixelpipe 3] for module `%s', have bufs %lX and %lX \n", module->op, (long int)cl_mem_input, (long int)*cl_mem_output);
 
         *cl_mem_output = NULL;
@@ -748,16 +908,35 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
           valid_input_on_gpu_only = FALSE;
         }
 
+        if(pipe->shutdown)
+        {
+          dt_pthread_mutex_unlock(&pipe->busy_mutex);
+          return 1;
+        }
+
         /* process module on cpu. use tiling if needed and possible. */
-        if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
-              !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
-              max(in_bpp, bpp), tiling.factor, tiling.overhead))
+        if((module->flags() & IOP_FLAGS_ALLOW_TILING) &&
+            !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height),
+                                              max(in_bpp, bpp), tiling.factor, tiling.overhead))
           module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
         else
           module->process(module, piece, input, *output, &roi_in, roi_out);
 
+        if(pipe->shutdown)
+        {
+          dt_pthread_mutex_unlock(&pipe->busy_mutex);
+          return 1;
+        }
+
         /* process blending */
         dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
+
+        if(pipe->shutdown)
+        {
+          dt_pthread_mutex_unlock(&pipe->busy_mutex);
+          return 1;
+        }
+
       }
 
       /* input is still only on GPU? Let's invalidate CPU input buffer then */
@@ -768,25 +947,83 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
       /* opencl is not inited or not enabled or we got no resource/device -> everything runs on cpu */
 
       /* process module on cpu. use tiling if needed and possible. */
-      if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
-            !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
-            max(in_bpp, bpp), tiling.factor, tiling.overhead))
+      if((module->flags() & IOP_FLAGS_ALLOW_TILING) &&
+          !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height),
+                                            max(in_bpp, bpp), tiling.factor, tiling.overhead))
         module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
       else
         module->process(module, piece, input, *output, &roi_in, roi_out);
 
+      if(pipe->shutdown)
+      {
+        dt_pthread_mutex_unlock(&pipe->busy_mutex);
+        return 1;
+      }
+
+      // Lab color picking for module
+      if(dev->gui_attached && pipe == dev->preview_pipe && // pick from preview pipe to get pixels outside the viewport
+          module == dev->gui_module && // only modules with focus can pick
+          module->request_color_pick) // and they need to want to pick ;)
+      {
+        pixelpipe_picker(module, (float *)input, &roi_in, module->picked_color, module->picked_color_min, module->picked_color_max);
+        pixelpipe_picker(module, (float *)(*output), roi_out, module->picked_output_color, module->picked_output_color_min, module->picked_output_color_max);
+
+        dt_pthread_mutex_unlock(&pipe->busy_mutex);
+
+        gboolean i_own_lock = dt_control_gdk_lock();
+        gtk_widget_queue_draw(module->widget);
+        if (i_own_lock) dt_control_gdk_unlock();
+
+        dt_pthread_mutex_lock(&pipe->busy_mutex);
+      }
+
+      if(pipe->shutdown)
+      {
+        dt_pthread_mutex_unlock(&pipe->busy_mutex);
+        return 1;
+      }
+
       /* process blending */
       dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
-    } 
+    }
 #else
     /* process module on cpu. use tiling if needed and possible. */
-    if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
-          !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
-          max(in_bpp, bpp), tiling.factor, tiling.overhead))
+    if((module->flags() & IOP_FLAGS_ALLOW_TILING) &&
+        !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height),
+                                          max(in_bpp, bpp), tiling.factor, tiling.overhead))
       module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
     else
       module->process(module, piece, input, *output, &roi_in, roi_out);
 
+    if(pipe->shutdown)
+    {
+      dt_pthread_mutex_unlock(&pipe->busy_mutex);
+      return 1;
+    }
+
+    // Lab color picking for module
+    if(dev->gui_attached && pipe == dev->preview_pipe && // pick from preview pipe to get pixels outside the viewport
+        module == dev->gui_module && // only modules with focus can pick
+        module->request_color_pick) // and they need to want to pick ;)
+    {
+      pixelpipe_picker(module, (float *)input, &roi_in, module->picked_color, module->picked_color_min, module->picked_color_max);
+      pixelpipe_picker(module, (float *)(*output), roi_out, module->picked_output_color, module->picked_output_color_min, module->picked_output_color_max);
+
+      dt_pthread_mutex_unlock(&pipe->busy_mutex);
+
+      gboolean i_own_lock = dt_control_gdk_lock();
+      gtk_widget_queue_draw(module->widget);
+      if (i_own_lock) dt_control_gdk_unlock();
+
+      dt_pthread_mutex_lock(&pipe->busy_mutex);
+    }
+
+    if(pipe->shutdown)
+    {
+      dt_pthread_mutex_unlock(&pipe->busy_mutex);
+      return 1;
+    }
+
     /* process blending */
     dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
 #endif
@@ -830,9 +1067,9 @@ post_process_collect_info:
     }
     // Picking RGB for the live samples and converting to Lab
     if(dev->gui_attached
-       && pipe == dev->preview_pipe
-       && (strcmp(module->op, "gamma") == 0)
-       && darktable.lib->proxy.colorpicker.live_samples) // samples to pick
+        && pipe == dev->preview_pipe
+        && (strcmp(module->op, "gamma") == 0)
+        && darktable.lib->proxy.colorpicker.live_samples) // samples to pick
     {
       dt_colorpicker_sample_t *sample = NULL;
       GSList *samples = darktable.lib->proxy.colorpicker.live_samples;
@@ -869,27 +1106,27 @@ post_process_collect_info:
         if(sample->size == DT_COLORPICKER_SIZE_BOX)
         {
           for(int j=box[1]; j<=box[3]; j++) for(int i=box[0]; i<=box[2]; i++)
-          {
-            for(int k=0; k<3; k++)
             {
-              sample->picked_color_rgb_min[k] =
+              for(int k=0; k<3; k++)
+              {
+                sample->picked_color_rgb_min[k] =
                   MIN(sample->picked_color_rgb_min[k],
                       pixel[4*(roi_out->width*j + i) + 2-k]);
-              sample->picked_color_rgb_max[k] =
+                sample->picked_color_rgb_max[k] =
                   MAX(sample->picked_color_rgb_max[k],
                       pixel[4*(roi_out->width*j + i) + 2-k]);
-              rgb[k] += w*pixel[4*(roi_out->width*j + i) + 2-k];
+                rgb[k] += w*pixel[4*(roi_out->width*j + i) + 2-k];
+              }
             }
-          }
           for(int k=0; k<3; k++) sample->picked_color_rgb_mean[k] = rgb[k];
         }
         else
         {
           for(int i = 0; i < 3; i++)
             sample->picked_color_rgb_mean[i]
-                = sample->picked_color_rgb_min[i]
-                  = sample->picked_color_rgb_max[i]
-                    = pixel[4*(roi_out->width*point[1] + point[0]) + 2-i];
+            = sample->picked_color_rgb_min[i]
+              = sample->picked_color_rgb_max[i]
+                = pixel[4*(roi_out->width*point[1] + point[0]) + 2-i];
         }
 
         // Converting the RGB values to Lab
@@ -911,8 +1148,8 @@ post_process_collect_info:
           cmsHPROFILE Lab = dt_colorspaces_create_lab_profile();
 
           cmsHTRANSFORM xform = cmsCreateTransform(out_profile, TYPE_RGB_FLT,
-                                                  Lab, TYPE_Lab_FLT,
-                                                  INTENT_PERCEPTUAL, 0);
+                                Lab, TYPE_Lab_FLT,
+                                INTENT_PERCEPTUAL, 0);
 
 
           // Preparing the data for transformation
@@ -920,11 +1157,11 @@ post_process_collect_info:
           for(int i = 0; i < 3; i++)
           {
             rgb_data[i] =
-                sample->picked_color_rgb_mean[i] / 255.0;
+              sample->picked_color_rgb_mean[i] / 255.0;
             rgb_data[i + 3] =
-                sample->picked_color_rgb_min[i] / 255.0;
+              sample->picked_color_rgb_min[i] / 255.0;
             rgb_data[i + 6] =
-                sample->picked_color_rgb_max[i] / 255.0;
+              sample->picked_color_rgb_max[i] / 255.0;
           }
 
           float Lab_data[9];
@@ -933,11 +1170,11 @@ post_process_collect_info:
           for(int i = 0; i < 3; i++)
           {
             sample->picked_color_lab_mean[i] =
-                Lab_data[i];
+              Lab_data[i];
             sample->picked_color_lab_min[i] =
-                Lab_data[i + 3];
+              Lab_data[i + 3];
             sample->picked_color_lab_max[i] =
-                Lab_data[i + 6];
+              Lab_data[i + 6];
           }
 
           cmsDeleteTransform(xform);
@@ -949,12 +1186,12 @@ post_process_collect_info:
     }
     //Picking RGB for primary colorpicker output and converting to Lab
     if(dev->gui_attached
-       && pipe == dev->preview_pipe
-       && (strcmp(module->op, "gamma") == 0) // only gamma provides meaningful RGB data
-       && dev->gui_module
-       && !strcmp(dev->gui_module->op, "colorout")
-       && dev->gui_module->request_color_pick
-       && darktable.lib->proxy.colorpicker.picked_color_rgb_mean) // colorpicker module active
+        && pipe == dev->preview_pipe
+        && (strcmp(module->op, "gamma") == 0) // only gamma provides meaningful RGB data
+        && dev->gui_module
+        && !strcmp(dev->gui_module->op, "colorout")
+        && dev->gui_module->request_color_pick
+        && darktable.lib->proxy.colorpicker.picked_color_rgb_mean) // colorpicker module active
     {
       uint8_t *pixel = (uint8_t*)*output;
 
@@ -968,14 +1205,14 @@ post_process_collect_info:
       for(int k=0; k<3; k++) rgb[k] = 0.0f;
       for(int k=0; k<4; k+=2)
         box[k] =
-            MIN(roi_out->width -1,
-                MAX(0, dev->gui_module->color_picker_box[k]
-                    * roi_out->width));
+          MIN(roi_out->width -1,
+              MAX(0, dev->gui_module->color_picker_box[k]
+                  * roi_out->width));
       for(int k=1; k<4; k+=2)
         box[k] =
-            MIN(roi_out->height-1,
-                MAX(0, dev->gui_module->color_picker_box[k]
-                    * roi_out->height));
+          MIN(roi_out->height-1,
+              MAX(0, dev->gui_module->color_picker_box[k]
+                  * roi_out->height));
       point[0] = MIN(roi_out->width -1,
                      MAX(0, dev->gui_module->color_picker_point[0]
                          * roi_out->width));
@@ -986,18 +1223,18 @@ post_process_collect_info:
       if(darktable.lib->proxy.colorpicker.size)
       {
         for(int j=box[1]; j<=box[3]; j++) for(int i=box[0]; i<=box[2]; i++)
-        {
-          for(int k=0; k<3; k++)
           {
-            darktable.lib->proxy.colorpicker.picked_color_rgb_min[k] =
+            for(int k=0; k<3; k++)
+            {
+              darktable.lib->proxy.colorpicker.picked_color_rgb_min[k] =
                 MIN(darktable.lib->proxy.colorpicker.picked_color_rgb_min[2-k],
                     pixel[4*(roi_out->width*j + i) + 2-k]);
-            darktable.lib->proxy.colorpicker.picked_color_rgb_max[k] =
+              darktable.lib->proxy.colorpicker.picked_color_rgb_max[k] =
                 MAX(darktable.lib->proxy.colorpicker.picked_color_rgb_max[k],
                     pixel[4*(roi_out->width*j + i) + 2-k]);
-            rgb[k] += w*pixel[4*(roi_out->width*j + i) + 2-k];
+              rgb[k] += w*pixel[4*(roi_out->width*j + i) + 2-k];
+            }
           }
-        }
         for(int k=0; k<3; k++)
           darktable.lib->proxy.colorpicker.picked_color_rgb_mean[k] = rgb[k];
       }
@@ -1005,9 +1242,9 @@ post_process_collect_info:
       {
         for(int i = 0; i < 3; i++)
           darktable.lib->proxy.colorpicker.picked_color_rgb_mean[i]
-              = darktable.lib->proxy.colorpicker.picked_color_rgb_min[i]
-                = darktable.lib->proxy.colorpicker.picked_color_rgb_max[i]
-                  = pixel[4*(roi_out->width*point[1] + point[0]) + 2-i];
+          = darktable.lib->proxy.colorpicker.picked_color_rgb_min[i]
+            = darktable.lib->proxy.colorpicker.picked_color_rgb_max[i]
+              = pixel[4*(roi_out->width*point[1] + point[0]) + 2-i];
       }
 
       // Converting the RGB values to Lab
@@ -1029,8 +1266,8 @@ post_process_collect_info:
         cmsHPROFILE Lab = dt_colorspaces_create_lab_profile();
 
         cmsHTRANSFORM xform = cmsCreateTransform(out_profile, TYPE_RGB_FLT,
-                                                Lab, TYPE_Lab_FLT,
-                                                INTENT_PERCEPTUAL, 0);
+                              Lab, TYPE_Lab_FLT,
+                              INTENT_PERCEPTUAL, 0);
 
 
         // Preparing the data for transformation
@@ -1038,11 +1275,11 @@ post_process_collect_info:
         for(int i = 0; i < 3; i++)
         {
           rgb_data[i] =
-              darktable.lib->proxy.colorpicker.picked_color_rgb_mean[i] / 255.0;
+            darktable.lib->proxy.colorpicker.picked_color_rgb_mean[i] / 255.0;
           rgb_data[i + 3] =
-              darktable.lib->proxy.colorpicker.picked_color_rgb_min[i] / 255.0;
+            darktable.lib->proxy.colorpicker.picked_color_rgb_min[i] / 255.0;
           rgb_data[i + 6] =
-              darktable.lib->proxy.colorpicker.picked_color_rgb_max[i] / 255.0;
+            darktable.lib->proxy.colorpicker.picked_color_rgb_max[i] / 255.0;
         }
 
         float Lab_data[9];
@@ -1051,11 +1288,11 @@ post_process_collect_info:
         for(int i = 0; i < 3; i++)
         {
           darktable.lib->proxy.colorpicker.picked_color_lab_mean[i] =
-              Lab_data[i];
+            Lab_data[i];
           darktable.lib->proxy.colorpicker.picked_color_lab_min[i] =
-              Lab_data[i + 3];
+            Lab_data[i + 3];
           darktable.lib->proxy.colorpicker.picked_color_lab_max[i] =
-              Lab_data[i + 6];
+            Lab_data[i + 6];
         }
 
         cmsDeleteTransform(xform);
@@ -1063,7 +1300,7 @@ post_process_collect_info:
       }
 
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
-      
+
       gboolean i_own_lock = dt_control_gdk_lock();
       if(module->widget) gtk_widget_queue_draw(module->widget);
       if (i_own_lock) dt_control_gdk_unlock();
@@ -1079,16 +1316,16 @@ post_process_collect_info:
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
       return 1;
     }
-    if(dev->gui_attached && !dev->gui_leaving && 
-       pipe == dev->preview_pipe && (strcmp(module->op, "gamma") == 0))
+    if(dev->gui_attached && !dev->gui_leaving &&
+        pipe == dev->preview_pipe && (strcmp(module->op, "gamma") == 0))
     {
       uint8_t *pixel = (uint8_t *)*output;
       float box[4];
       // Constraining the area if the colorpicker is active in area mode
       if(dev->gui_module
-         && !strcmp(dev->gui_module->op, "colorout")
-         && dev->gui_module->request_color_pick
-         && darktable.lib->proxy.colorpicker.restrict_histogram)
+          && !strcmp(dev->gui_module->op, "colorout")
+          && dev->gui_module->request_color_pick
+          && darktable.lib->proxy.colorpicker.restrict_histogram)
       {
         if(darktable.lib->proxy.colorpicker.size == DT_COLORPICKER_SIZE_BOX)
         {
@@ -1123,36 +1360,37 @@ post_process_collect_info:
       dev->histogram_max = 0;
       memset(dev->histogram, 0, sizeof(float)*4*64);
       for(int j=box[1]; j<=box[3]; j+=4) for(int i=box[0]; i<=box[2]; i+=4)
-      {
-        uint8_t rgb[3];
-        for(int k=0; k<3; k++)
-          rgb[k] = pixel[4*j*roi_out->width+4*i+2-k]>>2;
+        {
+          uint8_t rgb[3];
+          for(int k=0; k<3; k++)
+            rgb[k] = pixel[4*j*roi_out->width+4*i+2-k]>>2;
+
+          for(int k=0; k<3; k++)
+            dev->histogram[4*rgb[k]+k] ++;
+          uint8_t lum = MAX(MAX(rgb[0], rgb[1]), rgb[2]);
+          dev->histogram[4*lum+3] ++;
+        }
 
-        for(int k=0; k<3; k++)
-          dev->histogram[4*rgb[k]+k] ++;
-        uint8_t lum = MAX(MAX(rgb[0], rgb[1]), rgb[2]);
-        dev->histogram[4*lum+3] ++;
-      }
-      for(int k=0; k<4*64; k++) dev->histogram[k] = logf(1.0 + dev->histogram[k]);
       // don't count <= 0 pixels
       for(int k=19; k<4*64; k+=4) dev->histogram_max = dev->histogram_max > dev->histogram[k] ? dev->histogram_max : dev->histogram[k];
+
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
-      
+
       /* raise preview pipe finised signal */
       dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED);
 
     }
-    else 
+    else
     {
       dt_pthread_mutex_unlock(&pipe->busy_mutex);
 
       /* if gui attached, lets raise pipe finish signal */
       if (dev->gui_attached && !dev->gui_leaving && strcmp(module->op, "gamma") == 0)
-	dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_UI_PIPE_FINISHED);
-	
+        dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_UI_PIPE_FINISHED);
+
     }
-  } 
-  
+  }
+
   return 0;
 }
 
@@ -1178,7 +1416,7 @@ int dt_dev_pixelpipe_process_no_gamma(dt_dev_pixelpipe_t *pipe, dt_develop_t *de
 
 static int
 dt_dev_pixelpipe_process_rec_and_backcopy(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void **output, void **cl_mem_output, int *out_bpp,
-                             const dt_iop_roi_t *roi_out, GList *modules, GList *pieces, int pos)
+    const dt_iop_roi_t *roi_out, GList *modules, GList *pieces, int pos)
 {
 #ifdef HAVE_OPENCL
   int ret = dt_dev_pixelpipe_process_rec(pipe, dev, output, cl_mem_output, out_bpp, roi_out, modules, pieces, pos);
@@ -1253,6 +1491,13 @@ restart:
   // image max is normalized before
   for(int k=0; k<3; k++) pipe->processed_maximum[k] = 1.0f; // dev->image->maximum;
 
+  // check if we should obsolete caches
+  if(pipe->cache_obsolete) dt_dev_pixelpipe_cache_flush(&(pipe->cache));
+  pipe->cache_obsolete = 0;
+
+  // mask display off as a starting point
+  pipe->mask_display = 0;
+
   void *buf = NULL;
   void *cl_mem_out = NULL;
   int out_bpp;
@@ -1343,3 +1588,6 @@ void dt_dev_pixelpipe_get_dimensions(dt_dev_pixelpipe_t *pipe, struct dt_develop
   dt_pthread_mutex_unlock(&pipe->busy_mutex);
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
