@@ -192,6 +192,15 @@ GtkWidget* dtgtk_label_new(const gchar *text, _darktable_label_flags_t flags)
   return (GtkWidget *)label;
 }
 
+void dtgtk_label_set_text(GtkDarktableLabel *label,
+                          const gchar *text,
+                          _darktable_label_flags_t flags)
+{
+  gtk_label_set_text(GTK_LABEL(label),text);
+  label->flags=flags;
+  gtk_widget_queue_draw(GTK_WIDGET(label));
+}
+
 GtkType dtgtk_label_get_type()
 {
   static GtkType dtgtk_label_type = 0;
@@ -212,3 +221,6 @@ GtkType dtgtk_label_get_type()
   }
   return dtgtk_label_type;
 }
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
