@@ -46,7 +46,7 @@ public:
   uchar8* getData();
   uchar8* getData(uint32 x, uint32 y);    // Not super fast, but safe. Don't use per pixel.
   uchar8* getDataUncropped(uint32 x, uint32 y);
-  virtual void subFrame( iPoint2D offset, iPoint2D new_size );
+  virtual void subFrame( iRectangle2D cropped );
   iPoint2D getUncroppedDim();
   iPoint2D getCropOffset();
   virtual void scaleBlackWhite() = 0;
@@ -62,6 +62,12 @@ public:
   iPoint2D subsampling;
   string mode;
   int isoSpeed;
+  /* Vector containing silent errors that occurred doing decoding, that may have lead to */
+  /* an incomplete image. */
+  vector<const char*> errors;
+  pthread_mutex_t errMutex;   // Mutex for above
+  void setError(const char* err);
+
 protected:
   RawImageType dataType;
   RawImageData(void);
