@@ -118,7 +118,7 @@ void RawImageDataU16::calculateBlackAreas() {
 void RawImageDataU16::scaleBlackWhite() {
   const int skipBorder = 250;
   int gw = (dim.x - skipBorder) * cpp;
-  if ((blackAreas.empty() && blackLevelSeparate[0] < 0 && blackLevel < 0) || whitePoint == 65536) {  // Estimate
+  if ((blackAreas.empty() && blackLevelSeparate[0] < 0 && blackLevel < 0) || whitePoint >= 65536) {  // Estimate
     int b = 65536;
     int m = 0;
     for (int row = skipBorder*cpp;row < (dim.y - skipBorder);row++) {
@@ -131,19 +131,22 @@ void RawImageDataU16::scaleBlackWhite() {
     }
     if (blackLevel < 0)
       blackLevel = b;
-    if (whitePoint == 65536)
+    if (whitePoint >= 65536)
       whitePoint = m;
     printf("ISO:%d, Estimated black:%d, Estimated white: %d\n", isoSpeed, blackLevel, whitePoint);
   }
 
   /* Skip, if not needed */
-  if (blackAreas.size() == 0 && blackLevel == 0 && whitePoint == 65535 && blackLevelSeparate[0] < 0)
+  if ((blackAreas.size() == 0 && blackLevel == 0 && whitePoint == 65535 && blackLevelSeparate[0] < 0) || dim.area() <= 0)
     return;
 
   /* If filter has not set separate blacklevel, compute or fetch it */
   if (blackLevelSeparate[0] < 0)
     calculateBlackAreas();
 
+//  printf("ISO:%d, black[0]:%d, white: %d\n", isoSpeed, blackLevelSeparate[0], whitePoint);
+//  printf("black[1]:%d, black[2]:%d, black[3]:%d\n", blackLevelSeparate[1], blackLevelSeparate[2], blackLevelSeparate[3]);
+
   int threads = getThreadCount(); 
   if (threads <= 1)
     scaleValues(0, dim.y);
@@ -185,6 +188,8 @@ void RawImageDataU16::scaleValues(int start_y, int end_y) {
     __m128i ssesub2;
     __m128i ssesign;
     uint32* sub_mul = (uint32*)_aligned_malloc(16*4*2, 16);
+    if (!sub_mul)
+	  ThrowRDE("Out of memory, failed to allocate 128 bytes");
     uint32 gw = pitch / 16;
     // 10 bit fraction
     uint32 mul = (int)(1024.0f * 65535.0f / (float)(whitePoint - blackLevelSeparate[mOffset.x&1]));  
