@@ -25,13 +25,13 @@
 
 #include <stdlib.h>
 #include <math.h>
-// this is a dirty hack, this way nikon_curve will not even be compiled if we don't need it:
-#ifdef DT_CONTROL_H
-#include "common/curve_tools.c"
-#else
 #include "common/curve_tools.h"
-#endif
 #include <cairo.h>
+#include <stdint.h>
+#include <glib.h>
+
+#include "common/darktable.h"
+#include "develop/develop.h"
 
 /** wrapper around nikon curve or gegl. */
 typedef struct dt_draw_curve_t
@@ -56,6 +56,11 @@ static inline void dt_draw_star(cairo_t *cr, float x, float y, float r1, float r
   cairo_close_path(cr);
 }
 
+static inline void dt_draw_line(cairo_t *cr, float left, float top, float right, float bottom)
+{
+  cairo_move_to(cr, left, top);
+  cairo_line_to(cr, right,  bottom);
+}
 
 static inline void dt_draw_grid(cairo_t *cr, const int num, const int left, const int top, const int right, const int bottom)
 {
@@ -64,18 +69,16 @@ static inline void dt_draw_grid(cairo_t *cr, const int num, const int left, cons
 
   for(int k=1; k<num; k++)
   {
-    cairo_move_to(cr, left + k/(float)num*width, top);
-    cairo_line_to(cr, left +  k/(float)num*width, bottom);
+    dt_draw_line(cr, left + k/(float)num*width, top, left +  k/(float)num*width, bottom);
     cairo_stroke(cr);
-    cairo_move_to(cr, left, top + k/(float)num*height);
-    cairo_line_to(cr, right, top + k/(float)num*height);
+    dt_draw_line(cr, left, top + k/(float)num*height, right, top + k/(float)num*height);
     cairo_stroke(cr);
   }
 }
 
 static inline void dt_draw_vertical_lines(cairo_t *cr, const int num,
-                                          const int left, const int top,
-                                          const int right, const int bottom)
+    const int left, const int top,
+    const int right, const int bottom)
 {
   float width = right - left;
 
@@ -175,7 +178,7 @@ static inline int dt_draw_curve_add_point(dt_draw_curve_t *c, const            f
   return 0;
 }
 
-static inline void dt_draw_histogram_8(cairo_t *cr, float *hist, int32_t channel)
+static inline void dt_draw_histogram_8_linear(cairo_t *cr, float *hist, int32_t channel)
 {
   cairo_move_to(cr, 0, 0);
   for(int k=0; k<64; k++)
@@ -184,5 +187,55 @@ static inline void dt_draw_histogram_8(cairo_t *cr, float *hist, int32_t channel
   cairo_close_path(cr);
   cairo_fill(cr);
 }
+
+static inline void dt_draw_histogram_8_log(cairo_t *cr, float *hist, int32_t channel)
+{
+  cairo_move_to(cr, 0, 0);
+  for(int k=0; k<64; k++)
+    cairo_line_to(cr, k, logf(1.0 + hist[4*k+channel]));
+  cairo_line_to(cr, 63, 0);
+  cairo_close_path(cr);
+  cairo_fill(cr);
+}
+
+static inline void dt_draw_histogram_8(cairo_t *cr, float *hist, int32_t channel)
+{
+  if(darktable.develop->histogram_linear == TRUE)
+    dt_draw_histogram_8_linear(cr, hist, channel);
+  else
+    dt_draw_histogram_8_log(cr, hist, channel);
+}
+
+/** transform a data blob from cairo's premultiplied rgba/bgra to GdkPixbuf's un-premultiplied bgra/rgba */
+static inline void dt_draw_cairo_to_gdk_pixbuf(uint8_t *data, unsigned int width, unsigned int height)
+{
+  for(int y=0; y<height; y++)
+    for(int x=0; x<width; x++)
+    {
+      uint8_t *r, *g, *b, *a, tmp;
+      r = &data[(y*width+x)*4+0];
+      g = &data[(y*width+x)*4+1];
+      b = &data[(y*width+x)*4+2];
+      a = &data[(y*width+x)*4+3];
+
+      // switch r and b
+      tmp = *r;
+      *r = *b;
+      *b = tmp;
+
+      // cairo uses premultiplied alpha, reverse that
+      if(*a != 0)
+      {
+        float inv_a = 255.0 / *a;
+        *r *= inv_a;
+        *g *= inv_a;
+        *b *= inv_a;
+      }
+    }
+}
+
 #endif
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
