@@ -621,6 +621,12 @@ extension(dt_imageio_module_data_t *data)
   return "jpg";
 }
 
+int
+flags()
+{
+  return FORMAT_FLAGS_SUPPORT_XMP;
+}
+
 void init(dt_imageio_module_format_t *self) {}
 void cleanup(dt_imageio_module_format_t *self) {}
 
@@ -671,3 +677,6 @@ void gui_reset (dt_imageio_module_format_t *self)
   dtgtk_slider_set_value(g->quality, dt_conf_get_int("plugins/imageio/format/jpeg/quality"));
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
