@@ -28,6 +28,7 @@
 #include "control/control.h"
 #include "control/conf.h"
 #include "gui/gtk.h"
+#include "gui/gtkentry.h"
 #include "dtgtk/button.h"
 #include "dtgtk/paint.h"
 #include <stdio.h>
@@ -46,9 +47,9 @@ gallery_t;
 // saved params
 typedef struct dt_imageio_gallery_t
 {
-  char filename[1024];
+  char filename[DT_MAX_PATH_LEN];
   char title[1024];
-  char cached_dirname[1024]; // expanded during first img store, not stored in param struct.
+  char cached_dirname[DT_MAX_PATH_LEN]; // expanded during first img store, not stored in param struct.
   dt_variables_params_t *vp;
   GList *l;
 }
@@ -90,8 +91,8 @@ button_clicked (GtkWidget *widget, dt_imageio_module_storage_t *self)
   if (gtk_dialog_run (GTK_DIALOG (filechooser)) == GTK_RESPONSE_ACCEPT)
   {
     gchar *dir = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
-    char composed[1024];
-    snprintf(composed, 1024, "%s/$(FILE_NAME)", dir);
+    char composed[DT_MAX_PATH_LEN];
+    snprintf(composed, DT_MAX_PATH_LEN, "%s/$(FILE_NAME)", dir);
     gtk_entry_set_text(GTK_ENTRY(d->entry), composed);
     dt_conf_set_string("plugins/imageio/storage/gallery/file_directory", composed);
     g_free(dir);
@@ -119,30 +120,42 @@ gui_init (dt_imageio_module_storage_t *self)
   }
   d->entry = GTK_ENTRY(widget);
   dt_gui_key_accel_block_on_focus (GTK_WIDGET (d->entry));
-  g_object_set(G_OBJECT(widget), "tooltip-text", _("enter the path where to create the website gallery:\n"
-               "$(ROLL_NAME) - roll of the input image\n"
-               "$(FILE_DIRECTORY) - directory of the input image\n"
-               "$(FILE_NAME) - basename of the input image\n"
-               "$(FILE_EXTENSION) - extension of the input image\n"
-               "$(SEQUENCE) - sequence number\n"
-               "$(YEAR) - year\n"
-               "$(MONTH) - month\n"
-               "$(DAY) - day\n"
-               "$(HOUR) - hour\n"
-               "$(MINUTE) - minute\n"
-               "$(SECOND) - second\n"
-               "$(EXIF_YEAR) - exif year\n"
-               "$(EXIF_MONTH) - exif month\n"
-               "$(EXIF_DAY) - exif day\n"
-               "$(EXIF_HOUR) - exif hour\n"
-               "$(EXIF_MINUTE) - exif minute\n"
-               "$(EXIF_SECOND) - exif second\n"
-               "$(STARS) - star rating\n"
-               "$(LABELS) - colorlabels\n"
-               "$(PICTURES_FOLDER) - pictures folder\n"
-               "$(HOME_FOLDER) - home folder\n"
-               "$(DESKTOP_FOLDER) - desktop folder"
-                                                  ), (char *)NULL);
+
+  dt_gtkentry_completion_spec compl_list[] =
+  {
+    { "ROLL_NAME", _("$(ROLL_NAME) - roll of the input image") },
+    { "FILE_FOLDER", _("$(FILE_FOLDER) - folder containing the input image") },
+    { "FILE_NAME", _("$(FILE_NAME) - basename of the input image") },
+    { "FILE_EXTENSION", _("$(FILE_EXTENSION) - extension of the input image") },
+    { "SEQUENCE", _("$(SEQUENCE) - sequence number") },
+    { "YEAR", _("$(YEAR) - year") },
+    { "MONTH", _("$(MONTH) - month") },
+    { "DAY", _("$(DAY) - day") },
+    { "HOUR", _("$(HOUR) - hour") },
+    { "MINUTE", _("$(MINUTE) - minute") },
+    { "SECOND", _("$(SECOND) - second") },
+    { "EXIF_YEAR", _("$(EXIF_YEAR) - exif year") },
+    { "EXIF_MONTH", _("$(EXIF_MONTH) - exif month") },
+    { "EXIF_DAY", _("$(EXIF_DAY) - exif day") },
+    { "EXIF_HOUR", _("$(EXIF_HOUR) - exif hour") },
+    { "EXIF_MINUTE", _("$(EXIF_MINUTE) - exif minute") },
+    { "EXIF_SECOND", _("$(EXIF_SECOND) - exif second") },
+    { "STARS", _("$(STARS) - star rating") },
+    { "LABELS", _("$(LABELS) - colorlabels") },
+    { "PICTURES_FOLDER", _("$(PICTURES_FOLDER) - pictures folder") },
+    { "HOME", _("$(HOME) - home folder") },
+    { "DESKTOP", _("$(DESKTOP) - desktop folder") },
+    { NULL, NULL }
+  };
+
+  dt_gtkentry_setup_completion(GTK_ENTRY(widget), compl_list);
+
+  char *tooltip_text = dt_gtkentry_build_completion_tooltip_text (
+                         _("enter the path where to put exported images\nrecognized variables:"),
+                         compl_list);
+  g_object_set(G_OBJECT(widget), "tooltip-text", tooltip_text, (char *)NULL);
+  g_free(tooltip_text);
+
   widget = dtgtk_button_new(dtgtk_cairo_paint_directory, 0);
   gtk_widget_set_size_request(widget, 18, 18);
   g_object_set(G_OBJECT(widget), "tooltip-text", _("select directory"), (char *)NULL);
@@ -187,33 +200,34 @@ sort_pos(pair_t *a, pair_t *b)
 }
 
 int
-store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_format_t *format, dt_imageio_module_data_t *fdata, const int num, const int total)
+store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_format_t *format, dt_imageio_module_data_t *fdata,
+       const int num, const int total, const gboolean high_quality)
 {
   dt_imageio_gallery_t *d = (dt_imageio_gallery_t *)sdata;
 
-  char filename[1024]= {0};
-  char dirname[1024]= {0};
-  dt_image_full_path(imgid, dirname, 1024);
+  char filename[DT_MAX_PATH_LEN]= {0};
+  char dirname[DT_MAX_PATH_LEN]= {0};
+  dt_image_full_path(imgid, dirname, DT_MAX_PATH_LEN);
   // we're potentially called in parallel. have sequence number synchronized:
   dt_pthread_mutex_lock(&darktable.plugin_threadsafe);
   {
 
-    char tmp_dir[1024];
+    char tmp_dir[DT_MAX_PATH_LEN];
     dt_variables_expand(d->vp, d->filename, TRUE);
-    g_strlcpy(tmp_dir, dt_variables_get_result(d->vp), 1024);
+    g_strlcpy(tmp_dir, dt_variables_get_result(d->vp), DT_MAX_PATH_LEN);
 
     // if filenamepattern is a directory just let att ${FILE_NAME} as default..
     if ( g_file_test(tmp_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) || ((d->filename+strlen(d->filename)-1)[0]=='/' || (d->filename+strlen(d->filename)-1)[0]=='\\') )
-      snprintf (d->filename+strlen(d->filename), 1024-strlen(d->filename), "/$(FILE_NAME)");
+      snprintf (d->filename+strlen(d->filename), DT_MAX_PATH_LEN-strlen(d->filename), "/$(FILE_NAME)");
 
     // avoid braindead export which is bound to overwrite at random:
     if(total > 1 && !g_strrstr(d->filename, "$"))
     {
-      snprintf(d->filename+strlen(d->filename), 1024-strlen(d->filename), "_$(SEQUENCE)");
+      snprintf(d->filename+strlen(d->filename), DT_MAX_PATH_LEN-strlen(d->filename), "_$(SEQUENCE)");
     }
 
     gchar* fixed_path = dt_util_fix_path(d->filename);
-    g_strlcpy(d->filename, fixed_path, 1024);
+    g_strlcpy(d->filename, fixed_path, DT_MAX_PATH_LEN);
     g_free(fixed_path);
 
     d->vp->filename = dirname;
@@ -221,8 +235,8 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
     d->vp->imgid = imgid;
     d->vp->sequence = num;
     dt_variables_expand(d->vp, d->filename, TRUE);
-    g_strlcpy(filename, dt_variables_get_result(d->vp), 1024);
-    g_strlcpy(dirname, filename, 1024);
+    g_strlcpy(filename, dt_variables_get_result(d->vp), DT_MAX_PATH_LEN);
+    g_strlcpy(dirname, filename, DT_MAX_PATH_LEN);
 
     const char *ext = format->extension(fdata);
     char *c = dirname + strlen(dirname);
@@ -237,7 +251,7 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
     }
 
     // store away dir.
-    snprintf(d->cached_dirname, 1024, "%s", dirname);
+    snprintf(d->cached_dirname, DT_MAX_PATH_LEN, "%s", dirname);
 
     c = filename + strlen(filename);
     for(; c>filename && *c != '.' && *c != '/' ; c--);
@@ -298,17 +312,17 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
     if(c <= relthumbfilename) c = relthumbfilename + strlen(relthumbfilename);
     sprintf(c, "-thumb.%s", ext);
 
-    char subfilename[1024], relsubfilename[256];
-    snprintf(subfilename, 1024, "%s", d->cached_dirname);
+    char subfilename[DT_MAX_PATH_LEN], relsubfilename[256];
+    snprintf(subfilename, DT_MAX_PATH_LEN, "%s", d->cached_dirname);
     char* sc = subfilename + strlen(subfilename);
     sprintf(sc, "/img_%d.html", num);
     sprintf(relsubfilename, "img_%d.html", num);
 
     snprintf(pair->line, 4096,
              "\n"
-             "      <div><a class=\"dia\" href=\"%s\"><span></span><img src=\"%s\" alt=\"img%d\" class=\"img\"/></a>\n"
+             "      <div><a class=\"dia\" rel=\"lightbox[viewer]\" title=\"%s - %s\" href=\"%s\"><span></span><img src=\"%s\" alt=\"img%d\" class=\"img\"/></a>\n"
              "      <h1>%s</h1>\n"
-             "      %s<br/><span class=\"tags\">%s</span></div>\n", relsubfilename, relthumbfilename, num, title?title:"&nbsp;", description?description:"&nbsp;", tags?tags:"&nbsp;");
+             "      %s<br/><span class=\"tags\">%s</span></div>\n", title?title:relfilename, description?description:"&nbsp;", relfilename, relthumbfilename, num, title?title:"&nbsp;", description?description:"&nbsp;", tags?tags:"&nbsp;");
 
     char next[256];
     sprintf(next, "img_%d.html", (num)%total+1);
@@ -316,35 +330,37 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
     char prev[256];
     sprintf(prev, "img_%d.html", (num==1)?total:num-1);
 
-    FILE* subfile = fopen(subfilename, "wb");
-    fprintf(subfile,
-          "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
-          "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
-          "  <head>\n"
-          "    <meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\" />\n"
-          "    <link rel=\"shortcut icon\" href=\"style/favicon.ico\" />\n"
-          "    <link rel=\"stylesheet\" href=\"style/style.css\" type=\"text/css\" />\n"
-          "    <title>%s</title>\n"
-          "  </head>\n"
-          "  <body>\n"
-          "    <div class=\"title\"><a href=\"index.html\">%s</a></div>\n"
-          "    <div class=\"page\">\n"
-          "      <div style=\"width: 692px; max-width: 692px; height: 10px;\">\n"
-          "        <a style=\"float: left;\" href=\"%s\"><h1>prev</h1></a>\n" 
-          "        <a style=\"float: right;\"href=\"%s\"><h1>next</h1></a>\n"
-          "      </div>\n"
-          "      <a href=\"%s\"><img src=\"%s\" width=\"692\" class=\"img\"/></a>\n"
-          "      %s<br/><span class=\"tags\">%s</span></div>\n"
-          "      <p style=\"clear:both;\"></p>\n"
-          "    </div>\n"
-          "    <div class=\"footer\">\n"
-          "      created with darktable "PACKAGE_VERSION"\n"
-          "    </div>\n"
-          "  </body>\n"
-          "</html>\n",
-          relfilename, title?title:relfilename, prev, next, relfilename, relfilename, description?description:"&nbsp;", tags?tags:"&nbsp;");
-    fclose(subfile);
+    /* Becomes unecessary with the Lightbox image viewer overlay
 
+        FILE* subfile = fopen(subfilename, "wb");
+        fprintf(subfile,
+              "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
+              "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
+              "  <head>\n"
+              "    <meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\" />\n"
+              "    <link rel=\"shortcut icon\" href=\"style/favicon.ico\" />\n"
+              "    <link rel=\"stylesheet\" href=\"style/style.css\" type=\"text/css\" />\n"
+              "    <title>%s</title>\n"
+              "  </head>\n"
+              "  <body>\n"
+              "    <div class=\"title\"><a href=\"index.html\">%s</a></div>\n"
+              "    <div class=\"page\">\n"
+              "      <div style=\"width: 692px; max-width: 692px; height: 10px;\">\n"
+              "        <a style=\"float: left;\" href=\"%s\"><h1>prev</h1></a>\n"
+              "        <a style=\"float: right;\"href=\"%s\"><h1>next</h1></a>\n"
+              "      </div>\n"
+              "      <a href=\"%s\"><img src=\"%s\" width=\"692\" class=\"img\"/></a>\n"
+              "      %s<br/><span class=\"tags\">%s</span></div>\n"
+              "      <p style=\"clear:both;\"></p>\n"
+              "    </div>\n"
+              "    <div class=\"footer\">\n"
+              "      created with darktable "PACKAGE_VERSION"\n"
+              "    </div>\n"
+              "  </body>\n"
+              "</html>\n",
+              relfilename, title?title:relfilename, prev, next, relfilename, relfilename, description?description:"&nbsp;", tags?tags:"&nbsp;");
+        fclose(subfile);
+    */
     pair->pos = num;
     g_free(title);
     g_free(description);
@@ -354,7 +370,7 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
   dt_pthread_mutex_unlock(&darktable.plugin_threadsafe);
 
   /* export image to file */
-  if(dt_imageio_export(imgid, filename, format, fdata) != 0)
+  if(dt_imageio_export(imgid, filename, format, fdata, high_quality) != 0)
   {
     fprintf(stderr, "[imageio_storage_gallery] could not export to file: `%s'!\n", filename);
     dt_control_log(_("could not export to file `%s'!"), filename);
@@ -373,7 +389,7 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
   if(c <= filename || *c=='/') c = filename + strlen(filename);
   const char *ext = format->extension(fdata);
   sprintf(c,"-thumb.%s",ext);
-  if(dt_imageio_export(imgid, filename, format, fdata) != 0)
+  if(dt_imageio_export(imgid, filename, format, fdata, FALSE) != 0)
   {
     fprintf(stderr, "[imageio_storage_gallery] could not export to file: `%s'!\n", filename);
     dt_control_log(_("could not export to file `%s'!"), filename);
@@ -393,8 +409,8 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
 static void
 copy_res(const char *src, const char *dst)
 {
-  char share[1024];
-  dt_util_get_datadir(share, 1024);
+  char share[DT_MAX_PATH_LEN];
+  dt_loc_get_datadir(share, DT_MAX_PATH_LEN);
   gchar *sourcefile = g_build_filename(share, src, NULL);
   char* content = NULL;
   FILE *fin = fopen(sourcefile, "rb");
@@ -428,8 +444,8 @@ void
 finalize_store(dt_imageio_module_storage_t *self, void *dd)
 {
   dt_imageio_gallery_t *d = (dt_imageio_gallery_t *)dd;
-  char filename[1024];
-  snprintf(filename, 1024, "%s", d->cached_dirname);
+  char filename[DT_MAX_PATH_LEN];
+  snprintf(filename, DT_MAX_PATH_LEN, "%s", d->cached_dirname);
   char *c = filename + strlen(filename);
 
   // also create style/ subdir:
@@ -439,6 +455,44 @@ finalize_store(dt_imageio_module_storage_t *self, void *dd)
   copy_res("/style/style.css", filename);
   sprintf(c, "/style/favicon.ico");
   copy_res("/style/favicon.ico", filename);
+  sprintf(c, "/style/bullet.gif");
+  copy_res("/style/bullet.gif", filename);
+  sprintf(c, "/style/close.gif");
+  copy_res("/style/close.gif", filename);
+  sprintf(c, "/style/closelabel.gif");
+  copy_res("/style/closelabel.gif", filename);
+  sprintf(c, "/style/donate-button.gif");
+  copy_res("/style/donate-button.gif", filename);
+  sprintf(c, "/style/download-icon.gif");
+  copy_res("/style/download-icon.gif", filename);
+  sprintf(c, "/style/image-1.jpg");
+  copy_res("/style/image-1.jpg", filename);
+  sprintf(c, "/style/lightbox.css");
+  copy_res("/style/lightbox.css", filename);
+  sprintf(c, "/style/loading.gif");
+  copy_res("/style/loading.gif", filename);
+  sprintf(c, "/style/nextlabel.gif");
+  copy_res("/style/nextlabel.gif", filename);
+  sprintf(c, "/style/prevlabel.gif");
+  copy_res("/style/prevlabel.gif", filename);
+  sprintf(c, "/style/thumb-1.jpg");
+  copy_res("/style/thumb-1.jpg", filename);
+
+  // create subdir   js for lightbox2 viewer scripts
+  sprintf(c, "/js");
+  g_mkdir_with_parents(filename, 0755);
+  sprintf(c, "/js/builder.js");
+  copy_res("/js/builder.js", filename);
+  sprintf(c, "/js/effects.js");
+  copy_res("/js/effects.js", filename);
+  sprintf(c, "/js/lightbox.js");
+  copy_res("/js/lightbox.js", filename);
+  sprintf(c, "/js/lightbox-web.js");
+  copy_res("/js/lightbox-web.js", filename);
+  sprintf(c, "/js/prototype.js");
+  copy_res("/js/prototype.js", filename);
+  sprintf(c, "/js/scriptaculous.js");
+  copy_res("/js/scriptaculous.js", filename);
 
   sprintf(c, "/index.html");
 
@@ -453,6 +507,10 @@ finalize_store(dt_imageio_module_storage_t *self, void *dd)
           "    <meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\" />\n"
           "    <link rel=\"shortcut icon\" href=\"style/favicon.ico\" />\n"
           "    <link rel=\"stylesheet\" href=\"style/style.css\" type=\"text/css\" />\n"
+          "    <link rel=\"stylesheet\" href=\"style/lightbox.css\" type=\"text/css\" media=\"screen\" />"
+          "    <script type=\"text/javascript\" src=\"js/prototype.js\"></script>\n"
+          "    <script type=\"text/javascript\" src=\"js/scriptaculous.js?load=effects,builder\"></script>\n"
+          "    <script type=\"text/javascript\" src=\"js/lightbox.js\"></script>\n"
           "    <title>%s</title>\n"
           "  </head>\n"
           "  <body>\n"
@@ -495,10 +553,10 @@ get_params(dt_imageio_module_storage_t *self, int* size)
   d->l = NULL;
   dt_variables_params_init(&d->vp);
   const char *text = gtk_entry_get_text(GTK_ENTRY(g->entry));
-  g_strlcpy(d->filename, text, 1024);
+  g_strlcpy(d->filename, text, DT_MAX_PATH_LEN);
   dt_conf_set_string("plugins/imageio/storage/gallery/file_directory", d->filename);
   text = gtk_entry_get_text(GTK_ENTRY(g->title_entry));
-  g_strlcpy(d->title, text, 1024);
+  g_strlcpy(d->title, text, DT_MAX_PATH_LEN);
   dt_conf_set_string("plugins/imageio/storage/gallery/title", d->title);
   return d;
 }
@@ -524,4 +582,6 @@ set_params(dt_imageio_module_storage_t *self, void *params, int size)
   return 0;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
