@@ -3,6 +3,8 @@
     This file is part of darktable,
     copyright (c) 2009--2010 johannes hanika
     copyright (c) 2011 Sergey Astanin
+    copyright (c) 2012 Henrik Andersson
+
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,9 +22,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include "bauhaus/bauhaus.h"
 #include "develop/imageop.h"
 #include "control/control.h"
-#include "dtgtk/slider.h"
+#include "common/opencl.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include <gtk/gtk.h>
@@ -54,8 +57,8 @@ typedef struct dt_iop_colorcontrast_gui_data_t
   // whatever you need to make your gui happy.
   // stored in self->gui_data
   GtkVBox *vbox;
-  GtkDarktableSlider *a_scale; // this is needed by gui_update
-  GtkDarktableSlider *b_scale;
+  GtkWidget *a_scale; // this is needed by gui_update
+  GtkWidget *b_scale;
 }
 dt_iop_colorcontrast_gui_data_t;
 
@@ -73,9 +76,7 @@ dt_iop_colorcontrast_data_t;
 
 typedef struct dt_iop_colorcontrast_global_data_t
 {
-  // this is optionally stored in self->global_data
-  // and can be used to alloc globally needed stuff
-  // which is needed in gui mode and during processing.
+  int kernel_colorcontrast;
 }
 dt_iop_colorcontrast_global_data_t;
 
@@ -90,7 +91,7 @@ const char *name()
 int
 flags()
 {
-  return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_SUPPORTS_BLENDING;
+  return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_SUPPORTS_BLENDING | IOP_FLAGS_ALLOW_TILING;
 }
 
 // where does it appear in the gui?
@@ -100,6 +101,7 @@ groups ()
   return IOP_GROUP_COLOR;
 }
 
+#if 0 // BAUHAUS doesnt support keyaccels yet...
 void init_key_accels(dt_iop_module_so_t *self)
 {
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "green vs magenta"));
@@ -109,7 +111,7 @@ void init_key_accels(dt_iop_module_so_t *self)
 void connect_key_accels(dt_iop_module_t *self)
 {
   dt_iop_colorcontrast_gui_data_t *g =
-      (dt_iop_colorcontrast_gui_data_t*)self->gui_data;
+    (dt_iop_colorcontrast_gui_data_t*)self->gui_data;
 
   dt_accel_connect_slider_iop(self, "green vs magenta",
                               GTK_WIDGET(g->a_scale));
@@ -117,6 +119,8 @@ void connect_key_accels(dt_iop_module_t *self)
                               GTK_WIDGET(g->b_scale));
 }
 
+#endif
+
 /** modify regions of interest (optional, per pixel ops don't need this) */
 // void modify_roi_out(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, dt_iop_roi_t *roi_out, const dt_iop_roi_t *roi_in);
 // void modify_roi_in(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, const dt_iop_roi_t *roi_out, dt_iop_roi_t *roi_in);
@@ -155,8 +159,64 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
     }
   }
   _mm_sfence();
+
+  if(piece->pipe->mask_display)
+    dt_iop_alpha_copy(i, o, roi_out->width, roi_out->height);
 }
 
+
+#ifdef HAVE_OPENCL
+int
+process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
+{
+  dt_iop_colorcontrast_data_t *data = (dt_iop_colorcontrast_data_t *)piece->data;
+  dt_iop_colorcontrast_global_data_t *gd = (dt_iop_colorcontrast_global_data_t *)self->data;
+  cl_int err = -999;
+
+  const int devid = piece->pipe->devid;
+  const int width = roi_in->width;
+  const int height = roi_in->height;
+
+  float scale[4] =  { 1.0f, data->a_steepness, data->b_steepness, 1.0f };
+  float offset[4] = { 0.0f, data->a_offset, data->b_offset, 0.0f };
+
+  size_t sizes[] = { ROUNDUPWD(width), ROUNDUPHT(height), 1};
+
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorcontrast, 0, sizeof(cl_mem), (void *)&dev_in);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorcontrast, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorcontrast, 2, sizeof(int), (void *)&width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorcontrast, 3, sizeof(int), (void *)&height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorcontrast, 4, 4*sizeof(float), (void *)&scale);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorcontrast, 5, 4*sizeof(float), (void *)&offset);
+  err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_colorcontrast, sizes);
+
+  if(err != CL_SUCCESS) goto error;
+  return TRUE;
+
+error:
+  dt_print(DT_DEBUG_OPENCL, "[opencl_colorcontrast] couldn't enqueue kernel! %d\n", err);
+  return FALSE;
+}
+#endif
+
+
+void init_global(dt_iop_module_so_t *module)
+{
+  const int program = 8; // extended.cl, from programs.conf
+  dt_iop_colorcontrast_global_data_t *gd = (dt_iop_colorcontrast_global_data_t *)malloc(sizeof(dt_iop_colorcontrast_global_data_t));
+  module->data = gd;
+  gd->kernel_colorcontrast = dt_opencl_create_kernel(program, "colorcontrast");
+}
+
+void cleanup_global(dt_iop_module_so_t *module)
+{
+  dt_iop_colorcontrast_global_data_t *gd = (dt_iop_colorcontrast_global_data_t *)module->data;
+  dt_opencl_free_kernel(gd->kernel_colorcontrast);
+  free(module->data);
+  module->data = NULL;
+}
+
+
 /** optional: if this exists, it will be called to init new defaults if a new image is loaded from film strip mode. */
 void reload_defaults(dt_iop_module_t *module)
 {
@@ -174,14 +234,12 @@ void reload_defaults(dt_iop_module_t *module)
 /** init, cleanup, commit to pipeline */
 void init(dt_iop_module_t *module)
 {
-  // we don't need global data:
-  module->data = NULL; //malloc(sizeof(dt_iop_colorcontrast_global_data_t));
   module->params = malloc(sizeof(dt_iop_colorcontrast_params_t));
   module->default_params = malloc(sizeof(dt_iop_colorcontrast_params_t));
   // our module is disabled by default
   module->default_enabled = 0;
   // we are pretty late in the pipe:
-  module->priority = 784; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 788; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_colorcontrast_params_t);
   module->gui_data = NULL;
   // init defaults:
@@ -199,8 +257,8 @@ void cleanup(dt_iop_module_t *module)
   module->gui_data = NULL; // just to be sure
   free(module->params);
   module->params = NULL;
-  free(module->data); // just to be sure
-  module->data = NULL;
+  free(module->default_params);
+  module->default_params = NULL;
 }
 
 /** commit is the synch point between core and gui, so it copies params to pipe data. */
@@ -233,7 +291,7 @@ a_slider_callback(GtkRange *range, dt_iop_module_t *self)
   if(darktable.gui->reset) return;
   dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
   dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
-  p->a_steepness = dtgtk_slider_get_value(g->a_scale);
+  p->a_steepness = dt_bauhaus_slider_get(g->a_scale);
   // let core know of the changes
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
@@ -245,7 +303,7 @@ b_slider_callback(GtkRange *range, dt_iop_module_t *self)
   if(darktable.gui->reset) return;
   dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
   dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
-  p->b_steepness = dtgtk_slider_get_value(g->b_scale);
+  p->b_steepness = dt_bauhaus_slider_get(g->b_scale);
   // let core know of the changes
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
@@ -256,8 +314,8 @@ void gui_update    (dt_iop_module_t *self)
   // let gui slider match current parameters:
   dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
   dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
-  dtgtk_slider_set_value(g->a_scale, p->a_steepness);
-  dtgtk_slider_set_value(g->b_scale, p->b_steepness);
+  dt_bauhaus_slider_set(g->a_scale, p->a_steepness);
+  dt_bauhaus_slider_set(g->b_scale, p->b_steepness);
 }
 
 void gui_init     (dt_iop_module_t *self)
@@ -266,26 +324,27 @@ void gui_init     (dt_iop_module_t *self)
   self->gui_data = malloc(sizeof(dt_iop_colorcontrast_gui_data_t));
   dt_iop_colorcontrast_gui_data_t *g = (dt_iop_colorcontrast_gui_data_t *)self->gui_data;
   dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
-  g->a_scale = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 5.0, 0.01, p->a_steepness, 2));
-  dtgtk_slider_set_label(g->a_scale,_("green vs magenta"));
-  g->b_scale = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 5.0, 0.01, p->b_steepness, 2));
-  dtgtk_slider_set_label(g->b_scale,_("blue vs yellow"));
-  
-  self->widget = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
-  g->vbox = GTK_VBOX(gtk_vbox_new(FALSE, DT_GUI_IOP_MODULE_CONTROL_SPACING));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->vbox), TRUE, TRUE, 5);
-  gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->a_scale), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->b_scale), TRUE, TRUE, 0);
+
+  self->widget = gtk_vbox_new(FALSE, DT_BAUHAUS_SPACE);
+
+  /* a scale */
+  g->a_scale = dt_bauhaus_slider_new_with_range(self, 0.0, 5.0, 0.01, p->a_steepness, 2);
+  dt_bauhaus_widget_set_label(g->a_scale, _("green vs magenta"));
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->a_scale), TRUE, TRUE, 0);
   g_object_set(G_OBJECT(g->a_scale), "tooltip-text",
-	       _("steepness of the a* curve in Lab"), (char *)NULL);
-  g_object_set(G_OBJECT(g->b_scale), "tooltip-text",
-	       _("steepness of the b* curve in Lab"), (char *)NULL);
-
-
+               _("steepness of the a* curve in Lab"), (char *)NULL);
   g_signal_connect(G_OBJECT(g->a_scale), "value-changed",
-		   G_CALLBACK(a_slider_callback), self);
+                   G_CALLBACK(a_slider_callback), self);
+
+
+  /* b scale */
+  g->b_scale = dt_bauhaus_slider_new_with_range(self, 0.0, 5.0, 0.01, p->b_steepness, 2);
+  dt_bauhaus_widget_set_label(g->b_scale, _("blue vs yellow"));
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->b_scale), TRUE, TRUE, 0);
+  g_object_set(G_OBJECT(g->b_scale), "tooltip-text",
+               _("steepness of the b* curve in Lab"), (char *)NULL);
   g_signal_connect(G_OBJECT(g->b_scale), "value-changed",
-		   G_CALLBACK(b_slider_callback), self);
+                   G_CALLBACK(b_slider_callback), self);
 }
 
 void gui_cleanup  (dt_iop_module_t *self)
@@ -302,4 +361,6 @@ void gui_cleanup  (dt_iop_module_t *self)
 // int button_released(struct dt_iop_module_t *self, double x, double y, int which, uint32_t state);
 // int scrolled(dt_iop_module_t *self, double x, double y, int up, uint32_t state);
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
