@@ -1,6 +1,8 @@
 /*
     This file is part of darktable,
     copyright (c) 2011 bruce guenter
+    copyright (c) 2012 henrik andersson
+
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,10 +20,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include "bauhaus/bauhaus.h"
 #include "common/darktable.h"
 #include "control/control.h"
 #include "develop/imageop.h"
-#include "dtgtk/slider.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include <gtk/gtk.h>
@@ -38,7 +40,7 @@ dt_iop_rawdenoise_params_t;
 
 typedef struct dt_iop_rawdenoise_gui_data_t
 {
-  GtkDarktableSlider *threshold;
+  GtkWidget *threshold;
 }
 dt_iop_rawdenoise_gui_data_t;
 
@@ -83,7 +85,7 @@ void init_key_accels(dt_iop_module_so_t *self)
 void connect_key_accels(dt_iop_module_t *self)
 {
   dt_iop_rawdenoise_gui_data_t *g =
-      (dt_iop_rawdenoise_gui_data_t*)self->gui_data;
+    (dt_iop_rawdenoise_gui_data_t*)self->gui_data;
 
   dt_accel_connect_slider_iop(self, "noise threshold",
                               GTK_WIDGET(g->threshold));
@@ -193,7 +195,7 @@ static void wavelet_denoise(const float *const in, float *const out, const dt_io
 
       const float thold = threshold * noise[lev];
 #ifdef _OPENMP
-      #pragma omp parallel for default(none) shared(lev) 
+      #pragma omp parallel for default(none) shared(lev)
 #endif
       for (int i=0; i < halfwidth*halfheight; i++)
       {
@@ -294,7 +296,7 @@ void init(dt_iop_module_t *module)
   module->default_enabled = 0;
 
   // raw denoise must come just before demosaicing.
-  module->priority = 78; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 115; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_rawdenoise_params_t);
   module->gui_data = NULL;
 }
@@ -334,16 +336,16 @@ void gui_update(dt_iop_module_t *self)
   dt_iop_module_t *module = (dt_iop_module_t *)self;
   dt_iop_rawdenoise_gui_data_t *g = (dt_iop_rawdenoise_gui_data_t *)self->gui_data;
   dt_iop_rawdenoise_params_t *p = (dt_iop_rawdenoise_params_t *)module->params;
-  dtgtk_slider_set_value(g->threshold, p->threshold);
+  dt_bauhaus_slider_set(g->threshold, p->threshold);
 }
 
 static void
-threshold_callback (GtkDarktableSlider *slider, gpointer user_data)
+threshold_callback (GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   if(self->dt->gui->reset) return;
   dt_iop_rawdenoise_params_t *p = (dt_iop_rawdenoise_params_t *)self->params;
-  p->threshold = dtgtk_slider_get_value(slider);
+  p->threshold = dt_bauhaus_slider_get(slider);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -355,9 +357,10 @@ void gui_init(dt_iop_module_t *self)
 
   self->widget = GTK_WIDGET(gtk_vbox_new(FALSE, 0));
 
-  g->threshold = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 0.1, 0.001, p->threshold, 3));
+  /* threshold */
+  g->threshold = dt_bauhaus_slider_new_with_range(self, 0.0, 0.1, 0.001, p->threshold, 3);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->threshold), TRUE, TRUE, 0);
-  dtgtk_slider_set_label(g->threshold, _("noise threshold"));
+  dt_bauhaus_widget_set_label(g->threshold, _("noise threshold"));
   g_signal_connect(G_OBJECT(g->threshold), "value-changed", G_CALLBACK(threshold_callback), self);
 }
 
@@ -366,3 +369,6 @@ void gui_cleanup(dt_iop_module_t *self)
   free(self->gui_data);
   self->gui_data = NULL;
 }
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
