@@ -26,14 +26,14 @@
 #include "gui/presets.h"
 #include "develop/develop.h"
 #include "control/control.h"
+#include "bauhaus/bauhaus.h"
 #include "gui/gtk.h"
 #include "common/opencl.h"
+#include "libs/colorpicker.h"
 
-#define DT_GUI_CURVE_EDITOR_INSET 5
+#define DT_GUI_CURVE_EDITOR_INSET 1
 #define DT_GUI_CURVE_INFL .3f
 
-#define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
-
 DT_MODULE(3)
 
 const char *name()
@@ -61,9 +61,27 @@ legacy_params (dt_iop_module_t *self, const void *const old_params, const int ol
   {
     dt_iop_tonecurve_1_params_t *o = (dt_iop_tonecurve_1_params_t *)old_params;
     dt_iop_tonecurve_params_t *n = (dt_iop_tonecurve_params_t *)new_params;
-    dt_iop_tonecurve_params_t *d = (dt_iop_tonecurve_params_t *)self->default_params;
 
-    *n = *d;  // start with a fresh copy of default parameters
+    // start with a fresh copy of default parameters
+    // unfortunately default_params aren't inited at this stage.
+    *n = (dt_iop_tonecurve_params_t)
+    {
+      {
+        {
+          {0.0, 0.0}, {1.0, 1.0}
+        },
+        {
+          {0.0, 0.0}, {0.5, 0.5}, {1.0, 1.0}
+        },
+        {
+          {0.0, 0.0}, {0.5, 0.5}, {1.0, 1.0}
+        }
+      },
+      { 2, 3, 3 },
+      { MONOTONE_HERMITE, MONOTONE_HERMITE, MONOTONE_HERMITE},
+      1,
+      0
+    };
     for (int k=0; k<6; k++) n->tonecurve[ch_L][k].x = o->tonecurve_x[k];
     for (int k=0; k<6; k++) n->tonecurve[ch_L][k].y = o->tonecurve_y[k];
     n->tonecurve_nodes[ch_L] = 6;
@@ -90,13 +108,13 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   const int height = roi_in->height;
   const int autoscale_ab = d->autoscale_ab;
 
-  size_t sizes[] = { ROUNDUP(width, 4), ROUNDUP(height, 4), 1};
+  size_t sizes[] = { ROUNDUPWD(width), ROUNDUPHT(height), 1};
   dev_L = dt_opencl_copy_host_to_device(devid, d->table[ch_L], 256, 256, sizeof(float));
   dev_a = dt_opencl_copy_host_to_device(devid, d->table[ch_a], 256, 256, sizeof(float));
   dev_b = dt_opencl_copy_host_to_device(devid, d->table[ch_b], 256, 256, sizeof(float));
   if (dev_L == NULL || dev_a == NULL || dev_b == NULL) goto error;
 
-  dev_coeffs = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*2, d->unbounded_coeffs);
+  dev_coeffs = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*3, d->unbounded_coeffs);
   if (dev_coeffs == NULL) goto error;
   dt_opencl_set_kernel_arg(devid, gd->kernel_tonecurve, 0, sizeof(cl_mem), (void *)&dev_in);
   dt_opencl_set_kernel_arg(devid, gd->kernel_tonecurve, 1, sizeof(cl_mem), (void *)&dev_out);
@@ -112,7 +130,7 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   if(err != CL_SUCCESS) goto error;
   dt_opencl_release_mem_object(dev_L);
   dt_opencl_release_mem_object(dev_a);
-  dt_opencl_release_mem_object(dev_b); 
+  dt_opencl_release_mem_object(dev_b);
   dt_opencl_release_mem_object(dev_coeffs);
   return TRUE;
 
@@ -130,6 +148,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 {
   const int ch = piece->colors;
   dt_iop_tonecurve_data_t *d = (dt_iop_tonecurve_data_t *)(piece->data);
+  const float xm = 1.0f/d->unbounded_coeffs[0];
 #ifdef _OPENMP
   #pragma omp parallel for default(none) shared(roi_out, i, o, d) schedule(static)
 #endif
@@ -144,8 +163,8 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
     {
       const float L_in = in[0]/100.0f;
 
-      out[0] = (L_in < 1.0f) ? d->table[ch_L][CLAMP((int)(L_in*0xfffful), 0, 0xffff)] :
-        dt_iop_eval_exp(d->unbounded_coeffs, L_in);
+      out[0] = (L_in < xm) ? d->table[ch_L][CLAMP((int)(L_in*0xfffful), 0, 0xffff)] :
+               dt_iop_eval_exp(d->unbounded_coeffs, L_in);
 
       if (d->autoscale_ab == 0)
       {
@@ -166,6 +185,8 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
         out[1] = in[1] * low_approximation;
         out[2] = in[2] * low_approximation;
       }
+
+      out[3] = in[3];
     }
   }
 }
@@ -173,6 +194,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 void init_presets (dt_iop_module_so_t *self)
 {
   dt_iop_tonecurve_params_t p;
+  memset(&p, 0, sizeof(p));
   p.tonecurve_nodes[ch_L] = 6;
   p.tonecurve_nodes[ch_a] = 7;
   p.tonecurve_nodes[ch_b] = 7;
@@ -236,7 +258,6 @@ void init_presets (dt_iop_module_so_t *self)
 
 void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
-  // pull in new params to gegl
   dt_iop_tonecurve_data_t *d = (dt_iop_tonecurve_data_t *)(piece->data);
   dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)p1;
   for(int ch=0; ch<ch_max; ch++)
@@ -249,7 +270,7 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
       d->curve_nodes[ch] = p->tonecurve_nodes[ch];
       d->curve_type[ch] = p->tonecurve_type[ch];
       for(int k=0; k<p->tonecurve_nodes[ch]; k++)
-	(void)dt_draw_curve_add_point(d->curve[ch], p->tonecurve[ch][k].x, p->tonecurve[ch][k].y);
+        (void)dt_draw_curve_add_point(d->curve[ch], p->tonecurve[ch][k].x, p->tonecurve[ch][k].y);
     }
     else
     {
@@ -265,11 +286,13 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   d->autoscale_ab = p->tonecurve_autoscale_ab;
 
   // now the extrapolation stuff (for L curve only):
-  const float x[4] = {0.7f, 0.8f, 0.9f, 1.0f};
+  const float xm = p->tonecurve[ch_L][p->tonecurve_nodes[ch_L]-1].x;
+  const float x[4] = {0.7f*xm, 0.8f*xm, 0.9f*xm, 1.0f*xm};
   const float y[4] = {d->table[ch_L][CLAMP((int)(x[0]*0x10000ul), 0, 0xffff)],
                       d->table[ch_L][CLAMP((int)(x[1]*0x10000ul), 0, 0xffff)],
                       d->table[ch_L][CLAMP((int)(x[2]*0x10000ul), 0, 0xffff)],
-                      d->table[ch_L][CLAMP((int)(x[3]*0x10000ul), 0, 0xffff)]};
+                      d->table[ch_L][CLAMP((int)(x[3]*0x10000ul), 0, 0xffff)]
+                     };
   dt_iop_estimate_exp(x, y, 4, d->unbounded_coeffs);
 }
 
@@ -286,7 +309,7 @@ void init_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
     d->curve_nodes[ch] = default_params->tonecurve_nodes[ch];
     d->curve_type[ch] = default_params->tonecurve_type[ch];
     for(int k=0; k<default_params->tonecurve_nodes[ch]; k++)
-	(void)dt_draw_curve_add_point(d->curve[ch], default_params->tonecurve[ch][k].x, default_params->tonecurve[ch][k].y);
+      (void)dt_draw_curve_add_point(d->curve[ch], default_params->tonecurve[ch][k].x, default_params->tonecurve[ch][k].y);
   }
   for(int k=0; k<0x10000; k++) d->table[ch_L][k] = 100.0f*k/0x10000; // identity for L
   for(int k=0; k<0x10000; k++) d->table[ch_a][k] = 256.0f*k/0x10000 - 128.0f; // identity for a
@@ -305,7 +328,7 @@ void gui_update(struct dt_iop_module_t *self)
 {
   dt_iop_tonecurve_gui_data_t *g = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
   dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
-  gtk_toggle_button_set_active(g->autoscale_ab, p->tonecurve_autoscale_ab);
+  dt_bauhaus_combobox_set(g->autoscale_ab, 1-p->tonecurve_autoscale_ab);
   // that's all, gui curve is read directly from params during expose event.
   gtk_widget_queue_draw(self->widget);
 }
@@ -316,24 +339,27 @@ void init(dt_iop_module_t *module)
   module->params = malloc(sizeof(dt_iop_tonecurve_params_t));
   module->default_params = malloc(sizeof(dt_iop_tonecurve_params_t));
   module->default_enabled = 0;
-  module->priority = 607; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 615; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_tonecurve_params_t);
   module->gui_data = NULL;
   dt_iop_tonecurve_params_t tmp = (dt_iop_tonecurve_params_t)
   {
     {
-      {							// three curves (L, a, b) with a number of nodes
-        {0.0, 0.0}, {0.08, 0.08}, {0.4, 0.4}, {0.6, 0.6}, {0.92, 0.92}, {1.0, 1.0}
+      {
+        // three curves (L, a, b) with a number of nodes
+        {0.0, 0.0}, {1.0, 1.0}
       },
       {
-        {0.0, 0.0}, {0.08, 0.08}, {0.3, 0.3}, {0.5, 0.5}, {0.7, 0.7}, {0.92, 0.92}, {1.0, 1.0}
+        {0.0, 0.0}, {0.5, 0.5}, {1.0, 1.0}
       },
       {
-        {0.0, 0.0}, {0.08, 0.08}, {0.3, 0.3}, {0.5, 0.5}, {0.7, 0.7}, {0.92, 0.92}, {1.0, 1.0}
+        {0.0, 0.0}, {0.5, 0.5}, {1.0, 1.0}
       }
     },
-    { 6, 7, 7 },					// number of nodes per curve
-    { CUBIC_SPLINE, CUBIC_SPLINE, CUBIC_SPLINE }, 	// curve types
+    { 2, 3, 3 },					// number of nodes per curve
+    // { CATMULL_ROM, CATMULL_ROM, CATMULL_ROM},  // curve types
+    { MONOTONE_HERMITE, MONOTONE_HERMITE, MONOTONE_HERMITE},
+    // { CUBIC_SPLINE, CUBIC_SPLINE, CUBIC_SPLINE},
     1,							// autoscale_ab
     0
   };
@@ -366,12 +392,12 @@ void cleanup(dt_iop_module_t *module)
 }
 
 static void
-autoscale_ab_callback(GtkRange *range, dt_iop_module_t *self)
+autoscale_ab_callback(GtkWidget *widget, dt_iop_module_t *self)
 {
   if(darktable.gui->reset) return;
   dt_iop_tonecurve_gui_data_t *g = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
   dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
-  p->tonecurve_autoscale_ab = gtk_toggle_button_get_active(g->autoscale_ab);
+  p->tonecurve_autoscale_ab = 1-dt_bauhaus_combobox_get(g->autoscale_ab);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -385,6 +411,54 @@ tab_switch(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointe
   gtk_widget_queue_draw(self->widget);
 }
 
+static gboolean
+area_resized(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  GtkRequisition r;
+  r.width  = widget->allocation.width;
+  r.height = widget->allocation.width;
+  gtk_widget_size_request(widget, &r);
+  return TRUE;
+}
+
+static void
+pick_toggled(GtkToggleButton *togglebutton, dt_iop_module_t *self)
+{
+  self->request_color_pick = gtk_toggle_button_get_active(togglebutton);
+  if(darktable.gui->reset) return;
+
+  /* set the area sample size*/
+  if (self->request_color_pick)
+    dt_lib_colorpicker_set_point(darktable.lib, 0.5, 0.5);
+
+  if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
+  dt_iop_request_focus(self);
+}
+
+
+static gboolean scrolled(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
+{
+  dt_iop_module_t *self = (dt_iop_module_t *)user_data;
+  dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
+  dt_iop_tonecurve_gui_data_t *c = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
+
+  int ch = c->channel;
+  dt_iop_tonecurve_node_t *tonecurve = p->tonecurve[ch];
+  int autoscale_ab = p->tonecurve_autoscale_ab;
+
+  // if autoscale_ab is on: do not modify a and b curves
+  if (autoscale_ab && ch != ch_L) return TRUE;
+
+  if(c->selected >= 0)
+  {
+    if(event->direction == GDK_SCROLL_UP  ) tonecurve[c->selected].y = MAX(0.0f, tonecurve[c->selected].y + 0.001f);
+    if(event->direction == GDK_SCROLL_DOWN) tonecurve[c->selected].y = MIN(1.0f, tonecurve[c->selected].y - 0.001f);
+    dt_dev_add_history_item(darktable.develop, self, TRUE);
+    gtk_widget_queue_draw(widget);
+  }
+  return TRUE;
+}
+
 
 void gui_init(struct dt_iop_module_t *self)
 {
@@ -397,20 +471,17 @@ void gui_init(struct dt_iop_module_t *self)
     c->minmax_curve[ch] = dt_draw_curve_new(0.0, 1.0, p->tonecurve_type[ch]);
     c->minmax_curve_nodes[ch] = p->tonecurve_nodes[ch];
     c->minmax_curve_type[ch] = p->tonecurve_type[ch];
-    for(int k=0; k<p->tonecurve_nodes[ch]; k++) (void)dt_draw_curve_add_point(c->minmax_curve[ch], p->tonecurve[ch][k].x, p->tonecurve[ch][k].y);
+    for(int k=0; k<p->tonecurve_nodes[ch]; k++)
+      (void)dt_draw_curve_add_point(c->minmax_curve[ch], p->tonecurve[ch][k].x, p->tonecurve[ch][k].y);
   }
 
   c->channel = ch_L;
   c->mouse_x = c->mouse_y = -1.0;
   c->selected = -1;
-  c->selected_offset = c->selected_y = c->selected_min = c->selected_max = 0.0;
-  c->dragging = 0;
-  c->x_move = -1;
 
-  self->widget = GTK_WIDGET(gtk_vbox_new(FALSE, 0));
+  self->widget = gtk_vbox_new(FALSE, DT_BAUHAUS_SPACE);
 
   // tabs
-  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
   c->channel_tabs = GTK_NOTEBOOK(gtk_notebook_new());
 
   gtk_notebook_append_page(GTK_NOTEBOOK(c->channel_tabs), GTK_WIDGET(gtk_hbox_new(FALSE,0)), gtk_label_new(_("  L  ")));
@@ -425,18 +496,26 @@ void gui_init(struct dt_iop_module_t *self)
 
   g_object_set(G_OBJECT(c->channel_tabs), "homogeneous", TRUE, (char *)NULL);
 
-  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(c->channel_tabs), TRUE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), hbox, FALSE, FALSE, 0);
+  GtkWidget *tb = dtgtk_togglebutton_new(dtgtk_cairo_paint_colorpicker, CPF_STYLE_FLAT);
+  g_object_set(G_OBJECT(tb), "tooltip-text", _("pick gui color from image"), (char *)NULL);
+
+  GtkWidget *notebook = gtk_hbox_new(FALSE,0);
+  gtk_box_pack_start(GTK_BOX(notebook), GTK_WIDGET(c->channel_tabs), FALSE, FALSE, 0);
+  gtk_box_pack_end(GTK_BOX(notebook), GTK_WIDGET(tb), FALSE, FALSE, 0);
+
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), vbox, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(notebook), TRUE, TRUE, 0);
 
   g_signal_connect(G_OBJECT(c->channel_tabs), "switch_page",
                    G_CALLBACK (tab_switch), self);
 
-
   c->area = GTK_DRAWING_AREA(gtk_drawing_area_new());
-  GtkWidget *asp = gtk_aspect_frame_new(NULL, 0.5, 0.5, 1.0, TRUE);
-  gtk_box_pack_start(GTK_BOX(self->widget), asp, TRUE, TRUE, 0);
-  gtk_container_add(GTK_CONTAINER(asp), GTK_WIDGET(c->area));
-  gtk_drawing_area_size(c->area, 258, 258);
+  //GtkWidget *asp = gtk_aspect_frame_new(NULL, 0.5, 0.5, 1.0, FALSE);//TRUE);
+  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(c->area), TRUE, TRUE, 0);
+  // gtk_box_pack_start(GTK_BOX(vbox), asp, TRUE, TRUE, 0);
+  // gtk_container_add(GTK_CONTAINER(asp), GTK_WIDGET(c->area));
+  gtk_drawing_area_size(c->area, 0, 258);
   g_object_set (GTK_OBJECT(c->area), "tooltip-text", _("double click to reset curve"), (char *)NULL);
 
   gtk_widget_add_events(GTK_WIDGET(c->area), GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_LEAVE_NOTIFY_MASK);
@@ -444,28 +523,30 @@ void gui_init(struct dt_iop_module_t *self)
                     G_CALLBACK (dt_iop_tonecurve_expose), self);
   g_signal_connect (G_OBJECT (c->area), "button-press-event",
                     G_CALLBACK (dt_iop_tonecurve_button_press), self);
-  g_signal_connect (G_OBJECT (c->area), "button-release-event",
-                    G_CALLBACK (dt_iop_tonecurve_button_release), self);
   g_signal_connect (G_OBJECT (c->area), "motion-notify-event",
                     G_CALLBACK (dt_iop_tonecurve_motion_notify), self);
   g_signal_connect (G_OBJECT (c->area), "leave-notify-event",
                     G_CALLBACK (dt_iop_tonecurve_leave_notify), self);
   g_signal_connect (G_OBJECT (c->area), "enter-notify-event",
                     G_CALLBACK (dt_iop_tonecurve_enter_notify), self);
+  g_signal_connect (G_OBJECT (c->area), "configure-event",
+                    G_CALLBACK (area_resized), self);
+  g_signal_connect (G_OBJECT(tb), "toggled",
+                    G_CALLBACK (pick_toggled), self);
+  g_signal_connect (G_OBJECT (c->area), "scroll-event",
+                    G_CALLBACK (scrolled), self);
 
-  c->autoscale_ab  = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("auto scale chroma")));
-  gtk_toggle_button_set_active(c->autoscale_ab, p->tonecurve_autoscale_ab);
-  hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(c->autoscale_ab), TRUE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), hbox, FALSE, FALSE, 10);
-  g_object_set (GTK_OBJECT(c->autoscale_ab), "tooltip-text", _("if checked a and b curves have no effect and are\nnot displayed. chroma values (a and b) of each pixel\nare then adjusted based on L curve data."), (char *)NULL);
-  g_signal_connect(G_OBJECT(c->autoscale_ab), "toggled", G_CALLBACK(autoscale_ab_callback), self);
+  c->autoscale_ab = dt_bauhaus_combobox_new(self);
+  dt_bauhaus_widget_set_label(c->autoscale_ab, _("scale chroma"));
+  dt_bauhaus_combobox_add(c->autoscale_ab, _("auto"));
+  dt_bauhaus_combobox_add(c->autoscale_ab, _("manual"));
+  gtk_box_pack_start(GTK_BOX(self->widget), c->autoscale_ab, TRUE, TRUE, 0);
+  g_object_set (GTK_OBJECT(c->autoscale_ab), "tooltip-text", _("if set to auto, a and b curves have no effect and are not displayed. chroma values (a and b) of each pixel are then adjusted based on L curve data."), (char *)NULL);
+  g_signal_connect(G_OBJECT(c->autoscale_ab), "value-changed", G_CALLBACK(autoscale_ab_callback), self);
 
   c->sizegroup = GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));
   gtk_size_group_add_widget(c->sizegroup, GTK_WIDGET(c->area));
   gtk_size_group_add_widget(c->sizegroup, GTK_WIDGET(c->channel_tabs));
-  gtk_size_group_add_widget(c->sizegroup, GTK_WIDGET(c->autoscale_ab));
-
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
@@ -502,6 +583,14 @@ static gboolean dt_iop_tonecurve_leave_notify(GtkWidget *widget, GdkEventCrossin
   return TRUE;
 }
 
+
+static void picker_scale(const float *in, float *out)
+{
+  out[0] = CLAMP(in[0] / 100.0f, 0.0f, 1.0f);
+  out[1] = CLAMP((in[1] + 128.0f)/256.0f, 0.0f, 1.0f);
+  out[2] = CLAMP((in[2] + 128.0f)/256.0f, 0.0f, 1.0f);
+}
+
 static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
@@ -509,20 +598,46 @@ static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event
   dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
 
   const float color_labels_left[3][3] = { { 0.3f, 0.3f,  0.3f  },
-                                          { 0.0f, 0.34f, 0.27f },
-                                          { 0.0f, 0.27f, 0.58f }};
+    { 0.0f, 0.34f, 0.27f },
+    { 0.0f, 0.27f, 0.58f }
+  };
 
   const float color_labels_right[3][3] = {{ 0.3f, 0.3f, 0.3f   },
-                                          { 0.53f, 0.08f, 0.28f},
-                                          { 0.81f, 0.66f, 0.0f }};
+    { 0.53f, 0.08f, 0.28f},
+    { 0.81f, 0.66f, 0.0f }
+  };
 
   int ch = c->channel;
   int nodes = p->tonecurve_nodes[ch];
   dt_iop_tonecurve_node_t *tonecurve = p->tonecurve[ch];
   int autoscale_ab = p->tonecurve_autoscale_ab;
+  if(c->minmax_curve_type[ch] != p->tonecurve_type[ch] || c->minmax_curve_nodes[ch] != p->tonecurve_nodes[ch])
+  {
+    dt_draw_curve_destroy(c->minmax_curve[ch]);
+    c->minmax_curve[ch] = dt_draw_curve_new(0.0, 1.0, p->tonecurve_type[ch]);
+    c->minmax_curve_nodes[ch] = p->tonecurve_nodes[ch];
+    c->minmax_curve_type[ch] = p->tonecurve_type[ch];
+    for(int k=0; k<p->tonecurve_nodes[ch]; k++)
+      (void)dt_draw_curve_add_point(c->minmax_curve[ch], p->tonecurve[ch][k].x, p->tonecurve[ch][k].y);
+  }
+  else
+  {
+    for(int k=0; k<p->tonecurve_nodes[ch]; k++)
+      dt_draw_curve_set_point(c->minmax_curve[ch], k, p->tonecurve[ch][k].x, p->tonecurve[ch][k].y);
+  }
   dt_draw_curve_t *minmax_curve = c->minmax_curve[ch];
+  dt_draw_curve_calc_values(minmax_curve, 0.0, 1.0, DT_IOP_TONECURVE_RES, c->draw_xs, c->draw_ys);
+
+  const float xm = tonecurve[nodes-1].x;
+  const float x[4] = {0.7f*xm, 0.8f*xm, 0.9f*xm, 1.0f*xm};
+  const float y[4] = {c->draw_ys[CLAMP((int)(x[0]*DT_IOP_TONECURVE_RES), 0, DT_IOP_TONECURVE_RES-1)],
+                      c->draw_ys[CLAMP((int)(x[1]*DT_IOP_TONECURVE_RES), 0, DT_IOP_TONECURVE_RES-1)],
+                      c->draw_ys[CLAMP((int)(x[2]*DT_IOP_TONECURVE_RES), 0, DT_IOP_TONECURVE_RES-1)],
+                      c->draw_ys[CLAMP((int)(x[3]*DT_IOP_TONECURVE_RES), 0, DT_IOP_TONECURVE_RES-1)]
+                     };
+  float unbounded_coeffs[3];
+  dt_iop_estimate_exp(x, y, 4, unbounded_coeffs);
 
-  for(int k=0; k<nodes; k++) dt_draw_curve_set_point(minmax_curve, k, tonecurve[k].x, tonecurve[k].y);
   const int inset = DT_GUI_CURVE_EDITOR_INSET;
   int width = widget->allocation.width, height = widget->allocation.height;
   cairo_surface_t *cst = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
@@ -564,39 +679,15 @@ static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event
     {
       const float f = (cells-1-j+i)/(2.0f*cells-2.0f);
       cairo_set_source_rgba (cr,
-          (1.0f-f)*color_labels_left[ch][0] + f*color_labels_right[ch][0],
-          (1.0f-f)*color_labels_left[ch][1] + f*color_labels_right[ch][1],
-          (1.0f-f)*color_labels_left[ch][2] + f*color_labels_right[ch][2],
-          .5f); // blend over to make colors darker, so the overlay is more visible
+                             (1.0f-f)*color_labels_left[ch][0] + f*color_labels_right[ch][0],
+                             (1.0f-f)*color_labels_left[ch][1] + f*color_labels_right[ch][1],
+                             (1.0f-f)*color_labels_left[ch][2] + f*color_labels_right[ch][2],
+                             .5f); // blend over to make colors darker, so the overlay is more visible
       cairo_rectangle(cr, width*i/(float)cells, height*j/(float)cells, width/(float)cells, height/(float)cells);
       cairo_fill(cr);
     }
   }
 
-  if(c->mouse_y > 0 || c->dragging)
-  {
-    float oldx1, oldy1;
-    oldx1 = tonecurve[c->selected].x;
-    oldy1 = tonecurve[c->selected].y;
-
-    if(c->selected == 0) dt_draw_curve_set_point(minmax_curve, 1, tonecurve[1].x, fmaxf(c->selected_min, tonecurve[1].y));
-    if(c->selected == 2) dt_draw_curve_set_point(minmax_curve, 1, tonecurve[1].x, fminf(c->selected_min, fmaxf(0.0, tonecurve[1].y + DT_GUI_CURVE_INFL*(c->selected_min - oldy1))));
-    if(c->selected == nodes-3) dt_draw_curve_set_point(minmax_curve, nodes-2, tonecurve[nodes-2].x, fmaxf(c->selected_min, fminf(1.0, tonecurve[nodes-2].y + DT_GUI_CURVE_INFL*(c->selected_min - oldy1))));
-    if(c->selected == nodes-1) dt_draw_curve_set_point(minmax_curve, nodes-2, tonecurve[nodes-2].x, fminf(c->selected_min, tonecurve[nodes-2].y));
-    dt_draw_curve_set_point(minmax_curve, c->selected, oldx1, c->selected_min);
-    dt_draw_curve_calc_values(minmax_curve, 0.0, 1.0, DT_IOP_TONECURVE_RES, c->draw_min_xs, c->draw_min_ys);
-
-    if(c->selected == 0) dt_draw_curve_set_point(minmax_curve, 1, tonecurve[1].x, fmaxf(c->selected_max, tonecurve[1].y));
-    if(c->selected == 2) dt_draw_curve_set_point(minmax_curve, 1, tonecurve[1].x, fminf(c->selected_max, fmaxf(0.0, tonecurve[1].y + DT_GUI_CURVE_INFL*(c->selected_max - oldy1))));
-    if(c->selected == nodes-3) dt_draw_curve_set_point(minmax_curve, nodes-2, tonecurve[nodes-2].x, fmaxf(c->selected_max, fminf(1.0, tonecurve[nodes-2].y + DT_GUI_CURVE_INFL*(c->selected_max - oldy1))));
-    if(c->selected == nodes-1) dt_draw_curve_set_point(minmax_curve, nodes-2, tonecurve[nodes-2].x, fminf(c->selected_max, tonecurve[nodes-2].y));
-    dt_draw_curve_set_point  (minmax_curve, c->selected, oldx1, c->selected_max);
-    dt_draw_curve_calc_values(minmax_curve, 0.0, 1.0, DT_IOP_TONECURVE_RES, c->draw_max_xs, c->draw_max_ys);
-
-  }
-  for(int k=0; k<nodes; k++) dt_draw_curve_set_point(minmax_curve, k, tonecurve[k].x, tonecurve[k].y);
-  dt_draw_curve_calc_values(minmax_curve, 0.0, 1.0, DT_IOP_TONECURVE_RES, c->draw_xs, c->draw_ys);
-
   // draw grid
   cairo_set_line_width(cr, .4);
   cairo_set_source_rgb (cr, .1, .1, .1);
@@ -605,24 +696,19 @@ static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event
   // if autoscale_ab is on: do not display a and b curves
   if (autoscale_ab && ch != ch_L) goto finally;
 
-  // draw x positions
+  // draw nodes positions
   cairo_set_line_width(cr, 1.);
   cairo_set_source_rgb(cr, 0.6, 0.6, 0.6);
-  const float arrw = 7.0f;
-  for(int k=1; k<nodes-1; k++)
+  cairo_translate(cr, 0, height);
+
+  for(int k=0; k<nodes; k++)
   {
-    cairo_move_to(cr, width*tonecurve[k].x, height+inset-1);
-    cairo_rel_line_to(cr, -arrw*.5f, 0);
-    cairo_rel_line_to(cr, arrw*.5f, -arrw);
-    cairo_rel_line_to(cr, arrw*.5f, arrw);
-    cairo_close_path(cr);
-    if(c->x_move == k) cairo_fill(cr);
-    else               cairo_stroke(cr);
+    cairo_arc(cr, tonecurve[k].x*width, -tonecurve[k].y*height, 3, 0, 2.*M_PI);
+    cairo_stroke(cr);
   }
 
   // draw selected cursor
   cairo_set_line_width(cr, 1.);
-  cairo_translate(cr, 0, height);
 
   // draw lum h istogram in background
   // only if module is enabled
@@ -630,8 +716,18 @@ static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event
   {
     dt_develop_t *dev = darktable.develop;
     float *hist, hist_max;
+    float *raw_mean, *raw_min, *raw_max;
+    float *raw_mean_output;
+    float picker_mean[3], picker_min[3], picker_max[3];
+    char text[256];
+
+    raw_mean = self->picked_color;
+    raw_min = self->picked_color_min;
+    raw_max = self->picked_color_max;
+    raw_mean_output = self->picked_output_color;
+
     hist = dev->histogram_pre_tonecurve;
-    hist_max = dev->histogram_pre_tonecurve_max;
+    hist_max = dev->histogram_linear?dev->histogram_pre_tonecurve_max:logf(1.0 + dev->histogram_pre_tonecurve_max);
     if(hist_max > 0 && ch == ch_L)
     {
       cairo_save(cr);
@@ -640,27 +736,59 @@ static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event
       dt_draw_histogram_8(cr, hist, 3);
       cairo_restore(cr);
     }
+
+    if(self->request_color_pick)
+    {
+      // the global live samples ...
+      GSList *samples = darktable.lib->proxy.colorpicker.live_samples;
+      dt_colorpicker_sample_t *sample = NULL;
+      while(samples)
+      {
+        sample = samples->data;
+
+        picker_scale(sample->picked_color_lab_mean, picker_mean);
+        picker_scale(sample->picked_color_lab_min, picker_min);
+        picker_scale(sample->picked_color_lab_max, picker_max);
+
+        cairo_set_source_rgba(cr, 0.5, 0.7, 0.5, 0.15);
+        cairo_rectangle(cr, width*picker_min[ch], 0, width*fmax(picker_max[ch]-picker_min[ch], 0.0f), -height);
+        cairo_fill(cr);
+        cairo_set_source_rgba(cr, 0.5, 0.7, 0.5, 0.5);
+        cairo_move_to(cr, width*picker_mean[ch], 0);
+        cairo_line_to(cr, width*picker_mean[ch], -height);
+        cairo_stroke(cr);
+
+        samples = g_slist_next(samples);
+      }
+
+      // ... and the local sample
+      picker_scale(raw_mean, picker_mean);
+      picker_scale(raw_min, picker_min);
+      picker_scale(raw_max, picker_max);
+
+      cairo_set_source_rgba(cr, 0.7, 0.5, 0.5, 0.33);
+      cairo_rectangle(cr, width*picker_min[ch], 0, width*fmax(picker_max[ch]-picker_min[ch], 0.0f), -height);
+      cairo_fill(cr);
+      cairo_set_source_rgba(cr, 0.9, 0.7, 0.7, 0.5);
+      cairo_move_to(cr, width*picker_mean[ch], 0);
+      cairo_line_to(cr, width*picker_mean[ch], -height);
+      cairo_stroke(cr);
+
+      snprintf(text, 256, "%.1f → %.1f", raw_mean[ch], raw_mean_output[ch]);
+
+      cairo_set_source_rgb(cr, 0.1, 0.1, 0.1);
+      cairo_select_font_face (cr, "sans-serif", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+      cairo_set_font_size (cr, 0.06*height);
+      cairo_move_to (cr, 0.02f*width, -0.94*height);
+      cairo_show_text(cr, text);
+      cairo_stroke(cr);
+    }
   }
 
-  if(c->mouse_y > 0 || c->dragging)
+  if(c->selected >= 0)
   {
-    // draw min/max, if selected
-    cairo_set_source_rgba(cr, .6, .6, .6, .5);
-    cairo_move_to(cr, 0, - height*c->draw_min_ys[0]);
-    for(int k=1; k<DT_IOP_TONECURVE_RES; k++)   cairo_line_to(cr, k*width/(DT_IOP_TONECURVE_RES-1.0), - height*c->draw_min_ys[k]);
-    cairo_line_to(cr, width, - height*c->draw_min_ys[DT_IOP_TONECURVE_RES-1]);
-    cairo_line_to(cr, width, - height*c->draw_max_ys[DT_IOP_TONECURVE_RES-1]);
-    for(int k=DT_IOP_TONECURVE_RES-2; k>=0; k--) cairo_line_to(cr, k*width/(DT_IOP_TONECURVE_RES-1.0), - height*c->draw_max_ys[k]);
-    cairo_close_path(cr);
-    cairo_fill(cr);
-    // draw mouse focus circle
     cairo_set_source_rgb(cr, .9, .9, .9);
-    const float pos = MAX(0, (DT_IOP_TONECURVE_RES-1) * c->mouse_x/(float)width - 1);
-    int k = (int)pos;
-    const float f = k - pos;
-    if(k >= DT_IOP_TONECURVE_RES-1) k = DT_IOP_TONECURVE_RES - 2;
-    float ht = -height*(f*c->draw_ys[k] + (1-f)*c->draw_ys[k+1]);
-    cairo_arc(cr, c->mouse_x, ht, 4, 0, 2.*M_PI);
+    cairo_arc(cr, tonecurve[c->selected].x*width, -tonecurve[c->selected].y*height, 4, 0, 2.*M_PI);
     cairo_stroke(cr);
   }
 
@@ -669,7 +797,19 @@ static gboolean dt_iop_tonecurve_expose(GtkWidget *widget, GdkEventExpose *event
   cairo_set_source_rgb(cr, .9, .9, .9);
   // cairo_set_line_cap  (cr, CAIRO_LINE_CAP_SQUARE);
   cairo_move_to(cr, 0, -height*c->draw_ys[0]);
-  for(int k=1; k<DT_IOP_TONECURVE_RES; k++) cairo_line_to(cr, k*width/(DT_IOP_TONECURVE_RES-1.0), - height*c->draw_ys[k]);
+  for(int k=1; k<DT_IOP_TONECURVE_RES; k++)
+  {
+    const float xx = k/(DT_IOP_TONECURVE_RES-1.0);
+    if(xx > xm)
+    {
+      const float yy = dt_iop_eval_exp(unbounded_coeffs, xx);
+      cairo_line_to(cr, xx*width, - height*yy);
+    }
+    else
+    {
+      cairo_line_to(cr, xx*width, - height*c->draw_ys[k]);
+    }
+  }
   cairo_stroke(cr);
 
 finally:
@@ -688,12 +828,6 @@ static gboolean dt_iop_tonecurve_motion_notify(GtkWidget *widget, GdkEventMotion
   dt_iop_tonecurve_gui_data_t *c = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
   dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
 
-
-  const float selected_min_left[3] = { 0.0f, 0.0f, 0.0f };
-  const float selected_max_left[3] = { 0.2f, 0.5f, 0.5f };
-  const float selected_min_right[3] = { 0.8f, 0.5f, 0.5f };
-  const float selected_max_right[3] = { 1.0f, 1.0f, 1.0f };
-
   int ch = c->channel;
   int nodes = p->tonecurve_nodes[ch];
   dt_iop_tonecurve_node_t *tonecurve = p->tonecurve[ch];
@@ -704,61 +838,71 @@ static gboolean dt_iop_tonecurve_motion_notify(GtkWidget *widget, GdkEventMotion
 
   const int inset = DT_GUI_CURVE_EDITOR_INSET;
   int height = widget->allocation.height - 2*inset, width = widget->allocation.width - 2*inset;
-  if(!c->dragging) c->mouse_x = CLAMP(event->x - inset, 0, width);
+  c->mouse_x = CLAMP(event->x - inset, 0, width);
   c->mouse_y = CLAMP(event->y - inset, 0, height);
 
-  if(c->dragging)
+  const float mx = c->mouse_x/(float)width;
+  const float my = 1.0f - c->mouse_y/(float)height;
+
+  if(event->state & GDK_BUTTON1_MASK)
   {
-    if(c->x_move >= 0)
+    // got a vertex selected:
+    if(c->selected >= 0)
     {
-      const float mx = CLAMP(event->x - inset, 0, width)/(float)width;
-      if(c->x_move > 0 && c->x_move < nodes-1)
+      tonecurve[c->selected].x = mx;
+      tonecurve[c->selected].y = my;
+
+      // delete vertex if order has changed:
+      if(nodes > 2)
+        if((c->selected > 0 && tonecurve[c->selected-1].x >= mx) ||
+            (c->selected < nodes-1 && tonecurve[c->selected+1].x <= mx))
+        {
+          for(int k=c->selected; k<nodes-1; k++)
+          {
+            tonecurve[k].x = tonecurve[k+1].x;
+            tonecurve[k].y = tonecurve[k+1].y;
+          }
+          c->selected = -2; // avoid re-insertion of that point immediately after this
+          p->tonecurve_nodes[ch] --;
+        }
+      dt_dev_add_history_item(darktable.develop, self, TRUE);
+    }
+    else if(nodes < 20 && c->selected >= -1)
+    {
+      // no vertex was close, create a new one!
+      if(tonecurve[0].x > mx)
+        c->selected = 0;
+      else for(int k=1; k<nodes; k++)
+        {
+          if(tonecurve[k].x > mx)
+          {
+            c->selected = k;
+            break;
+          }
+        }
+      if(c->selected == -1) c->selected = nodes;
+      for(int i=nodes; i>c->selected; i--)
       {
-        const float minx = tonecurve[c->x_move-1].x + 0.001f;
-        const float maxx = tonecurve[c->x_move+1].x - 0.001f;
-        tonecurve[c->x_move].x = fminf(maxx, fmaxf(minx, mx));
+        tonecurve[i].x = tonecurve[i-1].x;
+        tonecurve[i].y = tonecurve[i-1].y;
       }
+      // found a new point
+      tonecurve[c->selected].x = mx;
+      tonecurve[c->selected].y = my;
+      p->tonecurve_nodes[ch] ++;
+      dt_dev_add_history_item(darktable.develop, self, TRUE);
     }
-    else
-    {
-      float f = c->selected_y - (c->mouse_y-c->selected_offset)/height;
-      f = fmaxf(c->selected_min, fminf(c->selected_max, f));
-      if(c->selected == 0) tonecurve[1].y = fmaxf(f, tonecurve[1].y);
-      if(c->selected == 2) tonecurve[1].y = fminf(f, fmaxf(0.0, tonecurve[1].y + DT_GUI_CURVE_INFL*(f - tonecurve[2].y)));
-      if(c->selected == nodes-3) tonecurve[nodes-2].y = fmaxf(f, fminf(1.0, tonecurve[nodes-2].y + DT_GUI_CURVE_INFL*(f - tonecurve[nodes-3].y)));
-      if(c->selected == nodes-1) tonecurve[nodes-2].y = fminf(f, tonecurve[nodes-2].y);
-      tonecurve[c->selected].y = f;
-    }
-    dt_dev_add_history_item(darktable.develop, self, TRUE);
   }
   else
   {
-    if(event->y > height)
-    {
-      c->x_move = 0;
-      const float mx = CLAMP(event->x - inset, 0, width)/(float)width;
-      float dist = fabsf(tonecurve[0].x - mx);
-      for(int k=1; k<nodes; k++)
-      {
-        float d2 = fabsf(tonecurve[k].x - mx);
-        if(d2 < dist)
-        {
-          c->x_move = k;
-          dist = d2;
-        }
-      }
-    }
-    else
-    {
-      c->x_move = -1;
-    }
-    float pos = (event->x - inset)/width;
-    float min = 100.0;
-    int nearest = 0;
+    // minimum area around the node to select it:
+    float min = .04f;
+    min *= min; // comparing against square
+    int nearest = -1;
     for(int k=0; k<nodes; k++)
     {
-      float dist = (pos - tonecurve[k].x);
-      dist *= dist;
+      float dist = (my - tonecurve[k].y)*(my - tonecurve[k].y)
+                   + (mx - tonecurve[k].x)*(mx - tonecurve[k].x);
       if(dist < min)
       {
         min = dist;
@@ -766,29 +910,6 @@ static gboolean dt_iop_tonecurve_motion_notify(GtkWidget *widget, GdkEventMotion
       }
     }
     c->selected = nearest;
-    c->selected_y = tonecurve[c->selected].y;
-    c->selected_offset = c->mouse_y;
-    const float f = 0.8f;
-    if(c->selected == 0)
-    {
-      c->selected_min = selected_min_left[ch];
-      c->selected_max = selected_max_left[ch];
-    }
-    else if(c->selected == nodes-1)
-    {
-      c->selected_min = selected_min_right[ch];
-      c->selected_max = selected_max_right[ch];
-    }
-    else
-    {
-      c->selected_min = fmaxf(c->selected_y - 0.2f, (1.-f)*c->selected_y + f*tonecurve[c->selected-1].y);
-      c->selected_max = fminf(c->selected_y + 0.2f, (1.-f)*c->selected_y + f*tonecurve[c->selected+1].y);
-    }
-    if (ch == 0)
-    {
-      if(c->selected == 1) c->selected_max *= 0.7;
-      if(c->selected == nodes-2) c->selected_min = 1.0 - 0.7*(1.0 - c->selected_min);
-    }
   }
 finally:
   gtk_widget_queue_draw(widget);
@@ -798,50 +919,35 @@ finally:
 static gboolean dt_iop_tonecurve_button_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
+  dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
+  dt_iop_tonecurve_params_t *d = (dt_iop_tonecurve_params_t *)self->factory_params;
+  dt_iop_tonecurve_gui_data_t *c = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
+
+  int ch = c->channel;
+  int autoscale_ab = p->tonecurve_autoscale_ab;
 
   if(event->button == 1 && event->type == GDK_2BUTTON_PRESS)
   {
     // reset current curve
-    dt_iop_tonecurve_params_t *p = (dt_iop_tonecurve_params_t *)self->params;
-    dt_iop_tonecurve_params_t *d = (dt_iop_tonecurve_params_t *)self->factory_params;
-    dt_iop_tonecurve_gui_data_t *c = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
-
-    int ch = c->channel;
-    int nodes = p->tonecurve_nodes[ch];
-    int autoscale_ab = p->tonecurve_autoscale_ab;
-
     // if autoscale_ab is on: allow only reset of L curve
     if (!(autoscale_ab && ch != ch_L))
     {
-      for(int k=0; k<nodes; k++)
+      p->tonecurve_nodes[ch] = d->tonecurve_nodes[ch];
+      p->tonecurve_type[ch] = d->tonecurve_type[ch];
+      for(int k=0; k<d->tonecurve_nodes[ch]; k++)
       {
-          p->tonecurve[ch][k].x = d->tonecurve[ch][k].x;
-          p->tonecurve[ch][k].y = d->tonecurve[ch][k].y;
+        p->tonecurve[ch][k].x = d->tonecurve[ch][k].x;
+        p->tonecurve[ch][k].y = d->tonecurve[ch][k].y;
       }
+      c->selected = -2; // avoid motion notify re-inserting immediately.
       dt_dev_add_history_item(darktable.develop, self, TRUE);
       gtk_widget_queue_draw(self->widget);
     }
-  }
-  // set active point
-  else if(event->button == 1)
-  {
-    dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-    dt_iop_tonecurve_gui_data_t *c = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
-    c->dragging = 1;
-    return TRUE;
-  }
-  return FALSE;
-}
-
-static gboolean dt_iop_tonecurve_button_release(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
-{
-  if(event->button == 1)
-  {
-    dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-    dt_iop_tonecurve_gui_data_t *c = (dt_iop_tonecurve_gui_data_t *)self->gui_data;
-    c->dragging = 0;
     return TRUE;
   }
   return FALSE;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
