@@ -99,7 +99,7 @@ gui_reset (dt_lib_module_t *self)
 int
 position ()
 {
-  return 998;
+  return 997;
 }
 
 void init_key_accels(dt_lib_module_t *self)
@@ -131,6 +131,8 @@ dt_lib_camera_property_t *_lib_property_add_new(dt_lib_camera_t * lib, const gch
     if( (value=dt_camctl_camera_property_get_first_choice(darktable.camctl,NULL,propertyname)) != NULL )
     {
       // We got a value for property lets construct the gui for the property and add values
+      int i=0;
+      const char *current_value = dt_camctl_camera_get_property(darktable.camctl, NULL, propertyname);
       dt_lib_camera_property_t *prop = malloc(sizeof(dt_lib_camera_property_t));
       memset(prop,0,sizeof(dt_lib_camera_property_t));
       prop->name=label;
@@ -144,6 +146,9 @@ dt_lib_camera_property_t *_lib_property_add_new(dt_lib_camera_t * lib, const gch
       do
       {
         gtk_combo_box_append_text(prop->values, g_dgettext("libgphoto2-2", value));
+        if(!strcmp(current_value, g_dgettext("libgphoto2-2", value)))
+          gtk_combo_box_set_active(prop->values, i);
+        i++;
       }
       while( (value=dt_camctl_camera_property_get_next_choice(darktable.camctl,NULL,propertyname)) != NULL );
       lib->gui.properties=g_list_append(lib->gui.properties,prop);
@@ -342,7 +347,7 @@ static void _toggle_capture_mode_clicked(GtkWidget *widget, gpointer user_data)
 }
 
 
-#define BAR_HEIGHT 18
+#define BAR_HEIGHT 18 /* also change in views/capture.c */
 static void _expose_info_bar(dt_lib_module_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t pointerx, int32_t pointery)
 {
   dt_lib_camera_t *lib=(dt_lib_camera_t *)self->data;
@@ -712,4 +717,6 @@ gui_cleanup (dt_lib_module_t *self)
 
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
