@@ -28,6 +28,7 @@ dt_lib_collect_mode_t;
 const char *dt_lib_collect_string[] =
 {
   N_("film roll"),
+  N_("folders"),
   N_("camera"),
   N_("tag"),
   N_("date"),
@@ -43,5 +44,8 @@ const char *dt_lib_collect_string[] =
   N_("aperture"),
   N_("filename")
 };
-const int dt_lib_collect_string_cnt = 15;
+const int dt_lib_collect_string_cnt = 16;
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
