@@ -17,6 +17,7 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "common/darktable.h"
+#include "common/collection.h"
 #include "common/tags.h"
 #include "common/debug.h"
 #include "control/control.h"
@@ -28,8 +29,12 @@
 #include <gdk/gdkkeysyms.h>
 #include <math.h>
 
+#define FLOATING_ENTRY_WIDTH 150
+
 DT_MODULE(1)
 
+static gboolean _lib_tagging_tag_show(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval, GdkModifierType modifier, dt_lib_module_t *self);
+
 typedef struct dt_lib_tagging_t
 {
   char keyword[1024];
@@ -38,6 +43,9 @@ typedef struct dt_lib_tagging_t
   int imgsel;
 
   GtkWidget *attach_button, *detach_button, *new_button, *delete_button;
+
+  GtkWidget *floating_tag_window;
+  int floating_tag_imgid;
 }
 dt_lib_tagging_t;
 
@@ -57,7 +65,7 @@ name ()
 
 uint32_t views()
 {
-  return DT_VIEW_LIGHTTABLE | DT_VIEW_TETHERING;
+  return DT_VIEW_LIGHTTABLE | DT_VIEW_MAP | DT_VIEW_TETHERING;
 }
 
 uint32_t container()
@@ -71,6 +79,7 @@ void init_key_accels(dt_lib_module_t *self)
   dt_accel_register_lib(self, NC_("accel", "detach"), 0, 0);
   dt_accel_register_lib(self, NC_("accel", "new"), 0, 0);
   dt_accel_register_lib(self, NC_("accel", "delete"), 0, 0);
+  dt_accel_register_lib(self, NC_("accel", "tag"), GDK_t, GDK_CONTROL_MASK);
 }
 
 void connect_key_accels(dt_lib_module_t *self)
@@ -81,6 +90,7 @@ void connect_key_accels(dt_lib_module_t *self)
   dt_accel_connect_button_lib(self, "detach", d->detach_button);
   dt_accel_connect_button_lib(self, "new", d->new_button);
   dt_accel_connect_button_lib(self, "delete", d->delete_button);
+  dt_accel_connect_lib(self, "tag", g_cclosure_new(G_CALLBACK(_lib_tagging_tag_show), self, NULL));
 }
 
 static void
@@ -232,15 +242,16 @@ new_button_clicked (GtkButton *button, gpointer user_data)
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_lib_tagging_t *d   = (dt_lib_tagging_t *)self->data;
   const gchar *tag = gtk_entry_get_text(d->entry);
-  /* create new tag */
-  guint tid=0;
-  dt_tag_new(tag, &tid);
 
   /** attach tag to selected images  */
-  dt_tag_attach(tid,-1);
+  dt_tag_attach_string_list(tag, -1);
   dt_image_synch_xmp(-1);
 
   update(self, 1);
+  update(self, 0);
+
+  /** clear input box */
+  gtk_entry_set_text(d->entry, "");
 }
 
 static void
@@ -251,12 +262,8 @@ entry_activated (GtkButton *button, gpointer user_data)
   const gchar *tag = gtk_entry_get_text(d->entry);
   if(!tag || tag[0] == '\0') return;
 
- /* create new tag */
-  guint tid=0;
-  dt_tag_new(tag, &tid);
-
   /** attach tag to selected images  */
-  dt_tag_attach(tid,-1);
+  dt_tag_attach_string_list(tag, -1);
   dt_image_synch_xmp(-1);
 
   update(self, 1);
@@ -356,9 +363,9 @@ static void _lib_tagging_redraw_callback(gpointer instance, gpointer user_data)
 {
   dt_lib_module_t *self =(dt_lib_module_t *)user_data;
   dt_lib_tagging_t *d   = (dt_lib_tagging_t *)self->data;
-  int imgsel = -1; 
+  int imgsel = -1;
   DT_CTL_GET_GLOBAL(imgsel, lib_image_mouse_over_id);
-  if(imgsel != d->imgsel) update (self, 0); 
+  if(imgsel != d->imgsel) update (self, 0);
 }
 
 void
@@ -394,6 +401,7 @@ gui_init (dt_lib_module_t *self)
   gtk_tree_selection_set_mode(gtk_tree_view_get_selection(d->current),
                               GTK_SELECTION_SINGLE);
   gtk_tree_view_set_model(d->current, GTK_TREE_MODEL(liststore));
+  g_object_unref(liststore);
   g_object_set(G_OBJECT(d->current), "tooltip-text", _("attached tags,\ndoubleclick to detach"), (char *)NULL);
   g_signal_connect(G_OBJECT (d->current), "row-activated", G_CALLBACK (detach_activated), (gpointer)self);
   gtk_container_add(GTK_CONTAINER(w), GTK_WIDGET(d->current));
@@ -450,6 +458,7 @@ gui_init (dt_lib_module_t *self)
   gtk_tree_selection_set_mode(gtk_tree_view_get_selection(d->related),
                               GTK_SELECTION_SINGLE);
   gtk_tree_view_set_model(d->related, GTK_TREE_MODEL(liststore));
+  g_object_unref(liststore);
   g_object_set(G_OBJECT(d->related), "tooltip-text", _("related tags,\ndoubleclick to attach"), (char *)NULL);
   g_signal_connect(G_OBJECT (d->related), "row-activated", G_CALLBACK (attach_activated), (gpointer)self);
   gtk_container_add(GTK_CONTAINER(w), GTK_WIDGET(d->related));
@@ -493,3 +502,123 @@ gui_cleanup (dt_lib_module_t *self)
   free(self->data);
   self->data = NULL;
 }
+
+// http://stackoverflow.com/questions/4631388/transparent-floating-gtkentry
+static gboolean
+_lib_tagging_tag_key_press(GtkWidget *entry, GdkEventKey *event, dt_lib_module_t *self)
+{
+  dt_lib_tagging_t *d = (dt_lib_tagging_t*)self->data;
+  switch(event->keyval)
+  {
+    case GDK_Escape:
+      gtk_widget_destroy(d->floating_tag_window);
+      return TRUE;
+    case GDK_Tab:
+      return TRUE;
+    case GDK_Return:
+    case GDK_KP_Enter:
+    {
+      const gchar *tag = gtk_entry_get_text(GTK_ENTRY(entry));
+      /* attach tag to images  */
+      if(d->floating_tag_imgid > 0) // just a single image
+      {
+        dt_tag_attach_string_list(tag, d->floating_tag_imgid);
+        dt_image_synch_xmp(d->floating_tag_imgid);
+      }
+      else // all selected images
+      {
+        GList *selected_images = g_list_first(dt_collection_get_selected(darktable.collection));
+        if(selected_images)
+        {
+          GList *iter = selected_images;
+          do
+          {
+            int imgid = GPOINTER_TO_INT(iter->data);
+            dt_tag_attach_string_list(tag, imgid);
+            dt_image_synch_xmp(imgid);
+          }
+          while( (iter=g_list_next(iter)) !=NULL );
+        }
+        g_list_free(selected_images);
+      }
+      update(self, 1);
+      update(self, 0);
+      gtk_widget_destroy(d->floating_tag_window);
+      return TRUE;
+    }
+  }
+  return FALSE; /* event not handled */
+}
+
+static gboolean
+_lib_tagging_tag_show(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval, GdkModifierType modifier, dt_lib_module_t* self)
+{
+  int mouse_over_id = -1;
+  int zoom = dt_conf_get_int("plugins/lighttable/images_in_row");
+
+  // the order is:
+  // if(zoom == 1) => currently shown image
+  // else if(selection not empty) => slected images
+  // else if(cursor over image) => hovered image
+  // else => return
+  if(zoom == 1 || dt_collection_get_selected_count(darktable.collection) == 0)
+  {
+    DT_CTL_GET_GLOBAL(mouse_over_id, lib_image_mouse_over_id);
+    if(mouse_over_id < 0)
+      return TRUE;
+  }
+
+  dt_lib_tagging_t *d = (dt_lib_tagging_t*)self->data;
+  d->floating_tag_imgid = mouse_over_id;
+
+  gint x, y;
+  gint px, py, w, h;
+  GtkWidget *window = dt_ui_main_window(darktable.gui->ui);
+  GtkWidget *center = dt_ui_center(darktable.gui->ui);
+  gdk_window_get_origin(gtk_widget_get_window(center), &px, &py);
+  gdk_window_get_size(gtk_widget_get_window(center),&w,&h);
+  x = px + 0.5*(w-FLOATING_ENTRY_WIDTH);
+  y = py + h - 50;
+
+  /* put the floating box at the mouse pointer */
+//   gint pointerx, pointery;
+//   gtk_widget_get_pointer(center, &pointerx, &pointery);
+//   x = px + pointerx + 1;
+//   y = py + pointery + 1;
+
+  d->floating_tag_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  /* stackoverflow.com/questions/1925568/how-to-give-keyboard-focus-to-a-pop-up-gtk-window */
+  GTK_WIDGET_SET_FLAGS(d->floating_tag_window, GTK_CAN_FOCUS);
+  gtk_window_set_decorated(GTK_WINDOW(d->floating_tag_window), FALSE);
+  gtk_window_set_has_frame(GTK_WINDOW(d->floating_tag_window), FALSE);
+  gtk_window_set_type_hint(GTK_WINDOW(d->floating_tag_window), GDK_WINDOW_TYPE_HINT_POPUP_MENU);
+  gtk_window_set_transient_for(GTK_WINDOW(d->floating_tag_window), GTK_WINDOW(window));
+  gtk_window_set_opacity(GTK_WINDOW(d->floating_tag_window), 0.8);
+  gtk_window_move(GTK_WINDOW(d->floating_tag_window), x, y);
+
+
+  GtkWidget *entry = gtk_entry_new();
+  gtk_widget_set_size_request(entry, FLOATING_ENTRY_WIDTH, -1);
+  gtk_widget_add_events(entry, GDK_FOCUS_CHANGE_MASK);
+
+  GtkEntryCompletion *completion = gtk_entry_completion_new();
+  gtk_entry_completion_set_model(completion, gtk_tree_view_get_model(GTK_TREE_VIEW(d->related)));
+  gtk_entry_completion_set_text_column(completion, 0);
+  gtk_entry_completion_set_inline_completion(completion, TRUE);
+  gtk_entry_set_completion(GTK_ENTRY(entry), completion);
+
+  gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
+  gtk_container_add(GTK_CONTAINER(d->floating_tag_window), entry);
+  g_signal_connect_swapped(entry, "focus-out-event", G_CALLBACK(gtk_widget_destroy), d->floating_tag_window);
+  g_signal_connect(entry, "key-press-event", G_CALLBACK(_lib_tagging_tag_key_press), self);
+
+  gtk_widget_show_all(d->floating_tag_window);
+  gtk_widget_grab_focus(entry);
+  gtk_window_present(GTK_WINDOW(d->floating_tag_window));
+
+  return TRUE;
+}
+
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
