@@ -45,26 +45,26 @@
 DT_MODULE(1)
 
 static gboolean film_strip_key_accel(GtkAccelGroup *accel_group,
-                                 GObject *acceleratable,
-                                 guint keyval, GdkModifierType modifier,
-                                 gpointer data);
+                                     GObject *acceleratable,
+                                     guint keyval, GdkModifierType modifier,
+                                     gpointer data);
 
 static gboolean zoom_key_accel(GtkAccelGroup *accel_group, GObject *acceleratable,
-                           guint keyval, GdkModifierType modifier,
-                           gpointer data);
+                               guint keyval, GdkModifierType modifier,
+                               gpointer data);
 
 static gboolean export_key_accel_callback(GtkAccelGroup *accel_group,
-                                      GObject *acceleratable, guint keyval,
-                                      GdkModifierType modifier,gpointer data);
+    GObject *acceleratable, guint keyval,
+    GdkModifierType modifier,gpointer data);
 
 static gboolean skip_f_key_accel_callback(GtkAccelGroup *accel_group,
-                                      GObject *acceleratable,
-                                      guint keyval, GdkModifierType modifier,
-                                      gpointer data);
+    GObject *acceleratable,
+    guint keyval, GdkModifierType modifier,
+    gpointer data);
 static gboolean skip_b_key_accel_callback(GtkAccelGroup *accel_group,
-                                      GObject *acceleratable,
-                                      guint keyval, GdkModifierType modifier,
-                                      gpointer data);
+    GObject *acceleratable,
+    guint keyval, GdkModifierType modifier,
+    gpointer data);
 
 /* signal handler for filmstrip image switching */
 static void _view_darkroom_filmstrip_activate_callback(gpointer instance,gpointer user_data);
@@ -110,9 +110,9 @@ void expose(dt_view_t *self, cairo_t *cri, int32_t width_i, int32_t height_i, in
   cairo_set_fill_rule(cri, CAIRO_FILL_RULE_EVEN_ODD);
   cairo_rectangle(cri, 0, 0, width_i, height_i);
   cairo_rectangle(cri,
-      MAX(1.0, width_i -capwd) *.5f,
-      MAX(1.0, height_i-capht) *.5f,
-      MIN(width, width_i-1), MIN(height, height_i-1));
+                  MAX(1.0, width_i -capwd) *.5f,
+                  MAX(1.0, height_i-capht) *.5f,
+                  MIN(width, width_i-1), MIN(height, height_i-1));
   cairo_fill (cri);
   cairo_restore(cri);
 
@@ -122,7 +122,7 @@ void expose(dt_view_t *self, cairo_t *cri, int32_t width_i, int32_t height_i, in
 
   dt_develop_t *dev = (dt_develop_t *)self->data;
 
-  if(dev->gui_synch)
+  if(dev->gui_synch && !dev->image_loading)
   {
     // synch module guis from gtk thread:
     darktable.gui->reset = 1;
@@ -138,7 +138,7 @@ void expose(dt_view_t *self, cairo_t *cri, int32_t width_i, int32_t height_i, in
   }
 
   if(dev->image_dirty || dev->pipe->input_timestamp < dev->preview_pipe->input_timestamp) dt_dev_process_image(dev);
-  if(dev->preview_dirty) dt_dev_process_preview(dev);
+  if(dev->preview_dirty || dev->pipe->input_timestamp > dev->preview_pipe->input_timestamp) dt_dev_process_preview(dev);
 
   dt_pthread_mutex_t *mutex = NULL;
   int wd, ht, stride, closeup;
@@ -158,6 +158,7 @@ void expose(dt_view_t *self, cairo_t *cri, int32_t width_i, int32_t height_i, in
     image_surface_height = height;
     if(image_surface) cairo_surface_destroy(image_surface);
     image_surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, width, height);
+    image_surface_imgid = -1; // invalidate old stuff
   }
   cairo_surface_t *surface;
   cairo_t *cr = cairo_create(image_surface);
@@ -239,17 +240,17 @@ void expose(dt_view_t *self, cairo_t *cri, int32_t width_i, int32_t height_i, in
     cairo_set_source_surface(cri, image_surface, 0, 0);
     cairo_paint(cri);
   }
-  
+
   /* check if we should create a snapshot of view */
   if(darktable.develop->proxy.snapshot.request)
   {
     /* reset the request */
     darktable.develop->proxy.snapshot.request = FALSE;
-    
+
     /* validation of snapshot filename */
     g_assert(darktable.develop->proxy.snapshot.filename != NULL);
-    
-    /* Store current image surface to snapshot file. 
+
+    /* Store current image surface to snapshot file.
        FIXME: add checks so that we dont make snapshots of preview pipe image surface.
     */
     cairo_surface_write_to_png(image_surface, darktable.develop->proxy.snapshot.filename);
@@ -257,7 +258,7 @@ void expose(dt_view_t *self, cairo_t *cri, int32_t width_i, int32_t height_i, in
 
   // Displaying sample areas if enabled
   if(darktable.lib->proxy.colorpicker.live_samples &&
-     darktable.lib->proxy.colorpicker.display_samples)
+      darktable.lib->proxy.colorpicker.display_samples)
   {
     GSList *samples = darktable.lib->proxy.colorpicker.live_samples;
     dt_colorpicker_sample_t *sample = NULL;
@@ -420,10 +421,10 @@ int try_enter(dt_view_t *self)
     if(sqlite3_step(stmt) == SQLITE_ROW)
       selected = sqlite3_column_int(stmt, 0);
     sqlite3_finalize(stmt);
-    
+
     // Leave as selected only the image being edited
     DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "delete from selected_images", NULL, NULL, NULL);
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into selected_images values (?1)", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert or ignore into selected_images values (?1)", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, selected);
     sqlite3_step(stmt);
     sqlite3_finalize(stmt);
@@ -439,8 +440,8 @@ int try_enter(dt_view_t *self)
   // this loads the image from db if needed:
   const dt_image_t *img = dt_image_cache_read_get(darktable.image_cache, selected);
   // get image and check if it has been deleted from disk first!
-  char imgfilename[1024];
-  dt_image_full_path(img->id, imgfilename, 1024);
+  char imgfilename[DT_MAX_PATH_LEN];
+  dt_image_full_path(img->id, imgfilename, DT_MAX_PATH_LEN);
   if(!g_file_test(imgfilename, G_FILE_TEST_IS_REGULAR))
   {
     dt_control_log(_("image `%s' is currently unavailable"), img->filename);
@@ -460,16 +461,11 @@ static void
 select_this_image(const int imgid)
 {
   // select this image, if no multiple selection:
-  int count = 0;
-  sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select count(imgid) from selected_images", -1, &stmt, NULL);
-  if(sqlite3_step(stmt) == SQLITE_ROW)
-    count = sqlite3_column_int(stmt, 0);
-  sqlite3_finalize(stmt);
-  if(count < 2)
+  if(dt_collection_get_selected_count(NULL) < 2)
   {
+    sqlite3_stmt *stmt;
     DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "delete from selected_images", NULL, NULL, NULL);
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into selected_images values (?1)", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert or ignore into selected_images values (?1)", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
     sqlite3_step(stmt);
     sqlite3_finalize(stmt);
@@ -532,6 +528,7 @@ dt_dev_change_image(dt_develop_t *dev, const uint32_t imgid)
   {
     dt_iop_module_t *module = (dt_iop_module_t *)(modules->data);
     dt_iop_reload_defaults(module);
+    dt_iop_gui_update(module);
     modules = g_list_previous(modules);
   }
 
@@ -558,6 +555,9 @@ dt_dev_change_image(dt_develop_t *dev, const uint32_t imgid)
   // to avoid crop/rotate for example to add another history item.
   darktable.gui->reset = 0;
 
+  // Signal develop initialize
+  dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_IMAGE_CHANGED);
+
   // prefetch next few from first selected image on.
   dt_view_filmstrip_prefetch();
 }
@@ -584,7 +584,6 @@ static void _view_darkroom_filmstrip_activate_callback(gpointer instance,gpointe
 static void
 dt_dev_jump_image(dt_develop_t *dev, int diff)
 {
-  char query[1024];
   const gchar *qin = dt_collection_get_query (darktable.collection);
   int offset = 0;
   if(qin)
@@ -597,14 +596,7 @@ dt_dev_jump_image(dt_develop_t *dev, int diff)
       orig_imgid = sqlite3_column_int(stmt, 0);
     sqlite3_finalize(stmt);
 
-    snprintf(query, 1024, "select rowid from (%s) where id=?3", qin);
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), query, -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_INT(stmt, 1,  0);
-    DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, -1);
-    DT_DEBUG_SQLITE3_BIND_INT(stmt, 3, orig_imgid);
-    if(sqlite3_step(stmt) == SQLITE_ROW)
-      offset = sqlite3_column_int(stmt, 0) - 1;
-    sqlite3_finalize(stmt);
+    offset = dt_collection_image_offset (orig_imgid);
 
     DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), qin, -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, offset + diff);
@@ -620,7 +612,8 @@ dt_dev_jump_image(dt_develop_t *dev, int diff)
         return;
       }
 
-      if (!dev->image_loading) {
+      if (!dev->image_loading)
+      {
         dt_view_filmstrip_scroll_to_image(darktable.view_manager, imgid, FALSE);
       }
       dt_dev_change_image(dev, imgid);
@@ -666,6 +659,7 @@ zoom_key_accel(GtkAccelGroup *accel_group,
     default:
       break;
   }
+  dt_control_queue_redraw_center();
   return TRUE;
 }
 
@@ -674,7 +668,7 @@ film_strip_key_accel(GtkAccelGroup *accel_group,
                      GObject *acceleratable, guint keyval,
                      GdkModifierType modifier, gpointer data)
 {
-  dt_lib_module_t *m = darktable.view_manager->proxy.filmstrip.module; 
+  dt_lib_module_t *m = darktable.view_manager->proxy.filmstrip.module;
   gboolean vs = dt_lib_is_visible(m);
   dt_lib_set_visible(m,!vs);
   return TRUE;
@@ -690,22 +684,27 @@ export_key_accel_callback(GtkAccelGroup *accel_group,
   dt_dev_write_history((dt_develop_t *)data);
 
   /* export current image */
-  dt_control_export();
+  int max_width  = dt_conf_get_int ("plugins/lighttable/export/width");
+  int max_height = dt_conf_get_int ("plugins/lighttable/export/height");
+  int format_index = dt_conf_get_int ("plugins/lighttable/export/format");
+  int storage_index = dt_conf_get_int ("plugins/lighttable/export/storage");
+  gboolean high_quality = dt_conf_get_bool("plugins/lighttable/export/high_quality_processing");
+  dt_control_export(max_width, max_height, format_index, storage_index, high_quality);
   return TRUE;
 }
 
 static gboolean skip_f_key_accel_callback(GtkAccelGroup *accel_group,
-                                      GObject *acceleratable, guint keyval,
-                                      GdkModifierType modifier, gpointer data)
+    GObject *acceleratable, guint keyval,
+    GdkModifierType modifier, gpointer data)
 {
   dt_dev_jump_image((dt_develop_t*)data, 1);
   return TRUE;
 }
 
 static gboolean skip_b_key_accel_callback(GtkAccelGroup *accel_group,
-                                      GObject *acceleratable,
-                                      guint keyval, GdkModifierType modifier,
-                                      gpointer data)
+    GObject *acceleratable,
+    guint keyval, GdkModifierType modifier,
+    gpointer data)
 {
   dt_dev_jump_image((dt_develop_t*)data, -1);
   return TRUE;
@@ -720,7 +719,7 @@ static void _darkroom_ui_favorite_presets_popupmenu(GtkWidget *w, gpointer user_
 {
   /* create favorites menu and popup */
   dt_gui_favorite_presets_menu_show();
-  
+
   /* if we got any styles, lets popup menu for selection */
   if (darktable.gui->presets_popup_menu)
   {
@@ -733,10 +732,10 @@ static void _darkroom_ui_favorite_presets_popupmenu(GtkWidget *w, gpointer user_
 static void _darkroom_ui_apply_style_activate_callback(gchar *name)
 {
   dt_control_log(_("applied style `%s' on current image"),name);
- 
+
   /* write current history changes so nothing gets lost */
   dt_dev_write_history(darktable.develop);
-  
+
   /* apply style on image and reload*/
   dt_styles_apply_to_image (name, FALSE, darktable.develop->image_storage.id);
   dt_dev_reload_image(darktable.develop, darktable.develop->image_storage.id);
@@ -754,11 +753,32 @@ static void _darkroom_ui_apply_style_popupmenu(GtkWidget *w, gpointer user_data)
     {
       dt_style_t *style=(dt_style_t *)styles->data;
       GtkWidget *mi=gtk_menu_item_new_with_label(style->name);
+
+      char* items_string = dt_styles_get_item_list_as_string(style->name);
+      gchar* tooltip = NULL;
+
+      if((style->description) && strlen(style->description))
+      {
+        tooltip = g_strconcat("<b><i>", style->description, "</i></b>\n", items_string, NULL);
+      }
+      else
+      {
+        tooltip = g_strdup(items_string);
+      }
+
+      gtk_widget_set_tooltip_markup(mi, tooltip);
+
       gtk_menu_append (GTK_MENU (menu), mi);
       gtk_signal_connect_object (GTK_OBJECT (mi), "activate",
                                  GTK_SIGNAL_FUNC (_darkroom_ui_apply_style_activate_callback),
                                  (gpointer) g_strdup (style->name));
       gtk_widget_show (mi);
+
+      g_free(style->name);
+      g_free(style->description);
+      g_free(style);
+      g_free(items_string);
+      g_free(tooltip);
     }
     while ((styles=g_list_next(styles))!=NULL);
   }
@@ -775,9 +795,9 @@ static void _darkroom_ui_apply_style_popupmenu(GtkWidget *w, gpointer user_data)
 void enter(dt_view_t *self)
 {
   /* connect to ui pipe finished signal for redraw */
-  dt_control_signal_connect(darktable.signals, 
-			    DT_SIGNAL_DEVELOP_UI_PIPE_FINISHED,G_CALLBACK(_darkroom_ui_pipe_finish_signal_callback), 
-			    (gpointer)self);
+  dt_control_signal_connect(darktable.signals,
+                            DT_SIGNAL_DEVELOP_UI_PIPE_FINISHED,G_CALLBACK(_darkroom_ui_pipe_finish_signal_callback),
+                            (gpointer)self);
 
   dt_print(DT_DEBUG_CONTROL, "[run_job+] 11 %f in darkroom mode\n", dt_get_wtime());
   dt_develop_t *dev = (dt_develop_t *)self->data;
@@ -812,15 +832,15 @@ void enter(dt_view_t *self)
 
   /* create quick styles popup menu tool */
   GtkWidget *styles = dtgtk_button_new (dtgtk_cairo_paint_styles,CPF_STYLE_FLAT|CPF_DO_NOT_USE_BORDER);
-  g_signal_connect (G_OBJECT (styles), "clicked", 
-		    G_CALLBACK (_darkroom_ui_apply_style_popupmenu),
-		    NULL);
-  g_object_set (G_OBJECT (styles), "tooltip-text", _("quick access for applying any of your styles"), 
-		(char *)NULL);
+  g_signal_connect (G_OBJECT (styles), "clicked",
+                    G_CALLBACK (_darkroom_ui_apply_style_popupmenu),
+                    NULL);
+  g_object_set (G_OBJECT (styles), "tooltip-text", _("quick access for applying any of your styles"),
+                (char *)NULL);
   dt_view_manager_view_toolbox_add(darktable.view_manager, styles);
 
-  /* 
-   * add IOP modules to plugin list 
+  /*
+   * add IOP modules to plugin list
    */
   // avoid triggering of events before plugin is ready:
   darktable.gui->reset = 1;
@@ -834,11 +854,11 @@ void enter(dt_view_t *self)
     if (!dt_iop_is_hidden(module))
     {
       module->gui_init(module);
-    
+
       /* add module to right panel */
       GtkWidget *expander = dt_iop_gui_get_expander(module);
       dt_ui_container_add_widget(darktable.gui->ui,
-                               DT_UI_CONTAINER_PANEL_RIGHT_CENTER, expander);
+                                 DT_UI_CONTAINER_PANEL_RIGHT_CENTER, expander);
 
       snprintf(option, 1024, "plugins/darkroom/%s/expanded", module->op);
       dt_iop_gui_set_expanded(module, dt_conf_get_bool(option));
@@ -892,10 +912,10 @@ void enter(dt_view_t *self)
   DT_CTL_SET_GLOBAL(dev_zoom_y, zoom_y);
 
   /* connect signal for filmstrip image activate */
-  dt_control_signal_connect(darktable.signals, 
-			    DT_SIGNAL_VIEWMANAGER_FILMSTRIP_ACTIVATE,
-			    G_CALLBACK(_view_darkroom_filmstrip_activate_callback),
-			    self);
+  dt_control_signal_connect(darktable.signals,
+                            DT_SIGNAL_VIEWMANAGER_FILMSTRIP_ACTIVATE,
+                            G_CALLBACK(_view_darkroom_filmstrip_activate_callback),
+                            self);
 
   // prefetch next few from first selected image on.
   dt_view_filmstrip_prefetch();
@@ -905,13 +925,13 @@ void leave(dt_view_t *self)
 {
   /* disconnect from filmstrip image activate */
   dt_control_signal_disconnect(darktable.signals,
-			       G_CALLBACK(_view_darkroom_filmstrip_activate_callback),
-			       (gpointer)self);
+                               G_CALLBACK(_view_darkroom_filmstrip_activate_callback),
+                               (gpointer)self);
 
   /* disconnect from pipe finish signal */
   dt_control_signal_disconnect(darktable.signals,
-			       G_CALLBACK(_darkroom_ui_pipe_finish_signal_callback),
-			       (gpointer)self);
+                               G_CALLBACK(_darkroom_ui_pipe_finish_signal_callback),
+                               (gpointer)self);
 
   // store groups for next time:
   dt_conf_set_int("plugins/darkroom/groups", dt_dev_modulegroups_get(darktable.develop));
@@ -945,7 +965,7 @@ void leave(dt_view_t *self)
   dt_pthread_mutex_lock(&dev->history_mutex);
   dt_dev_pixelpipe_cleanup_nodes(dev->pipe);
   dt_dev_pixelpipe_cleanup_nodes(dev->preview_pipe);
- 
+
   while(dev->history)
   {
     dt_dev_history_item_t *hist = (dt_dev_history_item_t *)(dev->history->data);
@@ -960,8 +980,8 @@ void leave(dt_view_t *self)
   {
     dt_iop_module_t *module = (dt_iop_module_t *)(dev->iop->data);
     if (!dt_iop_is_hidden(module))
-      module->gui_cleanup(module);
- 
+      dt_iop_gui_cleanup_module(module);
+
     dt_dev_cleanup_module_accels(module);
     module->accel_closures = NULL;
     dt_iop_cleanup_module(module) ;
@@ -980,7 +1000,7 @@ void mouse_leave(dt_view_t *self)
   dt_develop_t *dev = (dt_develop_t *)self->data;
   int32_t mouse_over_id = dev->image_storage.id;
   DT_CTL_SET_GLOBAL(lib_image_mouse_over_id, mouse_over_id);
- 
+
   // reset any changes the selected plugin might have made.
   dt_control_change_cursor(GDK_LEFT_PTR);
 }
@@ -1309,7 +1329,7 @@ void connect_key_accels(dt_view_t *self)
 
   // enable shortcut to export with current export settings:
   closure = g_cclosure_new(G_CALLBACK(export_key_accel_callback),
-         (gpointer)self->data, NULL);
+                           (gpointer)self->data, NULL);
   dt_accel_connect_view(self, "export", closure);
 
   // Shortcut to skip images
@@ -1323,4 +1343,6 @@ void connect_key_accels(dt_view_t *self)
 
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
