@@ -1,7 +1,7 @@
 /*
     This file is part of darktable,
     copyright (c) 2009--2011 johannes hanika.
-    copyright (c) 2011 Henrik Andersson.
+    copyright (c) 2011--2012 Henrik Andersson.
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
@@ -28,6 +28,7 @@
 #include "common/colorlabels.h"
 #include "common/selection.h"
 #include "common/debug.h"
+#include "common/grouping.h"
 #include "common/history.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
@@ -50,14 +51,14 @@
 DT_MODULE(1)
 
 static gboolean star_key_accel_callback(GtkAccelGroup *accel_group,
-                                    GObject *acceleratable, guint keyval,
-                                    GdkModifierType modifier, gpointer data);
+                                        GObject *acceleratable, guint keyval,
+                                        GdkModifierType modifier, gpointer data);
 static gboolean go_up_key_accel_callback(GtkAccelGroup *accel_group,
-                                     GObject *acceleratable, guint keyval,
-                                     GdkModifierType modifier, gpointer data);
+    GObject *acceleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
 static gboolean go_down_key_accel_callback(GtkAccelGroup *accel_group,
-                                       GObject *acceleratable, guint keyval,
-                                       GdkModifierType modifier, gpointer data);
+    GObject *acceleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
 static gboolean
 go_pgup_key_accel_callback(GtkAccelGroup *accel_group, GObject *acceleratable,
                            guint keyval, GdkModifierType modifier,
@@ -67,7 +68,6 @@ go_pgdown_key_accel_callback(GtkAccelGroup *accel_group, GObject *acceleratable,
                              guint keyval, GdkModifierType modifier,
                              gpointer data);
 
-
 /**
  * this organises the whole library:
  * previously imported film rolls..
@@ -89,18 +89,25 @@ typedef struct dt_library_t
   int32_t collection_count;
 
   /* prepared and reusable statements */
-  struct {
+  struct
+  {
     /* main query statment, should be update on listener signal of collection */
     sqlite3_stmt *main_query;
     /* select imgid from selected_images */
     sqlite3_stmt *select_imgid_in_selection;
     /* delete from selected_images where imgid != ?1 */
     sqlite3_stmt *delete_except_arg;
+    /* check if the group of the image under the mouse has others, too, ?1: group_id, ?2: imgid */
+    sqlite3_stmt *is_grouped;
   } statements;
 
 }
 dt_library_t;
 
+// needed for drag&drop
+static GtkTargetEntry target_list[] = { { "text/uri-list", GTK_TARGET_OTHER_APP, 0 } };
+static guint n_targets = G_N_ELEMENTS (target_list);
+
 const char *name(dt_view_t *self)
 {
   return _("lighttable");
@@ -151,16 +158,16 @@ void init(dt_view_t *self)
   lib->full_preview_id=-1;
 
   /* setup collection listener and initialize main_query statement */
-  dt_control_signal_connect(darktable.signals, 
-			    DT_SIGNAL_COLLECTION_CHANGED, 
-			    G_CALLBACK(_view_lighttable_collection_listener_callback),
-			    (gpointer) self);
+  dt_control_signal_connect(darktable.signals,
+                            DT_SIGNAL_COLLECTION_CHANGED,
+                            G_CALLBACK(_view_lighttable_collection_listener_callback),
+                            (gpointer) self);
 
   _view_lighttable_collection_listener_callback(NULL,self);
 
   /* initialize reusable sql statements */
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from selected_images where imgid != ?1", -1, &lib->statements.delete_except_arg, NULL);
-  
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select id from images where group_id = ?1 and id != ?2", -1, &lib->statements.is_grouped, NULL); //TODO: only check in displayed images?
 }
 
 
@@ -205,7 +212,7 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
 
   /* get image over id */
   lib->image_over = DT_VIEW_DESERT;
-  int32_t mouse_over_id;
+  int32_t mouse_over_id, mouse_over_group = -1;
   DT_CTL_GET_GLOBAL(mouse_over_id, lib_image_mouse_over_id);
 
   /* fill background */
@@ -225,7 +232,7 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
   if(lib->center) lib->offset = 0;
   lib->center = 0;
   int offset = lib->offset;
-  
+
   /* if offset differs then flag as changed */
   if (offset != lib->first_visible_filemanager)
     offset_changed = TRUE;
@@ -252,7 +259,7 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
   int clicked1 = (oldpan == 0 && pan == 1 && lib->button == 1);
 
   /* get the count of current collection */
-  
+
   if(lib->collection_count == 0)
   {
     const float fs = 15.0f;
@@ -295,10 +302,10 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
   if(!lib->statements.main_query)
     return;
 
-  if(offset < 0) 
+  if(offset < 0)
     lib->offset = offset = 0;
 
-  while(offset >= lib->collection_count) 
+  while(offset >= lib->collection_count)
     lib->offset = (offset -= iir);
 
   /* update scroll borders */
@@ -307,22 +314,50 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
   /* let's reset and reuse the main_query statement */
   DT_DEBUG_SQLITE3_CLEAR_BINDINGS(lib->statements.main_query);
   DT_DEBUG_SQLITE3_RESET(lib->statements.main_query);
- 
+
   /* setup offset and row for the main query */
   DT_DEBUG_SQLITE3_BIND_INT(lib->statements.main_query, 1, offset);
   DT_DEBUG_SQLITE3_BIND_INT(lib->statements.main_query, 2, max_rows*iir);
- 
+
+  if(mouse_over_id != -1)
+  {
+    const dt_image_t *mouse_over_image = dt_image_cache_read_get(darktable.image_cache, mouse_over_id);
+    mouse_over_group = mouse_over_image->group_id;
+    dt_image_cache_read_release(darktable.image_cache, mouse_over_image);
+    DT_DEBUG_SQLITE3_CLEAR_BINDINGS(lib->statements.is_grouped);
+    DT_DEBUG_SQLITE3_RESET(lib->statements.is_grouped);
+    DT_DEBUG_SQLITE3_BIND_INT(lib->statements.is_grouped, 1, mouse_over_group);
+    DT_DEBUG_SQLITE3_BIND_INT(lib->statements.is_grouped, 2, mouse_over_id);
+    if(sqlite3_step(lib->statements.is_grouped) != SQLITE_ROW)
+      mouse_over_group = -1;
+  }
+
+  // prefetch the ids so that we can peek into the future to see if there are adjacent images in the same group.
+  int *query_ids = (int*)calloc(max_rows*max_cols, sizeof(int));
+  if(!query_ids) goto after_drawing;
+  for(int row = 0; row < max_rows; row++)
+  {
+    for(int col = 0; col < max_cols; col++)
+    {
+      if(sqlite3_step(lib->statements.main_query) == SQLITE_ROW)
+        query_ids[row*iir+col] = sqlite3_column_int(lib->statements.main_query, 0);
+      else goto end_query_cache;
+    }
+  }
+
+end_query_cache:
+
+  cairo_save(cr);
   for(int row = 0; row < max_rows; row++)
   {
     for(int col = 0; col < max_cols; col++)
     {
       //curidx = grid_to_index(row, col, iir, offset);
 
-      if(sqlite3_step(lib->statements.main_query) == SQLITE_ROW)
+      id = query_ids[row*iir+col];
+      if(id > 0)
       {
-        id = sqlite3_column_int(lib->statements.main_query, 0);
-
-        if (iir == 1 && row) 
+        if (iir == 1 && row)
           continue;
 
         /* set mouse over id if pointer is in current row / col */
@@ -332,7 +367,7 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
           DT_CTL_SET_GLOBAL(lib_image_mouse_over_id, mouse_over_id);
         }
 
-        /* handle mouse click on current row / col 
+        /* handle mouse click on current row / col
            this could easily and preferable be moved to button_pressed()
          */
         if (clicked1 && (pi == col && pj == row))
@@ -345,20 +380,145 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
             dt_selection_select_range(darktable.selection, id);
         }
 
-
         cairo_save(cr);
         // if(iir == 1) dt_image_prefetch(image, DT_IMAGE_MIPF);
-        dt_view_image_expose(&(lib->image_over), id, cr, wd, iir == 1 ? height : ht, iir, img_pointerx, img_pointery);
+        dt_view_image_expose(&(lib->image_over), id, cr, wd, iir == 1 ? height : ht, iir, img_pointerx, img_pointery, FALSE);
+
         cairo_restore(cr);
       }
-      else 
-        goto failure;
+      else
+        goto escape_image_loop;
 
       cairo_translate(cr, wd, 0.0f);
     }
     cairo_translate(cr, -max_cols*wd, ht);
   }
+escape_image_loop:
+  cairo_restore(cr);
 
+  // and now the group borders
+  cairo_save(cr);
+  for(int row = 0; row < max_rows; row++)
+  {
+    for(int col = 0; col < max_cols; col++)
+    {
+      id = query_ids[row*iir+col];
+      if(id > 0)
+      {
+        const dt_image_t *image = dt_image_cache_read_get(darktable.image_cache, id);
+        int group_id = -1;
+        if(image)
+          group_id = image->group_id;
+        dt_image_cache_read_release(darktable.image_cache, image);
+
+        if (iir == 1 && row)
+          continue;
+
+        cairo_save(cr);
+
+        gboolean paint_border = FALSE;
+        // regular highlight border
+        if(group_id != -1)
+        {
+          if(mouse_over_group == group_id && iir > 1 && ((!darktable.gui->grouping && dt_conf_get_bool("plugins/lighttable/draw_group_borders")) || group_id == darktable.gui->expanded_group_id))
+          {
+            cairo_set_source_rgb(cr, 1, 0.8, 0);
+            paint_border = TRUE;
+          }
+          // border of expanded group
+          else if(darktable.gui->grouping && group_id == darktable.gui->expanded_group_id && iir > 1)
+          {
+            cairo_set_source_rgb(cr, 0, 0, 1);
+            paint_border = TRUE;
+          }
+        }
+
+        if(paint_border)
+        {
+          int neighbour_group = -1;
+          // top border
+          if(row > 0)
+          {
+            int _id = query_ids[(row-1)*iir+col];
+            if(_id > 0)
+            {
+              const dt_image_t *_img = dt_image_cache_read_get(darktable.image_cache, _id);
+              neighbour_group = _img->group_id;
+              dt_image_cache_read_release(darktable.image_cache, _img);
+            }
+          }
+          if(neighbour_group != group_id)
+          {
+            cairo_move_to(cr, 0, 0);
+            cairo_line_to(cr, wd, 0);
+          }
+          // left border
+          neighbour_group = -1;
+          if(col > 0)
+          {
+            int _id = query_ids[row*iir+(col-1)];
+            if(_id > 0)
+            {
+              const dt_image_t *_img = dt_image_cache_read_get(darktable.image_cache, _id);
+              neighbour_group = _img->group_id;
+              dt_image_cache_read_release(darktable.image_cache, _img);
+            }
+          }
+          if(neighbour_group != group_id)
+          {
+            cairo_move_to(cr, 0, 0);
+            cairo_line_to(cr, 0, ht);
+          }
+          // bottom border
+          neighbour_group = -1;
+          if(row < max_rows-1)
+          {
+            int _id = query_ids[(row+1)*iir+col];
+            if(_id > 0)
+            {
+              const dt_image_t *_img = dt_image_cache_read_get(darktable.image_cache, _id);
+              neighbour_group = _img->group_id;
+              dt_image_cache_read_release(darktable.image_cache, _img);
+            }
+          }
+          if(neighbour_group != group_id)
+          {
+            cairo_move_to(cr, 0, ht);
+            cairo_line_to(cr, wd, ht);
+          }
+          // right border
+          neighbour_group = -1;
+          if(col < max_cols-1)
+          {
+            int _id = query_ids[row*iir+(col+1)];
+            if(_id > 0)
+            {
+              const dt_image_t *_img = dt_image_cache_read_get(darktable.image_cache, _id);
+              neighbour_group = _img->group_id;
+              dt_image_cache_read_release(darktable.image_cache, _img);
+            }
+          }
+          if(neighbour_group != group_id)
+          {
+            cairo_move_to(cr, wd, 0);
+            cairo_line_to(cr, wd, ht);
+          }
+          cairo_set_line_width(cr, 0.01*wd);
+          cairo_stroke(cr);
+        }
+
+        cairo_restore(cr);
+      }
+      else
+        goto escape_border_loop;
+
+      cairo_translate(cr, wd, 0.0f);
+    }
+    cairo_translate(cr, -max_cols*wd, ht);
+  }
+escape_border_loop:
+  cairo_restore(cr);
+after_drawing:
   /* check if offset was changed and we need to prefetch thumbs */
   if (offset_changed)
   {
@@ -368,7 +528,7 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
     /* clear and reset main query */
     DT_DEBUG_SQLITE3_CLEAR_BINDINGS(lib->statements.main_query);
     DT_DEBUG_SQLITE3_RESET(lib->statements.main_query);
-    
+
     /* setup offest and row for prefetch */
     DT_DEBUG_SQLITE3_BIND_INT(lib->statements.main_query, 1, offset + max_rows*iir);
     DT_DEBUG_SQLITE3_BIND_INT(lib->statements.main_query, 2, prefetchrows*iir);
@@ -379,23 +539,23 @@ expose_filemanager (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height,
 
     float imgwd = iir == 1 ? 0.97 : 0.8;
     dt_mipmap_size_t mip = dt_mipmap_cache_get_matching_size(
-							     darktable.mipmap_cache, 
-							     imgwd*wd, imgwd*(iir==1?height:ht));
+                             darktable.mipmap_cache,
+                             imgwd*wd, imgwd*(iir==1?height:ht));
     while(imgids_num > 0)
     {
       imgids_num --;
       dt_mipmap_buffer_t buf;
       dt_mipmap_cache_read_get(
-			       darktable.mipmap_cache,
-			       &buf,
-			       imgids[imgids_num],
-			       mip,
-			       DT_MIPMAP_PREFETCH);
+        darktable.mipmap_cache,
+        &buf,
+        imgids[imgids_num],
+        mip,
+        DT_MIPMAP_PREFETCH);
     }
   }
 
-
-failure:
+  if(query_ids)
+    free(query_ids);
   oldpan = pan;
   if(darktable.unmuted & DT_DEBUG_CACHE)
     dt_mipmap_cache_print(darktable.mipmap_cache);
@@ -408,7 +568,8 @@ failure:
       g_object_set(G_OBJECT(dt_ui_center(darktable.gui->ui)), "tooltip-text", tooltip, (char *)NULL);
       g_free(tooltip);
     }
-  } else if(darktable.gui->center_tooltip == 2) // not set in this round
+  }
+  else if(darktable.gui->center_tooltip == 2)   // not set in this round
   {
     darktable.gui->center_tooltip = 0;
     g_object_set(G_OBJECT(dt_ui_center(darktable.gui->ui)), "tooltip-text", "", (char *)NULL);
@@ -457,7 +618,7 @@ expose_zoomable (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, in
     zoom_y = lib->select_offset_y - /* (zoom == 1 ? 2. : 1.)*/pointery;
   }
 
-  if(!lib->statements.main_query) 
+  if(!lib->statements.main_query)
     return;
 
   if     (track == 0);
@@ -575,8 +736,8 @@ expose_zoomable (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, in
   int id, clicked1, last_seli = 1<<30, last_selj = 1<<30;
   clicked1 = (oldpan == 0 && pan == 1 && lib->button == 1);
 
-  dt_view_set_scrollbar(self, MAX(0, offset_i), DT_LIBRARY_MAX_ZOOM, zoom, DT_LIBRARY_MAX_ZOOM*offset_j, 
-			lib->collection_count, DT_LIBRARY_MAX_ZOOM*max_cols);
+  dt_view_set_scrollbar(self, MAX(0, offset_i), DT_LIBRARY_MAX_ZOOM, zoom, DT_LIBRARY_MAX_ZOOM*offset_j,
+                        lib->collection_count, DT_LIBRARY_MAX_ZOOM*max_cols);
 
   cairo_translate(cr, -offset_x*wd, -offset_y*ht);
   cairo_translate(cr, -MIN(offset_i*wd, 0.0), 0.0);
@@ -593,7 +754,7 @@ expose_zoomable (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, in
     /* clear and reset main query */
     DT_DEBUG_SQLITE3_CLEAR_BINDINGS(lib->statements.main_query);
     DT_DEBUG_SQLITE3_RESET(lib->statements.main_query);
-    
+
     DT_DEBUG_SQLITE3_BIND_INT(lib->statements.main_query, 1, offset);
     DT_DEBUG_SQLITE3_BIND_INT(lib->statements.main_query, 2, max_cols);
     for(int col = 0; col < max_cols; col++)
@@ -617,13 +778,13 @@ expose_zoomable (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, in
 
             /* clear and resest statement */
             DT_DEBUG_SQLITE3_CLEAR_BINDINGS(lib->statements.delete_except_arg);
-            DT_DEBUG_SQLITE3_RESET(lib->statements.delete_except_arg); 
+            DT_DEBUG_SQLITE3_RESET(lib->statements.delete_except_arg);
 
             /* reuse statment */
             DT_DEBUG_SQLITE3_BIND_INT(lib->statements.delete_except_arg, 1, id);
             sqlite3_step(lib->statements.delete_except_arg);
           }
-          // FIXME: whatever comes first assumtion is broken!
+          // FIXME: whatever comes first assumption is broken!
           // if((lib->modifiers & GDK_SHIFT_MASK) && (last_seli == (1<<30)) &&
           //    (image->id == lib->last_selected_id || image->id == mouse_over_id)) { last_seli = col; last_selj = row; }
           // if(last_seli < (1<<30) && ((lib->modifiers & GDK_SHIFT_MASK) && (col >= MIN(last_seli,seli) && row >= MIN(last_selj,selj) &&
@@ -644,7 +805,7 @@ expose_zoomable (dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, in
         }
         cairo_save(cr);
         // if(zoom == 1) dt_image_prefetch(image, DT_IMAGE_MIPF);
-        dt_view_image_expose(&(lib->image_over), id, cr, wd, zoom == 1 ? height : ht, zoom, img_pointerx, img_pointery);
+        dt_view_image_expose(&(lib->image_over), id, cr, wd, zoom == 1 ? height : ht, zoom, img_pointerx, img_pointery, FALSE);
         cairo_restore(cr);
       }
       else goto failure;
@@ -664,20 +825,99 @@ failure:
     dt_mipmap_cache_print(darktable.mipmap_cache);
 }
 
+/**
+ * Displays a full screen preview of the image currently under the mouse pointer.
+ */
+void expose_full_preview(dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t pointerx, int32_t pointery)
+{
+    dt_library_t *lib = (dt_library_t *)self->data;
+    int offset = 0;
+    if(lib->track >  2) offset++;
+    if(lib->track < -2) offset--;
+    lib->track = 0;
+
+    if (offset) {
+      /* If more than one image is selected, iterate over these. */
+      /* If only one image is selected, scroll through all known images. */
+
+      int sel_img_count = 0;
+      {
+        sqlite3_stmt *stmt;
+        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select COUNT(*) from selected_images", -1, &stmt, NULL);
+        if (sqlite3_step(stmt) == SQLITE_ROW)
+        {
+          sel_img_count = sqlite3_column_int(stmt, 0);
+        }
+        sqlite3_finalize(stmt);
+      }
+
+      const dt_image_t *img = dt_image_cache_read_get(darktable.image_cache, lib->full_preview_id);
+
+      /* Build outer select criteria */
+      gchar *filter_criteria = g_strdup_printf(
+            "inner join images on s1.id=images.id WHERE ((images.filename = \"%s\") and (images.id %s %d)) or (images.filename %s \"%s\") ORDER BY images.filename %s, images.id %s LIMIT 1",
+            img->filename,
+            (offset > 0) ? ">" : "<",
+            lib->full_preview_id,
+            (offset > 0) ? ">" : "<",
+            img->filename,
+            (offset > 0) ? "" : "DESC",
+            (offset > 0) ? "" : "DESC");
+
+      dt_image_cache_read_release(darktable.image_cache, img);
+
+      sqlite3_stmt *stmt;
+      gchar *stmt_string = NULL;
+      if (sel_img_count > 1)
+      {
+        stmt_string = g_strdup_printf(
+            "select images.id as id from (select imgid as id from selected_images) as s1 %s",
+            filter_criteria);
+
+        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), stmt_string, -1, &stmt, NULL);
+      }
+      else
+      {
+        /* We need to augment the current main query a bit to fetch the
+         * row we need. */
+        const char *main_query = sqlite3_sql(lib->statements.main_query);
+        stmt_string = g_strdup_printf(
+                "select images.id as id from (%s) as s1 %s",
+                main_query, filter_criteria);
+
+        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), stmt_string, -1, &stmt, NULL);
+        DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, 0);
+        DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, -1);
+
+      }
+      g_free(stmt_string);
+      g_free(filter_criteria);
+
+      if(sqlite3_step(stmt) == SQLITE_ROW) {
+        lib->full_preview_id = sqlite3_column_int(stmt, 0);
+      }
+
+      sqlite3_finalize(stmt);
+    }
+
+    lib->image_over = DT_VIEW_DESERT;
+    cairo_set_source_rgb (cr, .1, .1, .1);
+    cairo_paint(cr);
+    dt_view_image_expose(&(lib->image_over), lib->full_preview_id, cr, width, height, 1, pointerx, pointery, TRUE);
+}
+
 void expose(dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t pointerx, int32_t pointery)
 {
   const int i = dt_conf_get_int("plugins/lighttable/layout");
 
+  const double start = dt_get_wtime();
+
   // Let's show full preview if in that state...
   dt_library_t *lib = (dt_library_t *)self->data;
-  int32_t mouse_over_id;
-  DT_CTL_GET_GLOBAL(mouse_over_id, lib_image_mouse_over_id);
+
   if( lib->full_preview_id!=-1 )
   {
-    lib->image_over = DT_VIEW_DESERT;
-    cairo_set_source_rgb (cr, .1, .1, .1);
-    cairo_paint(cr);
-    dt_view_image_expose(&(lib->image_over),mouse_over_id, cr, width, height, 1, pointerx, pointery);
+    expose_full_preview(self, cr, width, height, pointerx, pointery);
   }
   else // we do pass on expose to manager or zoomable
   {
@@ -691,6 +931,9 @@ void expose(dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t
         break;
     }
   }
+  const double end = dt_get_wtime();
+  if (darktable.unmuted & DT_DEBUG_PERF)
+    dt_print(DT_DEBUG_LIGHTTABLE, "[lighttable] expose took %0.04f sec\n", end-start);
 }
 
 static gboolean
@@ -811,12 +1054,40 @@ static void _lighttable_mipamps_updated_signal_callback(gpointer instance, gpoin
   dt_control_queue_redraw_center();
 }
 
+static void
+drag_and_drop_received(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data,
+                       guint target_type, guint time, gpointer data)
+{
+  gboolean success = FALSE;
+
+  if((selection_data != NULL) && (selection_data->length >= 0))
+  {
+    gchar **uri_list = g_strsplit_set((gchar*)selection_data->data, "\r\n", 0);
+    if(uri_list)
+    {
+      gchar **image_to_load = uri_list;
+      while(*image_to_load)
+      {
+        dt_load_from_string(*image_to_load, FALSE); // TODO: do we want to open the image in darkroom mode? If yes -> set to TRUE.
+        image_to_load++;
+      }
+    }
+    g_strfreev(uri_list);
+    success = TRUE;
+  }
+  gtk_drag_finish(context, success, FALSE, time);
+}
+
 void enter(dt_view_t *self)
 {
+  // init drag&drop of files/folders
+  gtk_drag_dest_set(dt_ui_center(darktable.gui->ui), GTK_DEST_DEFAULT_ALL, target_list, n_targets, GDK_ACTION_COPY);
+  g_signal_connect(dt_ui_center(darktable.gui->ui), "drag-data-received", G_CALLBACK(drag_and_drop_received), self);
+
   /* connect to signals */
   dt_control_signal_connect(darktable.signals, DT_SIGNAL_DEVELOP_MIPMAP_UPDATED,
-			    G_CALLBACK(_lighttable_mipamps_updated_signal_callback), 
-          (gpointer)self);
+                            G_CALLBACK(_lighttable_mipamps_updated_signal_callback),
+                            (gpointer)self);
 
   // clear some state variables
   dt_library_t *lib = (dt_library_t *)self->data;
@@ -831,6 +1102,8 @@ void dt_lib_remove_child(GtkWidget *widget, gpointer data)
 
 void leave(dt_view_t *self)
 {
+  gtk_drag_dest_unset(dt_ui_center(darktable.gui->ui));
+
   /* disconnect from signals */
   dt_control_signal_disconnect(darktable.signals, G_CALLBACK(_lighttable_mipamps_updated_signal_callback), (gpointer)self);
 
@@ -957,6 +1230,35 @@ int button_pressed(dt_view_t *self, double x, double y, int which, int type, uin
         dt_image_cache_read_release(darktable.image_cache, image);
         break;
       }
+      case DT_VIEW_GROUP:
+      {
+        int32_t mouse_over_id;
+        DT_CTL_GET_GLOBAL(mouse_over_id, lib_image_mouse_over_id);
+        const dt_image_t *image = dt_image_cache_read_get(darktable.image_cache, mouse_over_id);
+        if(!image) return 0;
+        int group_id = image->group_id;
+        int id = image->id;
+        dt_image_cache_read_release(darktable.image_cache, image);
+        if(state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) // just add the whole group to the selection. TODO: make this also work for collapsed groups.
+        {
+          sqlite3_stmt *stmt;
+          DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert or ignore into selected_images select id from images where group_id = ?1", -1, &stmt, NULL);
+          DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, group_id);
+          sqlite3_step(stmt);
+          sqlite3_finalize(stmt);
+        }
+        else if(group_id == darktable.gui->expanded_group_id) // the group is already expanded, so ...
+        {
+          if(id == darktable.gui->expanded_group_id) // ... collapse it
+            darktable.gui->expanded_group_id = -1;
+          else                                       // ... make the image the new representative of the group
+            darktable.gui->expanded_group_id = dt_grouping_change_representative(id);
+        }
+        else // expand the group
+          darktable.gui->expanded_group_id = group_id;
+        dt_collection_update_query(darktable.collection);
+        break;
+      }
       default:
         return 0;
     }
@@ -973,7 +1275,7 @@ int key_released(dt_view_t *self, guint key, guint state)
     return 0;
 
   if(key == accels->lighttable_preview.accel_key
-     && state == accels->lighttable_preview.accel_mods && lib->full_preview_id !=-1)
+      && state == accels->lighttable_preview.accel_mods && lib->full_preview_id !=-1)
   {
 
     lib->full_preview_id = -1;
@@ -1002,7 +1304,7 @@ int key_pressed(dt_view_t *self, guint key, guint state)
   const int layout = dt_conf_get_int("plugins/lighttable/layout");
 
   if(key == accels->lighttable_preview.accel_key
-     && state == accels->lighttable_preview.accel_mods)
+      && state == accels->lighttable_preview.accel_mods)
   {
     int32_t mouse_over_id;
     DT_CTL_GET_GLOBAL(mouse_over_id, lib_image_mouse_over_id);
@@ -1024,12 +1326,11 @@ int key_pressed(dt_view_t *self, guint key, guint state)
 
       //dt_dev_invalidate(darktable.develop);
     }
-
-    return 0;
+    return 1;
   }
 
   if(key == accels->lighttable_left.accel_key
-     && state == accels->lighttable_left.accel_mods)
+      && state == accels->lighttable_left.accel_mods)
   {
     if(layout == 1 && zoom == 1) lib->track = -DT_LIBRARY_MAX_ZOOM;
     else lib->track = -1;
@@ -1037,7 +1338,7 @@ int key_pressed(dt_view_t *self, guint key, guint state)
   }
 
   if(key == accels->lighttable_right.accel_key
-     && state == accels->lighttable_right.accel_mods)
+      && state == accels->lighttable_right.accel_mods)
   {
     if(layout == 1 && zoom == 1) lib->track = DT_LIBRARY_MAX_ZOOM;
     else lib->track = 1;
@@ -1045,21 +1346,21 @@ int key_pressed(dt_view_t *self, guint key, guint state)
   }
 
   if(key == accels->lighttable_up.accel_key
-     && state == accels->lighttable_up.accel_mods)
+      && state == accels->lighttable_up.accel_mods)
   {
     lib->track = -DT_LIBRARY_MAX_ZOOM;
     return 1;
   }
 
   if(key == accels->lighttable_down.accel_key
-     && state == accels->lighttable_down.accel_mods)
+      && state == accels->lighttable_down.accel_mods)
   {
     lib->track = DT_LIBRARY_MAX_ZOOM;
     return 1;
   }
 
   if(key == accels->lighttable_center.accel_key
-     && state == accels->lighttable_center.accel_mods)
+      && state == accels->lighttable_center.accel_mods)
   {
     lib->center = 1;
     return 1;
@@ -1128,7 +1429,6 @@ void init_key_accels(dt_view_t *self)
 
   // Preview key
   dt_accel_register_view(self, NC_("accel", "preview"), GDK_z, 0);
-
 }
 
 void connect_key_accels(dt_view_t *self)
@@ -1137,50 +1437,50 @@ void connect_key_accels(dt_view_t *self)
 
   // Rating keys
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_DESERT, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_DESERT, NULL);
   dt_accel_connect_view(self, "rate desert", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_STAR_1, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_STAR_1, NULL);
   dt_accel_connect_view(self, "rate 1", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_STAR_2, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_STAR_2, NULL);
   dt_accel_connect_view(self, "rate 2", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_STAR_3, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_STAR_3, NULL);
   dt_accel_connect_view(self, "rate 3", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_STAR_4, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_STAR_4, NULL);
   dt_accel_connect_view(self, "rate 4", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_STAR_5, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_STAR_5, NULL);
   dt_accel_connect_view(self, "rate 5", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(star_key_accel_callback),
-      (gpointer)DT_VIEW_REJECT, NULL);
+              G_CALLBACK(star_key_accel_callback),
+              (gpointer)DT_VIEW_REJECT, NULL);
   dt_accel_connect_view(self, "rate reject", closure);
 
   // Navigation keys
   closure = g_cclosure_new(
-      G_CALLBACK(go_up_key_accel_callback),
-      (gpointer)self, NULL);
+              G_CALLBACK(go_up_key_accel_callback),
+              (gpointer)self, NULL);
   dt_accel_connect_view(self, "navigate up", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(go_down_key_accel_callback),
-      (gpointer)self, NULL);
+              G_CALLBACK(go_down_key_accel_callback),
+              (gpointer)self, NULL);
   dt_accel_connect_view(self, "navigate down", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(go_pgup_key_accel_callback),
-      (gpointer)self, NULL);
+              G_CALLBACK(go_pgup_key_accel_callback),
+              (gpointer)self, NULL);
   dt_accel_connect_view(self, "navigate page up", closure);
   closure = g_cclosure_new(
-      G_CALLBACK(go_pgdown_key_accel_callback),
-      (gpointer)self, NULL);
+              G_CALLBACK(go_pgdown_key_accel_callback),
+              (gpointer)self, NULL);
   dt_accel_connect_view(self, "navigate page down", closure);
 
   // Color keys
